<?php

namespace Controller;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Framework\DataSource\DataSet;
use Util\ElapsedTime;

class KDSManager extends \Zeedhi\Framework\Controller\Simple
{
	const ERROR_MESSAGE = 'E';
	const SUCCESS_MESSAGE = 'S';

	protected $environment;
	protected $KDSMessage;
	protected $util;
	protected $KDSManagerService;
	protected $entityManager;
	protected $production;
	protected $impressaoService;
	protected $instanceManager;
	protected $statisticsService;
	private $productionItemProcessed = array();
	private $itemsForInsertion = array();
	private $itemsToHandleComposition = array();
	private $itemsRelations = array();
	private $itemsToFindMainParent = array();
	private $itemsToUpdateToReleased = array();
	private $winnerLine = FALSE;
	private $getOrigins;
	private $showDateTime;
	private $order;
	private $oldOrder;

	public function __construct(
		\Util\KDSMessage $KDSMessage,
		\Util\Util $util,
		\Service\KDSManager $KDSManagerService,
		\Doctrine\ORM\EntityManager $entityManager,
		\Controller\Production $production,
		\Service\Impressao $impressaoService,
		\Service\Statistics $statisticsService,
        \Lib\Environment $environment,
		\Service\GetOrigins $getOrigins,
		\Zeedhi\Framework\DependencyInjection\InstanceManager $instanceManager,
        $showDateTime
	) {

		$this->KDSMessage        = $KDSMessage;
		$this->util              = $util;
		$this->KDSManagerService = $KDSManagerService;
		$this->entityManager 	 = $entityManager;
		$this->production        = $production;
		$this->impressaoService  = $impressaoService;
		$this->statisticsService = $statisticsService;
        $this->environment       = $environment;
        $this->showDateTime      = $showDateTime;
		$this->getOrigins		 = $getOrigins;
		$this->instanceManager = $instanceManager;
	}

	public function handleProductsForDisplay($productsToDisplay)
	{	
		// ordena array por CHAVEPEDIDO
		$orderedArray = $this->orderArrayByOrderKey($productsToDisplay);
		// para cada CHAVEPEDIDO, passar os itens desse pedido para outra funcao
		// separa os pedidos, chama a função uma vez por pedido
		$currentOrder = null;
		$lastOrder = null;
		$arrayForTimers = array();
		$idx = 0;
		while ($idx < sizeof($orderedArray)) {
			$item = $orderedArray[$idx];
			$currentOrder = $item['CHAVEPEDIDO'];
			if (($currentOrder != $lastOrder) && ($lastOrder != null)) {
				// trocou de pedido, define os timers do pedido atual

				$this->handleOrderTimers($arrayForTimers);
				$arrayForTimers = array();
			}
			array_push($arrayForTimers, $item);
			$lastOrder = $currentOrder;

			// trata último item
			if ($idx == sizeof($orderedArray) - 1) {
				$this->handleOrderTimers($arrayForTimers);
			}

			$idx++;
		}
	}

	private function handleOrderTimers($arrayForInsertion)
	{
		// remove tempo de produção dos items da montagem e expedição
		$arrayForInsertion = $this->removeProductionTimeFromProducts($arrayForInsertion);
		// ordena array por tempo de produção
		$arrayForInsertion = $this->orderArrayByProductionTime($arrayForInsertion);
		// define os tempos de exibição de cada produto
		$arrayForInsertion = $this->defineOrderTimers($arrayForInsertion);
		// Ordena por posição da mesa
		// $arrayForInsertion = $this->orderArrayByPositionTable($arrayForInsertion);
		// insere na ITPEDIDOFOS
		$this->handleProductsInsertion($arrayForInsertion);
	}

	private function removeProductionTimeFromProducts($arrayForInsertion)
	{
		foreach ($arrayForInsertion as &$currentItem) {
			if ($currentItem['IDTIPOSETOR'] != 'P') {
				$currentItem['NRTEMPOPROD'] = '0';
				$currentItem['NRTEMPOEXIB'] = '0';
			}
		}
		return $arrayForInsertion;
	}

	private function handleProductsInsertion($arrayForInsertion)
	{
		foreach ($arrayForInsertion as $product) {
			$this->KDSManagerService->insertOrderItem(
				$product['CDFILIAL'],
				$product['NRPEDIDOFOS'],
				$product['NRITPEDIDOFOS'],
				$product['CDPRODUTO'],
				$product['QTPRODPEFOS'],
				$product['CDCAIXA'],
				$product['NRSEQVENDA'],
				$product['NRSEQUITVEND'],
				$product['NRVENDAREST'],
				$product['NRCOMANDA'],
				$product['NRPRODCOMVEN'],
                isset($product['TXPRODPED']) ? $product['TXPRODPED'] : null,
				$product['IDSITITPEDFOS'],
				$product['CDSETOR'],
				$product['CDGRPOCOR'],
				$product['CDOCORR'],
				$product['DTHREXIBICAOPROD'],
				$product['DSCHAVEPEDPAI'],
				$product['CDPRODUTOPAI'],
				$product['CDPRODPROMOKDS'],
				$product['NRSEQPRODCOMKDS'],
				$product['NRTEMPOPROD'],
				$product['NRTEMPOEXIB'],
				$product['NRATRAPROD'],
				$product['NRLUGARMESAIT'],
				isset($product['IDLIBERADO']) ? $product['IDLIBERADO'] : 'N',
				$product['NRMAIORTEMPO'],
				$product['IDORIGEMVENDA'],
				$product['NRSEQITVENDAEST'],
				$product['NRSEQCOMB']
			);
			// insere o relacionamento com a ITCOMANDAVEN (IDPEDIDOFOSREL)
			foreach ($product['RELITCOMANDAVEN'] as $currentRel) {
				$NRSEQITPEDREL = $this->generateNewNRSEQITPEDREL($product['CDFILIAL'], $product['NRPEDIDOFOS'], $product['NRITPEDIDOFOS']);
				$this->KDSManagerService->insertOrderRelation(
					$product['CDFILIAL'],
					$product['NRPEDIDOFOS'],
					$product['NRITPEDIDOFOS'],
					$NRSEQITPEDREL,
					$currentRel['NRVENDAREST'],
					$currentRel['NRCOMANDA'],
					$currentRel['NRPRODCOMVEN'],
					$currentRel['QTPRODPEFOS'],
					$currentRel['IDSITITPEDFOS']
				);
			}
		}
	}

	private function defineOrderTimers($products, $paymentDate = null) {
		// caso seja informado a data utiliza a variavel $date ao invés do DTPEDIDO (utilizado para os itens que aguardam pagamento)
		$arrayForInsertion = array();
		$delayedItems = array();
		$biggestTime = 0;
		$interval = new \DateInterval('PT0S'); // cria intevalo de 0 segundos
		$delay = 'first';
		foreach ($products as $product) {
			if (empty($product['NRATRAPROD'])) {
				//coloca que já está liberado.
				$product['IDLIBERADO'] = 'S';

				if ($delay === 'first') {
					$delay = 0;
					$biggestTime = $product['NRTEMPOPROD'];
				} else {
					$delay = $biggestTime - $product['NRTEMPOPROD'];
					$interval = new \DateInterval('PT' . $delay . 'S'); // cria intevalo de $interval segundos
				}
				if ($product['NRTEMPOPROD'] == 0) {
					$product['NRTEMPOPROD'] = $biggestTime;
					$interval = new \DateInterval('PT0S');
				}
				if ($paymentDate) {
					$DTHREXIBICAOPROD = new \Datetime($paymentDate, new \DateTimeZone(date_default_timezone_get()));
				} else {
					$DTHREXIBICAOPROD = new \Datetime($product['DTPEDIDOFOS'], new \DateTimeZone(date_default_timezone_get()));
				}
				$DTHREXIBICAOPROD = $DTHREXIBICAOPROD->add($interval);
				$DTHREXIBICAOPROD = $DTHREXIBICAOPROD->format('d/m/Y H:i:s');
				$product['DTHREXIBICAOPROD'] = $DTHREXIBICAOPROD;
				$product['NRMAIORTEMPO'] = null;

				// coloca no formato JSON pra mandar pro frontend
				if (!empty($product['TXPRODPED'])) {
					$product['TXPRODPED'] = json_encode($product['TXPRODPED']);
				}

				// expedição e montagem a hora de exibição é sempre a hora do pedido
				if ($product['IDTIPOSETOR'] != 'P') {
					if ($paymentDate) {
						$product['DTHREXIBICAOPROD'] = new \Datetime($paymentDate, new \DateTimeZone(date_default_timezone_get()));
					} else {
						$product['DTHREXIBICAOPROD'] = new \Datetime($product['DTPEDIDOFOS'], new \DateTimeZone(date_default_timezone_get()));
					}
				}

				array_push($arrayForInsertion, $product);
			} else {
				array_push($delayedItems, $product);
			}
		}

		// ordena array por tempo de produção
		$delayedItems = $this->orderArrayByProductionTime($delayedItems);
		// ordena por posição da mesa
		//$arrayForInsertion = $this->orderArrayByPositionTable($arrayForInsertion);
		
		// variável auxiliar usada para armazenar o tempo do item mais demorado entre os itens com atraso
		$biggestTime = 0;
		$delay = 'first';
		foreach ($delayedItems as $currentItem) {

			if ($delay === 'first') {
				$delay = 0;
				$biggestTime = $currentItem['NRTEMPOPROD'];
				$interval = new \DateInterval('PT0S');
			} else {
				$delay = $biggestTime - $currentItem['NRTEMPOPROD'];
				$interval = new \DateInterval('PT' . $delay . 'S'); // cria intevalo de $interval segundos
			}
			if ($currentItem['NRTEMPOPROD'] == 0) {
				$currentItem['NRTEMPOPROD'] = $biggestTime;
				$interval = new \DateInterval('PT0S');
			}

			$currentItem['NRMAIORTEMPO'] = $delay;
			//Coloca que está em espera.
			$currentItem['IDLIBERADO'] = 'N';

			// calcula tempo de atraso do item
			// será: maior tempo + tempo de atraso - tempo de produção
			// para que o item fique pronto X minutos de atraso depois do maior item do pedido
			// exemplo: pedi dois itens, coloquei atraso de 5 minutos no segundo,
			// produto 1 fica pronto 18:00, produto 2 fica pronto 18:05
			//$delay = ($currentItem['NRATRAPROD'] * 60 - $biggestTime) + ($biggestTime - $currentItem['NRTEMPOPROD']);
			// trata observação do atraso
			$obsAtraso = 'SEGURA DE ' . (string)$currentItem['NRATRAPROD'] . ' MINUTO';
			if ($currentItem['NRATRAPROD'] > 1) {
				$obsAtraso = $obsAtraso . 'S';
			}
			if (empty($currentItem['TXPRODPED'])) {
				$currentItem['TXPRODPED'] = array();
			}
			array_push($currentItem['TXPRODPED'], array(
				'OBSERVACAO' => $obsAtraso,
				'NRCORSINAL' => 'FFFFFF',
				'TIPO' => 'ATRASO'
			));
			// coloca no formato JSON pra mandar pro frontend
			$currentItem['TXPRODPED'] = json_encode($currentItem['TXPRODPED']);
			if ($paymentDate) {
				$DTHREXIBICAOPROD = new \Datetime($paymentDate, new \DateTimeZone(date_default_timezone_get()));
			} else {
				$DTHREXIBICAOPROD = new \Datetime($currentItem['DTPEDIDOFOS'], new \DateTimeZone(date_default_timezone_get()));
			}
			$DTHREXIBICAOPROD = $DTHREXIBICAOPROD->add($interval);
			$DTHREXIBICAOPROD = $DTHREXIBICAOPROD->add(new \DateInterval('PT' . $currentItem['NRATRAPROD'] . 'M'));
			$DTHREXIBICAOPROD = $DTHREXIBICAOPROD->format('d/m/Y H:i:s');

			$currentItem['DTHREXIBICAOPROD'] = $DTHREXIBICAOPROD;
			// if (($biggestTime != $currentItem['NRTEMPOPROD']) && ($currentItem['IDTIPOSETOR'] == 'P')) {
			// 	$currentItem['NRMAIORTEMPO'] = $biggestTime - $currentItem['NRTEMPOPROD'];
			// } else {
			// 	$currentItem['NRMAIORTEMPO'] = null;
			// }

			// expedição e montagem a hora de exibição é sempre a hora do pedido
			$currentItem['NRATRAPROD'] = $currentItem['NRATRAPROD'] * 60;

			array_push($arrayForInsertion, $currentItem);
		}

		return $arrayForInsertion;
	}

	public function redefineOrderTimers ($products) {
		// redefine o DTHREXIBICAOPROD para os itens que estavam aguardando pagamento
		$date = date('Y-m-d H:i:s');
		$products = $this->removeProductionTimeFromProducts($products);
		$products = $this->orderArrayByProductionTime($products);
		$products = $this->defineOrderTimers($products, $date);
		return $products;
	}
	private function compareProductionTime($a, $b)
	{
		return strnatcmp((string)$b['NRTEMPOPROD'], (string)$a['NRTEMPOPROD']);
	}

	private function orderArrayByProductionTime($order)
	{
		// sort alphabetically by name
		usort($order, array($this, 'compareProductionTime'));
		return $order;
	}

	/*private function orderArrayByPositionTable($order) {
		Ordena os itens por posição da mesa
		$position = array_column($order, 'NRLUGARMESAIT');
			array_multisort($order, SORT_ASC, SORT_NUMERIC, $position);
			return $order;
	}*/

	private function compareOrderKey($a, $b)
	{
		return strnatcmp($a['CHAVEPEDIDO'], $b['CHAVEPEDIDO']);
	}

	private function orderArrayByOrderKey($arrayToOrder)
	{
		// sort alphabetically by name
		usort($arrayToOrder, array($this, 'compareOrderKey'));
		return $arrayToOrder;
	}

	private function validateProduct($CDPRODUTO)
	{
		$product = $this->KDSManagerService->getProductByCode($CDPRODUTO);
		if (empty($product)) {
			$retorno = false;
			$this->util->logException('Produto' . $CDPRODUTO . ' não encontrado na tabela de produtos.');
		} else {
			$retorno = true;
		}
		return $retorno;
	}

	public function pedidoFosGenerator($newOrders)
	{
		$this->itemsForInsertion = array();
		$this->itemsToHandleComposition = array();
		$this->itemsRelations = array();
		$this->productionItemProcessed = array();

		if (count($newOrders) > 0) {

			foreach ($newOrders as $item) {
				// pega o item atual, verifica se é promoção inteligente
				if ($item['VENDA'] == 'VENDA') {
					// venda balcão
					$productsPromotion = $this->KDSManagerService->getProductPromoVenda($item['CDFILIAL'], $item['CDCAIXA'], $item['NRSEQVENDA'], $item['NRSEQUITVEND']);
				} 
				else if ($item['VENDA'] == 'RODIZIO') {
					$productsPromotion = $this->KDSManagerService->getProductPromoRodizio($item['CDFILIAL'], $item['NRCOMANDA'], $item['NRPRODCOMVEN'], $item['NRVENDAREST'], $item['CDPRODUTOROD']);
				}				
				else {
					// venda mesa, comanda e delivery
					$productsPromotion = $this->KDSManagerService->getProductPromoComanda($item['CDFILIAL'], $item['NRCOMANDA'], $item['NRPRODCOMVEN'], $item['NRVENDAREST']);
				}

				

				if (!empty($productsPromotion)) {
					for ($i = 0; $i < sizeof($productsPromotion); $i++) {
                        if ($productsPromotion[$i]['CDPRODUTO'] == $item['CDPRODUTO']) {
                            // copiar o item $productsPromotion[$i] para uma variável
                            // remover o item dessa posição de $productsPromotion[$i]
                            // interromper o for
                            $father = $productsPromotion[$i];
                            $productsPromotion[$i] = $productsPromotion[0];
                            $productsPromotion[0] = $father;
                        }
                    }

					foreach ($productsPromotion as $currentProduct) {
						$newItem = $item;
						
						$newItem['OBSDIGITADA'] = $currentProduct['OBSDIGITADA'];
						if ($newItem['CDPRODUTO'] == $currentProduct['CDPRODUTO']) {
                            if(stripos(trim($newItem['OBSDIGITADA']), trim($item['OBSDIGITADA'])) != false){
								$newItem['OBSDIGITADA'] .= $item['OBSDIGITADA'];
							}
                        }

						// parâmetros promoção inteligente
						$newItem['CDPRODPROMOKDS'] = $item['CDPRODUTO'];

						$newItem['NRSEQPRODCOMKDS'] = $item['NRSEQPRODCOM'];

						$newItem['NRATRAPROD'] = $item['NRATRAPROD'];

						$newItem['CDPRODUTO'] = $currentProduct['CDPRODUTO'];

						if ($newItem['VENDA'] == 'RODIZIO' && !empty($item['QTDEPROD'])) {
							$newItem['QTDEPROD'] = $item['QTDEPROD'];
						} else {
							$newItem['QTDEPROD'] = $currentProduct['QUANTIDADE'];
						}

						$newItem['CHAVEITEM'] = $currentProduct['CHAVEITEM'];
						//CHAVEITEM será usada nas querys de buscar observação.
						//seria tolice checar qual chave esta sendo usada toda vez, pois tem 4 possibilidades e teria que passar todas no item.						
						if(isset($currentProduct['TXPRODCOMVENEST'])){
							$newItem['TXPRODCOMVENEST'] = $currentProduct['TXPRODCOMVENEST'];
						}

						$newItem['NRSEQITVENDAEST'] = $currentProduct['NRSEQITVENDAEST'];
						$newItem['NRSEQCOMB'] = $currentProduct['NRSEQCOMB'];

						$this->handleSector($newItem);
					}
				} else {
					// itens que não são promoção inteligente
					// implemente - caso tenha outro item no array newOrders com mesmo NRSREQPRODCOM, o NRSEQPRODCOMKDS recebe NRSREQPRODCOM, caso contrario, recebe nulo
					$item['CDPRODPROMOKDS'] = $item['CDPRODPROMOCAO'];
					$irmaos = array_filter($newOrders, function ($item2) use ($item) {
						return $item2['NRSEQPRODCOM'] == $item['NRSEQPRODCOM'];
					});
					// Alterado o $irmaos para > 0 devido a não aparecer o item filho na promoção quando há apenas 1 item 
					if (sizeof($irmaos) > 0) {
						$item['NRSEQPRODCOMKDS'] = $item['NRSEQPRODCOM'];
					} else {
						$item['NRSEQPRODCOMKDS'] = null;
					}
					//CHAVEITEM será usada nas querys de buscar observação.
					//seria tolice checar qual chave esta sendo usada toda vez, pois tem 4 possibilidades e teria que passar todas no item.
					$this->handleSector($item);
				}
			}

			// inserindo os itens da composição do produto de venda no array de itens
			$this->handleComposition($this->itemsToHandleComposition);

			// insere no banco os itens da ITPEDIDOFOSPAI
			$this->insertProductsRelations($this->itemsRelations);

			// verifica se é um item do tipo entrada e se possui prato principal
			$this->itemsForInsertion = $this->buildKdsProdsEspera($this->itemsForInsertion);
			
			//imprime picklist nos setores parametrizados para imprimir direto
			$this->handlePicklist($this->itemsForInsertion, true);
			$this->winnerLine = NULL;
			return $this->itemsForInsertion;
		}
	}

	private function buildKdsProdsEspera($itemsForInsertion) {
		// verifica quais são os itens de entrada
		// e removem os demais itens con exeção dos itens marcarcados como exclusão para gerar um novo pedido posterior
		$kdsProdsEspera = array();
		$itensEmEspera = $this->getProdsEspera($itemsForInsertion);
		if (!empty($itensEmEspera)) {
			foreach ($itensEmEspera as $key => $ite) {
				$tpPedido = (isset($ite['NRSEQVENDA']) && $ite['NRSEQVENDA']) ? 'VEE' : 'COE';
				$itemEspera = array (
					'CDFILIAL' 		  => $ite['CDFILIAL'],
					'NRPEDIDOFOS' 	  => $ite['NRPEDIDOFOS'],
					'NRVENDAREST' 	  => $ite['NRVENDAREST'],
					'NRCOMANDA'   	  => $ite['NRCOMANDA'],
					'NRPRODCOMVEN'	  => $ite['NRPRODCOMVEN'],
					'NRSEQVENDA'  	  => $ite['NRSEQVENDA'],
					'NRSEQUITVEND'	  => $ite['NRSEQUITVEND'],
					'QTPRODPEFOS' 	  => $ite['QTPRODPEFOS'],
					'STATUS'		  => 'E',
					'TP_PEDIDO'		  => $tpPedido,
				);
				array_push($kdsProdsEspera, $itemEspera);
				unset($itemsForInsertion[$key]);
			}
		}
		if (!empty($kdsProdsEspera)) {
			$kdsProdsEspera = array_unique($kdsProdsEspera, SORT_REGULAR);
			foreach ($kdsProdsEspera as $itemEs) {
				// insere a relacao com a ITCOMANDAVEN para não dar problema com a transferencia de mesa
				if ($itemEs['TP_PEDIDO'] === 'COE') {
					$NRSEQITPEDREL = $this->generateNewNRSEQITPEDREL($itemEs['CDFILIAL'], $itemEs['NRPEDIDOFOS'], '0000000000');
					$this->KDSManagerService->insertOrderRelation(
						$itemEs['CDFILIAL'],
						$itemEs['NRPEDIDOFOS'],
						'0000000000',
						$NRSEQITPEDREL,
						$itemEs['NRVENDAREST'],
						$itemEs['NRCOMANDA'],
						$itemEs['NRPRODCOMVEN'],
						$itemEs['QTPRODPEFOS'],
						'E'
					);
				}
				$this->KDSManagerService->insertPedidoKdsTempEspera($itemEs);
			}
		}
		return $itemsForInsertion;
	}

	private function getProdsEspera ($itemsForInsertion) {
		$prodsEmEspera = array();
		$modalitysToExclude = array('Drive', 'Viagem');
		$CDFILIAL = $itemsForInsertion[0]['CDFILIAL'];
		$prodsEntrada = $this->getProdsEntrada($CDFILIAL);
		$prodsExcecao = $this->getProdsExcecao($CDFILIAL);

		$pedidosComEntrada = array_map(function($item) use ($prodsEntrada, $modalitysToExclude){
			$keyProd = $item['CDFILIAL'] . $item['CDLOJA'] . $item['CDPRODUTO'];
			$keyPromo = $item['CDFILIAL'] . $item['CDLOJA'] . $item['CDPRODPROMOKDS'];
			$modality = $this->getOrigins->manageOrigin($item['IDORIGEMVENDA'], "system", "modality");
			if ((in_array($keyProd, $prodsEntrada) || in_array($keyPromo, $prodsEntrada)) && !in_array($modality, $modalitysToExclude)) {
				return $item['NRPEDIDOFOS'];
			}	
		}, $itemsForInsertion);
		$pedidosComEntrada = array_filter($pedidosComEntrada);
		$pedidosComEntrada = array_unique($pedidosComEntrada, SORT_REGULAR);
		if (!empty($pedidosComEntrada)) {
			$prodsEmEspera = array_filter($itemsForInsertion, function ($orderItem) use ($prodsEntrada, $pedidosComEntrada, $prodsExcecao) {
				$keyProd = $orderItem['CDFILIAL'] . $orderItem['CDLOJA'] . $orderItem['CDPRODUTO'];
				$keyPromo = $orderItem['CDFILIAL'] . $orderItem['CDLOJA'] . $orderItem['CDPRODPROMOKDS'];
				return ((!in_array($keyProd, $prodsEntrada)) && 
						(!in_array($keyPromo, $prodsEntrada)) &&
						(!in_array($keyProd, $prodsExcecao)) &&
						(!in_array($keyPromo, $prodsExcecao)) &&
						(in_array($orderItem['NRPEDIDOFOS'], $pedidosComEntrada))
				);
			});
		}
		return $prodsEmEspera;
	}

	private function getProdsEntrada ($CDFILIAL) {
		$prodsEntrada = $this->environment->getSessionValue('prodsEntrada');
		if (!$prodsEntrada) {
			$prodsEntrada = $this->KDSManagerService->getProdsEntrada($CDFILIAL);
			$prodsEntrada = array_map(fn($prod) => $prod['CHAVE'], $prodsEntrada);
			$this->environment->setSessionValue('prodsEntrada', $prodsEntrada);
		}
		return $prodsEntrada;
	}

	private function getProdsExcecao ($CDFILIAL) {
		$prodsExcecao = $this->environment->getSessionValue('prodsExcecao');
		if (!$prodsExcecao) {
			$prodsExcecao = $this->KDSManagerService->getProdsExcecao($CDFILIAL);
			$prodsExcecao = array_map(fn($prod) => $prod['CHAVE'], $prodsExcecao);
			$this->environment->setSessionValue('prodsExcecao', $prodsExcecao);
		}
		return $prodsExcecao;
	}

	private function handlePicklist($items, $verificaBotao = false)
	{
		$itensPicklistSetor = array();
		$IDIMPPRODAPETIC = '';
		/*
			Dois campos que merecer serem destacados:
				- IDIMPRPRODPICK 
					Se SIM: vai ser feita a impressão do produto na etiqueta
					Se NÃO: o produto não será impresso na etiqueta
				- IDIMPPRODAPETIC
					Se SIM: Será impresso apenas na ETIQUETA
					Se NÃO: Será impresso na PICKLIST e na ETIQUETA
		*/
		foreach ($items as $key => $item) {
			$habilitaImpressaoBotao = false;
			if ($verificaBotao) {
				$habilitaImpressaoBotao = $this->KDSManagerService->verificaPicklistPorBotao($item['CDSETOR']);
			}
			if (!$habilitaImpressaoBotao){
				try {
					$imprimePicklistSetor = $this->verifySetorPicklist($item['CDSETOR']);
					$imprimePicklistProduto = $this->verificaProdutoPicklist($item['CDPRODUTO'], $item['CDSETOR']);
					
				} catch (\Exception $th) {
					
					$this->util->logException($th->getMessage());
	
					if (!isset($imprimePicklistSetor)){
						$imprimePicklistSetor = false;
					}
	
					if (!isset($imprimePicklistProduto)){
						$imprimePicklistProduto = false;
					}
				}

				if($imprimePicklistProduto){
					$item['IDIMPRPRODPICK'] = $imprimePicklistProduto['IDIMPRPRODPICK'];	
					if($IDIMPPRODAPETIC != 'N' || $IDIMPPRODAPETIC == ''){
						$IDIMPPRODAPETIC = $imprimePicklistProduto['IDIMPPRODAPETIC'];
					}
				}

				if ($imprimePicklistSetor) {
					if (array_key_exists($item['CDSETOR'], $itensPicklistSetor)) {
						array_push($itensPicklistSetor[$item['CDSETOR']], $item);
					} else {
						$itensPicklistSetor[$item['CDSETOR']] = array();
						array_push($itensPicklistSetor[$item['CDSETOR']], $item);
					}
				}
			}
		}

        if (count($itensPicklistSetor) > 0){
            foreach ($itensPicklistSetor as $cdsetor => $itensPicklist) {
                $result = $this->impressaoService->imprimePicklist($itensPicklist, $IDIMPPRODAPETIC);
				if(isset($result['error'])){
					$this->util->logException($result['message']);
				}
            }
        }
	}

    /**
    * Verifica se determinado setor está parametrizado para imprimir picklist.
    */  
	private function verifySetorPicklist($cdsetor)
	{
		$params = array(
			':CDSETOR' => $cdsetor
		);

		$setorPicklist = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::GET_PICKLIST_SECTOR, $params);

		if ($setorPicklist['IDIMPRPICKLIST'] == 'S') {
			return true;
		} else {
			return false;
		}
	}

    /**
    * Verifica se determinado produto está parametrizado para imprimir picklist, dentro de determinado setor.
    * Esta verificação por produto permite que mesmo um setor estando parametrizado para imprimir picklist, 
    * seja possível para o cliente excluir produtos específicos daquele setor que ele não deseja que sejam impressos.
    */
    private function verificaProdutoPicklist($cdproduto, $cdsetor)
    {
        $params = array(
            ':CDPRODUTO' => $cdproduto,
            ':CDSETOR' => $cdsetor
        );

        $produtoImprimePicklist = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::GET_PICKLIST_PRODUTO, $params);

        return $produtoImprimePicklist;
    }

	function printPicklist(Request\Row $request, Response $response) {
		try {
			$row = $request->getRow();
			$params = array();
			$imprimiu = false; 
			
			foreach($row as $currentItem) {
				array_push($params, $currentItem);
			}
			
			$habilitaImpressaoBotao = $this->KDSManagerService->verificaPicklistPorBotao($params[0]['CDSETOR']);
			if($habilitaImpressaoBotao) {
				$this->handlePicklist($params);
				$imprimiu = true;
			} 

			$response->addDataSet(new DataSet('printPicklist', array('success' => true, 'printPicklist' => $params, 'printed' => $imprimiu)));
		} catch (\Exception $e) {
			$response->addDataSet(new DataSet('printPicklist', array('error' => true, 'message' => $e->getMessage())));
		}
	}

	private function insertProductsRelations($itemsRelations)
	{
		foreach ($itemsRelations as $currentRelation) {
			$this->KDSManagerService->insertRelation(
				$currentRelation['CDFILIAL'],
				$currentRelation['NRPEDIDOFOS'],
				$currentRelation['NRITPEDIDOFOS'],
				$currentRelation['NRSEQITPEDPAI'],
				$currentRelation['CDFILIALPAI'],
				$currentRelation['NRPEDIDOFOSPAI'],
				$currentRelation['NRITPEDIDOFOSPAI']
			);
		}
	}

	private function distributeItems($CDFILIAL, $sectorItems, $winnerLine = null){
		// REGRA DE BALANCEAMENTO DOS PEDIDOS
		// Setores relacionados
		if ($winnerLine == NULL) {
			$winnerLine = $this->winnerLine;
		}
		$correlationatedSectors = $this->KDSManagerService->getCorrelationatedSectors($CDFILIAL);

		$winnerSector = null;
		$winnerSectors = array();

		if(sizeOf($sectorItems) > 1 && sizeOf($correlationatedSectors) > 0){
			// Verifica se algum setor possui correlação
			// Ou pelo menos que esteja presente em uma linha
			// Estou ciente que tem outro foreach parecido embaixo, mas vai
			// haver um desvio de fluxo tratando somente o setor necessário em uma relação
			foreach($sectorItems as $idxSetor => &$currentSector){
				//Procura uma relação para o setor atual
				$sectorRelation = array_values(
					array_filter($correlationatedSectors, function($sectorCorrelation) use($currentSector){
						return $sectorCorrelation['CDSETOR'] == $currentSector['CDSETOR'];
					})
				);
				if(!empty($sectorRelation)){
					if(!empty($sectorRelation[0]['CDSETORCORRESP'])){

						//Verifica se o setor correspondente está entre os setores trazidos
						$setorCorr = array_values(
							array_filter($correlationatedSectors, function($corrSector) use($sectorRelation){
								return $corrSector['CDSETOR'] == $sectorRelation[0]['CDSETORCORRESP'];
							})
						);
						
						$sectorRelation = $sectorRelation[0];
						if(isset($setorCorr[0])){
							$setorCorr = $setorCorr[0];
	
	
							$totalQuota = $sectorRelation['TOTALQTPRODPEFOS'] + $setorCorr['TOTALQTPRODPEFOS'];
	
							$sectorRelation['QUOTA'] = $sectorRelation['TOTALQTPRODPEFOS'] > 0 ? $sectorRelation['NRPERCDEMANDA'] - ($sectorRelation['TOTALQTPRODPEFOS'] * 100 / $totalQuota) : $sectorRelation['NRPERCDEMANDA'];
							$setorCorr['QUOTA'] = $setorCorr['TOTALQTPRODPEFOS'] > 0 ?  $setorCorr['NRPERCDEMANDA'] - ($setorCorr['TOTALQTPRODPEFOS'] * 100 / $totalQuota) : $setorCorr['NRPERCDEMANDA'];
	
							if($winnerLine != null){
								$winnerSector = (isset($setorCorr['CDLINHAPROD']) && $setorCorr['CDLINHAPROD'] == $winnerLine) ? $setorCorr['CDSETOR'] : $sectorRelation['CDSETOR'];
							} else if($setorCorr['QUOTA'] > $sectorRelation['QUOTA']){
								$winnerSector = $setorCorr['CDSETOR'];
							} else if( $setorCorr['QUOTA'] < $sectorRelation['QUOTA'] ){
								$winnerSector = $sectorRelation['CDSETOR'];
							} else {
								$winnerSector = $setorCorr['IDLINHAPRINCIPAL'] == 'S' ? $setorCorr['CDSETOR'] : $sectorRelation['CDSETOR'];
							}
						} else {
							$winnerSector = $sectorRelation['CDSETOR'];
						}
						if ($winnerLine == NULL){
							$winnerLine = ($winnerSector == $sectorRelation['CDSETOR']) ? $sectorRelation['CDLINHAPROD'] : $setorCorr['CDLINHAPROD'];
							$this->winnerLine = $winnerLine;
						}
						array_push($winnerSectors, $winnerSector);
					}
					else if ($winnerLine == NULL || $winnerLine == $sectorRelation[0]['CDLINHAPROD']){ 
						array_push($winnerSectors, $currentSector['CDSETOR']);
						if ($winnerLine == NULL){
							$winnerLine = $sectorRelation[0]['CDLINHAPROD'];
							$this->winnerLine = $winnerLine;
						}
					}
				}
			}
		}

		$winnerItems = array();

		if(!empty($winnerSectors)){
			foreach($winnerSectors as $currentWinnerSector){
				array_push($winnerItems, array_values(
						array_filter($sectorItems, function($productSector) use($currentWinnerSector){
							return $productSector['CDSETOR'] == $currentWinnerSector;
						})
					)[0]
				);
			}
		} else {
			$winnerItems = $sectorItems;
		}
		return $winnerItems;
		// FIM DA REGRA DE BALANCEAMENTO
	}
	
	private function handleSector($item) {
		// busca se item está no setor expedição (se estiver, deve ser pai do resto)
		$itemExpedition = $this->KDSManagerService->getProductSectorByType($item['CDPRODUTO'], $item['CDFILIAL'], $item['CDLOJA'], 'E');
		//distribuição de itens nos setores de expedição - desativado
		// $itemExpedition = $this->distributeItems($item['CDFILIAL'], $itemExpedition);

		//Redirecionemento do pedido por Origem da venda
		$itemExpedition = $this->KDSManagerService->redirectOrderSector($item['CDFILIAL'], $itemExpedition, $item['IDORIGEMVENDA']);

		// Expedições
		foreach ($itemExpedition as $currentSector) {
			$this->handleItem($item, $currentSector['CDSETOR'], $currentSector['IDTIPOSETOR'], null, $currentSector['NRTEMPOPROD'], 'EXPEDICAO', null, $currentSector['NRTEMPOEXIB'], $currentSector['NRCORVGM']);
		}

		if (empty($itemExpedition)) {
			// busca se item está no setor montagem (se estiver, deve ser pai do resto (junto com expedição))
			// Montagens
			$itemMontagem = $this->KDSManagerService->getProductSectorByType($item['CDPRODUTO'], $item['CDFILIAL'], $item['CDLOJA'], 'M');

			//Redirecionemento do pedido por Origem da venda
			$itemMontagem = $this->KDSManagerService->redirectOrderSector($item['CDFILIAL'], $itemMontagem, $item['IDORIGEMVENDA']);

			$itemMontagem = $this->distributeItems($item['CDFILIAL'], $itemMontagem);

			foreach ($itemMontagem as $currentSector) {
				$this->handleItem($item, $currentSector['CDSETOR'], $currentSector['IDTIPOSETOR'], null, $currentSector['NRTEMPOPROD'], 'MONTAGEM', null, $currentSector['NRTEMPOEXIB'], $currentSector['NRCORVGM']);
			}

			if (empty($itemMontagem)) {
				if (!in_array($item['CHAVEITEM'], $this->productionItemProcessed)) {
					array_push($this->productionItemProcessed, $item['CHAVEITEM']);

					// busca se produto está no setor produção se ainda não foi processado
					$itemProduction = $this->KDSManagerService->getProductSectorByType($item['CDPRODUTO'], $item['CDFILIAL'], $item['CDLOJA'], 'P');
					// Redirecionamento do pedido por origem da venda
					$itemProduction = $this->KDSManagerService->redirectOrderSector($item['CDFILIAL'], $itemProduction, $item['IDORIGEMVENDA']);
					
					foreach ($itemProduction as $currentSector) {
						$this->handleItem($item, $currentSector['CDSETOR'], $currentSector['IDTIPOSETOR'], null, $currentSector['NRTEMPOPROD'], 'PRODUCAO', null, $currentSector['NRTEMPOEXIB'], $currentSector['NRCORVGM']);
					}
				}
			}
		}
		if (empty($itemExpedition) && empty($itemMontagem) && empty($itemProduction)) {
			// da update no item na venda para avisar que já foi tratado pelo KDS
			$NRPEDIDOFOS = 'NPRODUZIDO';
			if (isset($item['CDFILIAL'], $item['NRVENDAREST'], $item['NRCOMANDA'], $item['NRPRODCOMVEN']) && $item['NRVENDAREST'] != '' && $item['NRCOMANDA'] != ''&& $item['NRPRODCOMVEN'] != '') {
				$this->KDSManagerService->updateSaleMesa($NRPEDIDOFOS, $item['CDFILIAL'], $item['NRVENDAREST'], $item['NRCOMANDA'], $item['NRPRODCOMVEN']);
			}
			if (isset($item['CDFILIAL'], $item['CDCAIXA'], $item['NRSEQVENDA'], $item['NRSEQUITVEND']) && $item['NRSEQVENDA'] != '' && $item['NRSEQUITVEND'] != '') {
				$this->KDSManagerService->updateSaleVenda($NRPEDIDOFOS, $item['CDFILIAL'], $item['CDCAIXA'], $item['NRSEQVENDA'], $item['NRSEQUITVEND']);
			}
		}
	}

	private function handleItem($item, $CDSETOR, $IDTIPOSETOR, $DSCHAVEPEDPAI, $NRTEMPOPROD, $TIPOSETOR, $arrayDadosPaiExpedicao, $NRTEMPOEXIB, $NRCORVGM) {
		// preparing order params to insertion
		$CDFILIAL = $item['CDFILIAL'];
		$CDLOJA = $item['CDLOJA'];
		$DSPEDIDOFOS = $item['CHAVE'];
		$DSCHAVEPEDFOS = $item['CHAVE'];
		$DTPEDIDOFOS = $item['DATA'];
		$IDSTATUSPEDFOS = '1';
		$IDATIVO = 'S';
		$DSTIPOPEDFOS = $item['VENDA'];
		$NRMESA = $item['NRMESA'];
		$NRCOMANDA = $item['NRCOMANDA'];
		$DSCOMANDA = $item['DSCOMANDA'];
		$QTMAXPEDFOS = null;
		$CDSENHAPED = $item['CDSENHAPED'];
		$NMCONSUMIDOR = $item['NMCONSUMIDOR'];
		$CDGRPOCOR = $item['CDGRPOCOR'];
		$CDOCORR = $item['CDOCORR'];
		if (isset($item['TXPRODCOMVENEST'])){
			$TXPRODCOMVENEST = $item['TXPRODCOMVENEST'];
		}
		$OBSDIGITADA = $item['OBSDIGITADA'];
		$DTHREXIBICAOPROD = new \Datetime($item['NOW'], new \DateTimeZone(date_default_timezone_get()));
		if (($item['NRATRAPROD'] == 0)) {
			$CHAVEPEDIDO = $item['CHAVE'];
			$NRATRAPROD = null;
		} else {
			$CHAVEPEDIDO = $item['CHAVE'] . $item['NRATRAPROD'];
			$NRATRAPROD = $item['NRATRAPROD'];
		}
		$NRLUGARMESAIT = intval($item['NRLUGARMESA']);
		$IDORIGEMVENDA = $item['IDORIGEMVENDA'];
		$IDITORIGEMVENDA = $item['IDITORIGEMVENDA'];
		
		// preparing order item params to insertion
		$CDPRODUTO = $item['CDPRODUTO'];
		$QTPRODPEFOS = $item['QTDEPROD'];
		$CDCAIXA = $item['CDCAIXA'];
		$NRSEQVENDA = $item['NRSEQVENDA'];
		$NRSEQUITVEND = $item['NRSEQUITVEND'];
		$NRVENDAREST = $item['NRVENDAREST'];
		$NRCOMANDA = $item['NRCOMANDA'];
		$NRPRODCOMVEN = $item['NRPRODCOMVEN'];
		$IDSITITPEDFOS = 'E'; // 'E' -> não inicializado
		
		// caso seja promoção inteligente
		$CDPRODPROMOKDS = $item['CDPRODPROMOKDS'];
		$NRSEQPRODCOMKDS = $item['NRSEQPRODCOMKDS'];
		$CHAVEITEM = $item['CHAVEITEM'];
		$IDITORIGEMVENDA = $item['IDITORIGEMVENDA'];

		// parametros de controle de combo do combo
		$NRSEQITVENDAEST = $item['NRSEQITVENDAEST'] ?? null;
		$NRSEQCOMB = $item['NRSEQCOMB'] ?? null;

		// confere se o pedido já existe
		// irá entrar aqui se o pedido vier do FOS pois a chave é diferente
		$chave = $this->KDSManagerService->checkOrderExistence($item['CHAVE'], $item['DTPEDIDO']);
		if (empty($chave)) {
			// se o pedido veio do FOS
			if ($item['IDORIPED'] === 'F') {
				$this->KDSManagerService->deleteItemPedidofos($CDFILIAL, $item['NRPEDIDOFOS'], $item['NRITPEDIDOFOS']);
				$this->KDSManagerService->deletePedidofos($CDFILIAL, $item['NRPEDIDOFOS']);
			}
			// se o pedido ainda não existir, insere na PEDIDOFOS
			$this->util->newCode('PEDIDOFOS' . $CDFILIAL);
			$NRPEDIDOFOS = $this->KDSManagerService->getNewCode('PEDIDOFOS' . $CDFILIAL);
			$NRPEDIDOFOS = $NRPEDIDOFOS[0]['NRSEQUENCIAL'];
			$NRPEDIDOFOS = substr($NRPEDIDOFOS, -10);
			$NRPEDIDOFOSMSDE = $NRPEDIDOFOS;
			$NRPEDIDOFOSAUX = $NRPEDIDOFOS;
			// insere o pedido
			$this->KDSManagerService->insertOrder(
				$CDFILIAL,
				$NRPEDIDOFOS,
				$DSPEDIDOFOS,
				$DSCHAVEPEDFOS,
				$DTPEDIDOFOS,
				$IDSTATUSPEDFOS,
				$DSTIPOPEDFOS,
				$NRMESA,
				$NRCOMANDA,
				$DSCOMANDA,
				$QTMAXPEDFOS,
				$CDSETOR,
				$NRPEDIDOFOSAUX,
				$CDLOJA,
				$CDSENHAPED,
				$NRPEDIDOFOSMSDE,
				$NMCONSUMIDOR,
				$IDATIVO
			);
		} else {
			// se o pedido já existir, insere somente na ITPEDIDOFOS
			// se esse pedido veio do FOS, apaga ele e recria
			if ($item['IDORIPED'] === 'F') {
				$this->KDSManagerService->deleteItemPedidofos($CDFILIAL, $item['NRPEDIDOFOS'], $item['NRITPEDIDOFOS']);
				$item['QUANTIDADE'] = $item['QTDEPROD'];
				$productsPromoction = array();
				array_push($productsPromoction, $item);
			}
			$NRPEDIDOFOS = $chave[0]['NRPEDIDOFOS'];
			$NRPEDIDOFOSMSDE = $NRPEDIDOFOS;
			$NRPEDIDOFOSAUX = $NRPEDIDOFOS;
		}

		$NRITPEDIDOFOS = $this->generateNewNRITPEDIDOFOS($CDFILIAL, $NRPEDIDOFOS);

		// insere o relacionamento do item atual com o item pai na expedição (junior na montagem por exemplo)
		if (!empty($arrayDadosPaiExpedicao)) {
			$NRSEQITPEDPAI = $this->generateNewNRSEQITPEDPAI($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS);
			array_push($this->itemsRelations, array(
				'CDFILIAL'         => $CDFILIAL,
				'NRPEDIDOFOS'      => $NRPEDIDOFOS,
				'NRITPEDIDOFOS'    => $NRITPEDIDOFOS,
				'NRSEQITPEDPAI'    => $NRSEQITPEDPAI,
				'CDFILIALPAI'      => $arrayDadosPaiExpedicao['CDFILIAL'],
				'NRPEDIDOFOSPAI'   => $arrayDadosPaiExpedicao['NRPEDIDOFOS'],
				'NRITPEDIDOFOSPAI' => $arrayDadosPaiExpedicao['NRITPEDIDOFOS']
			));
		}

		// monta dados do produto pai
		$DSCHAVEPEDPAI = $CDFILIAL . $NRPEDIDOFOS . $NRITPEDIDOFOS;
		$arrayDadosPaiExpedicao = array(
			'CDFILIAL' => $CDFILIAL,
			'NRPEDIDOFOS' => $NRPEDIDOFOS,
			'NRITPEDIDOFOS' => $NRITPEDIDOFOS
		);

		//monta observações
		$TXPRODPED = $this->buildTxProd($CHAVEITEM, $CDSETOR, $CDPRODUTO, $OBSDIGITADA, $item);
		
		if(empty($TXPRODPED)){
			$TXPRODPED = '';
		} else {
			// soma o tempo de produção das observações com o tempo de produção do item
			$TIMEPRODUCTION = $this->calcTimeObs($NRTEMPOPROD, $NRTEMPOEXIB, $TXPRODPED);
			$NRTEMPOPROD = $TIMEPRODUCTION['NRTEMPOPROD'];
			$NRTEMPOEXIB = $TIMEPRODUCTION['NRTEMPOEXIB'];
		}

		//Nova implementação para viagem
		if (!is_null($IDITORIGEMVENDA) && strpos(' ' . $IDITORIGEMVENDA, 'TGO') > 0) {
			if (!isset($TXPRODPED) || (isset($TXPRODPED) && !is_array($TXPRODPED))) {
				$TXPRODPED = array();
			}
			$has = false;
			foreach ($TXPRODPED as $key => $value) {
				if (strpos('  ' . $value['OBSERVACAO'], 'VIAGEM') != '') {
					//REMOVE OBSERVAÇÃO;
					$TXPRODPED[$key]['NRCORSINAL'] = $NRCORVGM;
					$has = true;
				}
			}

			if ($has == false) {
				array_push($TXPRODPED, array(
					'OBSERVACAO' => 'VIAGEM',
					'NRCORSINAL' => $NRCORVGM
				));
			}
		}

		// cria array para relacionamento de um item (não composição) com seu item de venda (ITCOMANDAVEN)
		// relacionamento tabela ITPEDIDOFOSREL
		$RELITCOMANDAVEN = array();
		if (($DSTIPOPEDFOS == 'COMANDA') || ($DSTIPOPEDFOS == 'MESA') || ($DSTIPOPEDFOS == 'RODIZIO')) {
			// se for venda balcão, esse relacionamento não existe
			array_push($RELITCOMANDAVEN, array(
				'NRVENDAREST'   => $NRVENDAREST,
				'NRCOMANDA'     => $NRCOMANDA,
				'NRPRODCOMVEN'  => $NRPRODCOMVEN,
				'QTPRODPEFOS'   => $QTPRODPEFOS,
				'IDSITITPEDFOS' => $IDSITITPEDFOS
			));
		}



		array_push($this->itemsForInsertion, array(
			'CDFILIAL'          => $CDFILIAL,
			'CDLOJA'            => $CDLOJA,
			'NRPEDIDOFOS'       => $NRPEDIDOFOS,
			'NRITPEDIDOFOS'     => $NRITPEDIDOFOS,
			'CDPRODUTO'         => $CDPRODUTO,
			'QTPRODPEFOS'      => $QTPRODPEFOS,
			'CDCAIXA' 		   => $CDCAIXA,
			'NRSEQVENDA'       => $NRSEQVENDA,
			'NRSEQUITVEND'     => $NRSEQUITVEND,
			'NRVENDAREST'      => $NRVENDAREST,
			'NRCOMANDA'        => $NRCOMANDA,
			'NRPRODCOMVEN'     => $NRPRODCOMVEN,
			'TXPRODPED'        => $TXPRODPED,
			'IDSITITPEDFOS'    => $IDSITITPEDFOS,
			'CDSETOR'          => $CDSETOR,
			'CDGRPOCOR'        => $CDGRPOCOR,
			'CDOCORR'          => $CDOCORR,
			'DTHREXIBICAOPROD' => $DTHREXIBICAOPROD,
			'DSCHAVEPEDPAI'    => $DSCHAVEPEDPAI,
			'CHAVEPEDIDO'      => $CHAVEPEDIDO,
			'DTPEDIDOFOS'      => $DTPEDIDOFOS,
			'CDPRODUTOPAI'     => null,
			'IDTIPOSETOR'      => $IDTIPOSETOR,
			'CDPRODPROMOKDS'   => $CDPRODPROMOKDS,
			'NRSEQPRODCOMKDS'  => $NRSEQPRODCOMKDS,
			'NRTEMPOPROD'      => $NRTEMPOPROD,
			'NRTEMPOEXIB'      => $NRTEMPOEXIB,
			'NRATRAPROD'       => $NRATRAPROD,
			'DSAPELIDOKDS'     => null,
			'RELITCOMANDAVEN'  => $RELITCOMANDAVEN,
			'NRLUGARMESAIT'    => $NRLUGARMESAIT,
			"IDORIGEMVENDA"    => $IDORIGEMVENDA,
			"NMCONSUMIDOR"	   => $NMCONSUMIDOR,
			"NRMESA" 		   => $NRMESA,
			"NRSEQITVENDAEST"  => $NRSEQITVENDAEST,
			"NRSEQCOMB" 	   => $NRSEQCOMB
		));

		// se for modo expedição, chama a handleItem para os filhos da expedição (para o mesmo produto)
		if ($TIPOSETOR == 'EXPEDICAO') {
			// busca se item está no setor montagem (se estiver, deve ser pai do resto (junto com expedição))
			$itemMontagem = $this->KDSManagerService->getProductSectorByType($item['CDPRODUTO'], $item['CDFILIAL'], $item['CDLOJA'], 'M');

			//Redirecionemento do pedido por Origem da venda
			$itemMontagem = $this->KDSManagerService->redirectOrderSector($item['CDFILIAL'], $itemMontagem, $item['IDORIGEMVENDA']);
			// olhar aqui
			$itemMontagem = $this->distributeItems($item['CDFILIAL'], $itemMontagem);

			foreach ($itemMontagem as $currentSector) {
				$this->handleItem($item, $currentSector['CDSETOR'], $currentSector['IDTIPOSETOR'], $DSCHAVEPEDPAI, $currentSector['NRTEMPOPROD'], 'MONTAGEM', $arrayDadosPaiExpedicao, $currentSector['NRTEMPOEXIB'], $currentSector['NRCORVGM']);
			}
		}

		if (($TIPOSETOR == 'MONTAGEM') || (empty($itemMontagem) && $TIPOSETOR == 'EXPEDICAO')) {
			// valida se é para buscar a composição deste produto
			$TXPRODCOMVENEST = isset($TXPRODCOMVENEST) ? $TXPRODCOMVENEST : null;

			$this->validateComposition(
				$CHAVEITEM,
				$CDFILIAL,
				$CDLOJA,
				$NRPEDIDOFOS,
				$NRITPEDIDOFOS,
				$CDPRODUTO,
				$TXPRODPED,
				$QTPRODPEFOS,
				$CDCAIXA,
				$NRSEQVENDA,
				$NRSEQUITVEND,
				$NRVENDAREST,
				$NRCOMANDA,
				$NRPRODCOMVEN,
				$IDSITITPEDFOS,
				$DSCHAVEPEDPAI,
				$DTPEDIDOFOS,
				$CHAVEPEDIDO,
				$CDOCORR,
				$CDGRPOCOR,
				$NRATRAPROD,
				$DSTIPOPEDFOS,
				$OBSDIGITADA,
				$NRLUGARMESAIT,
				$IDORIGEMVENDA,
				$IDITORIGEMVENDA,
				$NRTEMPOPROD,
				$NRTEMPOEXIB,
				$TXPRODCOMVENEST,
				$NRMESA,
				$NRSEQITVENDAEST,
				$NRSEQCOMB
			);

			if (!in_array($item['CHAVEITEM'], $this->productionItemProcessed)) {
				array_push($this->productionItemProcessed, $item['CHAVEITEM']);

				//Busca itens do setor de produção
				$itemProduction = $this->KDSManagerService->getProductSectorByType($item['CDPRODUTO'], $item['CDFILIAL'], $item['CDLOJA'], 'P');

				//Redirecionemento do pedido por Origem da venda
				$itemProduction = $this->KDSManagerService->redirectOrderSector($item['CDFILIAL'], $itemProduction, $item['IDORIGEMVENDA']);

				//distribuição de itens em setores de produção - desativado
				$itemProduction = $this->distributeItems($item['CDFILIAL'], $itemProduction, '1');
				//AQUI

				foreach ($itemProduction as $currentSector) {
					$this->handleItem($item, $currentSector['CDSETOR'], $currentSector['IDTIPOSETOR'], $DSCHAVEPEDPAI, $currentSector['NRTEMPOPROD'], 'PRODUCAO', $arrayDadosPaiExpedicao, $currentSector['NRTEMPOEXIB'], $currentSector['NRCORVGM']);
				}
			}
		}

		// da update no item na venda para avisar que já foi tratado pelo KDS
		switch ($DSTIPOPEDFOS) {
			case 'MESA';
			case 'COMANDA';
				$this->KDSManagerService->updateSaleMesa($NRPEDIDOFOS, $CDFILIAL, $NRVENDAREST, $NRCOMANDA, $NRPRODCOMVEN);
				break;
			case 'VENDA';
				$this->KDSManagerService->updateSaleVenda($NRPEDIDOFOS, $CDFILIAL, $CDCAIXA, $NRSEQVENDA, $NRSEQUITVEND);
				break;
		}
	}

	private function buildTxProd($CHAVEITEM, $CDSETOR, $CDPRODUTO, $OBSDIGITADA, $item){
		// busca se há alguma observação pré cadastrada pro item
		$observacoes = $this->KDSManagerService->getAllItemObservations($CHAVEITEM);
		if (!empty($observacoes)) {
			// define quais observações irão aparecer pra esse item
			// busca as observações cadastradas na SETORPRODOBS
			$todasObsSetorProduto = $this->KDSManagerService->getSetorProdutoObservations($CDSETOR, $CDPRODUTO);
			if (!empty($todasObsSetorProduto)) {
				// se possui algo na SETORPRODOBS para esse produto neste setor, busca as observações que são compatíveis
				$observacoes = $this->KDSManagerService->getItemObservationsBySetorProduto($CHAVEITEM, $CDSETOR, $CDPRODUTO);
			} /*else {
				// se esse produto não tiver nenhum registro na SETORPRODOBS (pra este setor),
				// mostra todas as observações do item
			}*/
		}

		// constrói array de observações para este item
		if (isset($item['TXPRODPED'])) {
			try {
				$TXPRODPED = json_decode($item['TXPRODPED'], TRUE);
			} catch (\Exception $e) {
			}
		} else {
			$TXPRODPED = null;
		}
		if (!empty($observacoes) && ($item['VENDA'] != 'RODIZIO')) {
			$TXPRODPED = array();
			foreach ($observacoes as $observacao) {
				array_push($TXPRODPED, array(
					'OBSERVACAO' => $observacao['OBSERVACAO'],
					'DSAPELIDOOBS'=> $observacao['DSAPELIDOOBS'],
					'NRCORSINAL' => $observacao['NRCORSINAL'],
					'NRTEMPOPROD' => intval($observacao['NRTEMPOPROD'])
				));
			}
			if (!empty($OBSDIGITADA)) {
				try {
					$replacedObservation = str_replace(':', ';', $OBSDIGITADA);
                    $explodedObservation = explode(';', $replacedObservation);

					$digitada = json_decode($OBSDIGITADA, TRUE);
					if (!empty($digitada)) {
						array_push($TXPRODPED, $digitada);
					} else {
						foreach ($explodedObservation as $obs) {
							$contains = false;
							foreach($observacoes as $observacao){
								if(stripos($observacao['OBSERVACAO'], trim($obs)) !== false || stripos($obs, $observacao['OBSERVACAO']) !== false){
									$contains = true;
								}
							}
							if(!$contains){
								if (trim($obs) !== '') {
									array_push($TXPRODPED, array(
										'OBSERVACAO' => $obs,
										'NRCORSINAL' => 'FFFFFF'
									));
								}
							}
                        }
					}
				} catch (\Exception $e) {
					array_push($TXPRODPED, array(
						'OBSERVACAO' => $OBSDIGITADA,
						'NRCORSINAL' => 'FFFFFF'
					));
				}
			}
		} else if (!empty($OBSDIGITADA) && ($item['VENDA'] != 'RODIZIO')) {
			$TXPRODPED = array();
			if (!empty($OBSDIGITADA)) {
				try {
					$replacedObservation = str_replace(':', ';', $OBSDIGITADA);
                    $explodedObservation = explode(';', $replacedObservation);

					$digitada = json_decode($OBSDIGITADA, TRUE);
					if (!empty($digitada)) {
						array_push($TXPRODPED, $digitada);
					} else {
						foreach ($explodedObservation as $obs) {
                            if (trim($obs) !== '') {
								array_push($TXPRODPED, array(
                                    'OBSERVACAO' => $obs,
                                    'NRCORSINAL' => 'FFFFFF'
                                ));
                            }
                        }
					}
				} catch (\Exception $e) {
					array_push($TXPRODPED, array(
						'OBSERVACAO' => $OBSDIGITADA,
						'NRCORSINAL' => 'FFFFFF'
					));
				}
			}
		}
		if (!empty($item['TXPRODCOMVENEST'])){
			$TXPRODCOMVENEST = $item['TXPRODCOMVENEST'];
			$explodedObservation = explode(';', $TXPRODCOMVENEST);
			$TXPRODPED = array();
			try{
				foreach ($explodedObservation as $obs) {
					if (trim($obs) !== '') {
						$obs = trim($obs);
						$obsTXPRODCOMVENEST = $this->KDSManagerService->verifyObservationByName($obs);
						if (!empty($obsTXPRODCOMVENEST)) {
							array_push($TXPRODPED, $obsTXPRODCOMVENEST[0]);
						} else { 
							array_push($TXPRODPED, array(
								'OBSERVACAO' => $obs,
								'NRCORSINAL' => 'FFFFFF'
							));
						}
					}
				}
			} catch(\Exception $e){
				array_push($TXPRODPED, array(
					'OBSERVACAO' => $OBSDIGITADA,
					'NRCORSINAL' => 'FFFFFF'
				));
			}
		}

		$NEWTXPRODPED = $this->removeObservationDollarSign($TXPRODPED);
        return $NEWTXPRODPED === [] ? "" : $NEWTXPRODPED;
	}

	public function removeObservationDollarSign($TXPRODPED){
        $NEWTXPRODPED = [];
        if (is_array($TXPRODPED)){
            foreach ($TXPRODPED as $value => $key){
                $observacao = explode('$', $key['OBSERVACAO']);
                $tamanho = sizeof($observacao);
                if ($tamanho != 2 ){
                    array_push($NEWTXPRODPED, $key);
                }
            }
            return $NEWTXPRODPED;
        } else {
            return $TXPRODPED;
        }
        // FUNÇÃO PARA REMOVER AS OBSERVAÇÕES QUE POSSUEM O SÍMBOLO DO DOLAR.
    }

	private function calcTimeObs ($NRTEMPOPROD, $NRTEMPOEXIB, $TXPRODPED) {

		$NRTEMPOPROD = empty($NRTEMPOPROD) ? 0 : intval($NRTEMPOPROD);
		$NRTEMPOEXIB = empty($NRTEMPOEXIB) ? 0 : intval($NRTEMPOEXIB);

		if (!empty($TXPRODPED)){
			foreach ($TXPRODPED as $OBS) {
				if (isset($OBS['NRTEMPOPROD'])) {
					$NRTEMPOPROD += intval($OBS['NRTEMPOPROD']);
					$NRTEMPOEXIB += intval($OBS['NRTEMPOPROD']); // Não tem NRTEMPOEXIB na Tabela SETORPRODOBS
				}
			}
		}

		// verifica se depois do calculo o tempo de produção não fica < 0
		$NRTEMPOPROD = ($NRTEMPOPROD < 0) ? 0 : $NRTEMPOPROD;
		$NRTEMPOEXIB = ($NRTEMPOEXIB < 0) ? 0 : $NRTEMPOEXIB;

		$time = array(
			'NRTEMPOPROD' => (string)$NRTEMPOPROD,
			'NRTEMPOEXIB' => (string)$NRTEMPOEXIB
		);
		return $time;
		// função para somar o tempo de produção do item com o tempo de produção das observações
	}

	private function validateComposition(
		$CHAVEITEM,
		$CDFILIAL,
		$CDLOJA,
		$NRPEDIDOFOS,
		$NRITPEDIDOFOS,
		$CDPRODUTO,
		$TXPRODPED,
		$QTPRODPEFOS,
		$CDCAIXA,
		$NRSEQVENDA,
		$NRSEQUITVEND,
		$NRVENDAREST,
		$NRCOMANDA,
		$NRPRODCOMVEN,
		$IDSITITPEDFOS,
		$DSCHAVEPEDPAI,
		$DTPEDIDOFOS,
		$CHAVEPEDIDO,
		$CDOCORR,
		$CDGRPOCOR,
		$NRATRAPROD,
		$DSTIPOPEDFOS,
		$OBSDIGITADA,
		$NRLUGARMESAIT,
		$IDORIGEMVENDA,
		$IDITORIGEMVENDA,
		$NRTEMPOPROD,
		$NRTEMPOEXIB,
		$TXPRODCOMVENEST = null,
		$NRMESA = null,
		$NRSEQITVENDAEST = null,
		$NRSEQCOMB = null
	) {
		// valida se o item atual já existe no array para procurar a composição, evita duplicar
		if (!$this->checkInCompositionArray($CHAVEITEM, $this->itemsToHandleComposition)) {

			// relacionamento tabela ITPEDIDOFOSPAI
			$arrayDadosPai = array(
				'CDFILIAL' => $CDFILIAL,
				'NRPEDIDOFOS' => $NRPEDIDOFOS,
				'NRITPEDIDOFOS' => $NRITPEDIDOFOS
			);

			// relacionamento tabela ITPEDIDOFOSREL
			$RELITCOMANDAVEN = array();
			if (($DSTIPOPEDFOS == 'COMANDA') || ($DSTIPOPEDFOS == 'MESA') || ($DSTIPOPEDFOS == 'RODIZIO')) {
				// se for venda balcão, esse relacionamento não existe
				array_push($RELITCOMANDAVEN, array(
					'NRVENDAREST' => $NRVENDAREST,
					'NRCOMANDA' => $NRCOMANDA,
					'NRPRODCOMVEN' => $NRPRODCOMVEN,
					'QTPRODPEFOS' => $QTPRODPEFOS,
					'IDSITITPEDFOS' => $IDSITITPEDFOS
				));
			}

			// tenta agrupar os produtos(PAIS) iguais, se não conseguir, coloca um novo item no array de composição
			if (!$this->groupComposition(
				$CDFILIAL,
				$NRPEDIDOFOS,
				$CDPRODUTO,
				$TXPRODPED,
				$QTPRODPEFOS,
				$CHAVEITEM,
				$NRATRAPROD,
				$arrayDadosPai,
				$RELITCOMANDAVEN,
				$NRLUGARMESAIT
			)) {
				$arrayChaveItem = array(
					'CHAVEITEM' => $CHAVEITEM
				);
				array_push($this->itemsToHandleComposition, array(
					'CDFILIAL'        => $CDFILIAL,
					'CDLOJA'        => $CDLOJA,
					'CDPRODUTO'       => $CDPRODUTO,
					'QTPRODPEFOS'     => $QTPRODPEFOS,
					'NRPEDIDOFOS'     => $NRPEDIDOFOS,
					'CDCAIXA'         => $CDCAIXA,
					'NRSEQVENDA'      => $NRSEQVENDA,
					'NRSEQUITVEND'    => $NRSEQUITVEND,
					'NRVENDAREST'     => $NRVENDAREST,
					'NRCOMANDA'       => $NRCOMANDA,
					'NRPRODCOMVEN'    => $NRPRODCOMVEN,
					'TXPRODPED'       => $TXPRODPED,
					'IDSITITPEDFOS'   => $IDSITITPEDFOS,
					'DSCHAVEPEDPAI'   => $DSCHAVEPEDPAI,
					'DTPEDIDOFOS'     => $DTPEDIDOFOS,
					'NRITPEDIDOFOS'   => $NRITPEDIDOFOS,
					'CHAVEPEDIDO'     => $CHAVEPEDIDO,
					'CDOCORR'         => $CDOCORR,
					'CDGRPOCOR'       => $CDGRPOCOR,
					'CHAVEITEM'       => array($arrayChaveItem),
					'ARRAYPAI'        => array($arrayDadosPai),
					'NRATRAPROD'      => $NRATRAPROD,
					'RELITCOMANDAVEN' => $RELITCOMANDAVEN,
					'OBSDIGITADA'     => $OBSDIGITADA,
					'NRLUGARMESAIT'   => $NRLUGARMESAIT,
					'IDORIGEMVENDA'   => $IDORIGEMVENDA,
					'IDITORIGEMVENDA' => $IDITORIGEMVENDA,
					'NRTEMPOPROD' => $NRTEMPOPROD,
					'NRTEMPOEXIB' => $NRTEMPOEXIB,
					'TXPRODCOMVENEST' => $TXPRODCOMVENEST,
					'DSTIPOPEDFOS' 	=> $DSTIPOPEDFOS,
					'NRMESA' 		=> $NRMESA,
					'NRSEQITVENDAEST' => $NRSEQITVENDAEST,
					'NRSEQCOMB' 	=> $NRSEQCOMB
				));
			}
		}
	}

	private function generateNewNRITPEDIDOFOS($CDFILIAL, $NRPEDIDOFOS)
	{
		$validKey = false;
		while ($validKey == false) {
			$this->util->newCode('ITPEDIDOFOS' . $CDFILIAL . $NRPEDIDOFOS);
			$NRITPEDIDOFOS = $this->KDSManagerService->getNewCode('ITPEDIDOFOS' . $CDFILIAL . $NRPEDIDOFOS);
			$NRITPEDIDOFOS = $NRITPEDIDOFOS[0]['NRSEQUENCIAL'];
			$NRITPEDIDOFOS = substr($NRITPEDIDOFOS, -10);

			$validKey = $this->KDSManagerService->checkIfKeyExists($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS);
		}
		return $NRITPEDIDOFOS;
	}

	private function generateNewNRSEQITPEDPAI($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS)
	{
		$validKey = false;
		while ($validKey == false) {
			$this->util->newCode('ITPEDIDOFOSPAI' . $CDFILIAL . $NRPEDIDOFOS . $NRITPEDIDOFOS);
			$NRSEQITPEDPAI = $this->KDSManagerService->getNewCode('ITPEDIDOFOSPAI' . $CDFILIAL . $NRPEDIDOFOS . $NRITPEDIDOFOS);
			$NRSEQITPEDPAI = $NRSEQITPEDPAI[0]['NRSEQUENCIAL'];
			$NRSEQITPEDPAI = substr($NRSEQITPEDPAI, -10);

			$validKey = $this->KDSManagerService->checkIfITPEDIDOFOSPAIExists($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS, $NRSEQITPEDPAI);
		}
		return $NRSEQITPEDPAI;
	}

	private function generateNewNRSEQITPEDREL($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS)
	{
		$this->util->newCode('ITPEDIDOFOSREL' . $CDFILIAL . $NRPEDIDOFOS . $NRITPEDIDOFOS);
		$NRSEQITPEDREL = $this->KDSManagerService->getNewCode('ITPEDIDOFOSREL' . $CDFILIAL . $NRPEDIDOFOS . $NRITPEDIDOFOS);
		$NRSEQITPEDREL = $NRSEQITPEDREL[0]['NRSEQUENCIAL'];
		$NRSEQITPEDREL = substr($NRSEQITPEDREL, -10);
		return $NRSEQITPEDREL;
	}

	private function checkInCompositionArray($key, $array)
	{
		foreach ($array as $currentItem) {
			foreach ($currentItem['CHAVEITEM'] as $currentKey) {
				if ($currentKey['CHAVEITEM'] == $key) {
					return true;
				}
			}
		}
		return false;
	}

	private function groupComposition(
		$CDFILIAL,
		$NRPEDIDOFOS,
		$CDPRODUTO,
		$TXPRODPED,
		$QTPRODPEFOS,
		$CHAVEITEM,
		$NRATRAPROD,
		$arrayDadosPai,
		$RELITCOMANDAVEN,
		$NRLUGARMESAIT
	) {
		foreach ($this->itemsToHandleComposition as &$currentItem) {

			if (($currentItem['CDFILIAL'] == $CDFILIAL) && ($currentItem['NRPEDIDOFOS'] == $NRPEDIDOFOS) &&
				($currentItem['CDPRODUTO'] == $CDPRODUTO) && ($currentItem['TXPRODPED'] == $TXPRODPED) &&
				($currentItem['NRATRAPROD'] == $NRATRAPROD) && ($currentItem['NRLUGARMESAIT'] == $NRLUGARMESAIT)
			) {

				$currentItem['QTPRODPEFOS'] += $QTPRODPEFOS;

				if (!empty($CHAVEITEM)) {
					// trata chave do item para não duplicar
					$arrayChaveItem = $currentItem['CHAVEITEM'];
					array_push($arrayChaveItem, array(
						'CHAVEITEM' => $CHAVEITEM
					));
					$currentItem['CHAVEITEM'] = $arrayChaveItem;
				}

				// coloca chave do pai do item no array para inserir na ITPEDIDOFOSPAI
				$arrayPai = $currentItem['ARRAYPAI'];
				array_push($arrayPai, $arrayDadosPai);
				$currentItem['ARRAYPAI'] = $arrayPai;

				// junta relacionamentos (ITPEDIDOFOSREL)
				$currentItem['RELITCOMANDAVEN'] = array_merge($currentItem['RELITCOMANDAVEN'], $RELITCOMANDAVEN);

				return true;
			}
		}
		return false;
	}

	public function buildDataSource($CDSETOR = null)
	{
		// busca todos os items para montar as caixas do KDS
		// será um pedido por item (uma caixa por produto)
		// $start = microtime(true);
		// AQUI
		$pedidos = array();
		$itemsArray = $this->KDSManagerService->getRequestItem($CDSETOR);
		foreach ($itemsArray as $currentItem) {

			// trata cancelamento na venda mesa, comanda ou delivery
			if (!empty($currentItem['NRVENDAREST'])) {
				// se a caixa não está toda cancelada
				if ($currentItem['IDSITITPEDFOS'] != 'C') {
					// relação com ITCOMANDAVEN
					$itemsRelations = $this->KDSManagerService->getItemsRelations($currentItem['CDFILIAL'], $currentItem['NRPEDIDOFOS'], $currentItem['NRITPEDIDOFOS']);
					$newQuantity = 0;
					$canceledQuantity = 0;
					foreach ($itemsRelations as $relation) {
						if ($relation['IDSITITPEDFOS'] != 'C') {
							$newQuantity = $newQuantity + $relation['QTPRODPEFOS'];
						} else {
							$canceledQuantity = $canceledQuantity + $relation['QTPRODPEFOS'];
						}
					}

					// se a nova quantidade for igual a zero, cancela o item pai (na ITPEDIDOFOS)
					if ($newQuantity <= 0) {
						$currentItem['IDSITITPEDFOS'] = 'C';
						$this->KDSManagerService->cancelItem($currentItem['CDFILIAL'], $currentItem['NRPEDIDOFOS'], $currentItem['NRITPEDIDOFOS']);

						if ($currentItem['IDLIBERADO'] == 'N') {
							// expede se não tiver liberado para impedir que ele apareça na produção
							$currentItem['IDSITITPEDFOS'] = 'X';
							$this->KDSManagerService->changeItemStatusReleased($currentItem['CDFILIAL'], $currentItem['NRPEDIDOFOS'], $currentItem['NRITPEDIDOFOS']);
						}
					} else if ($newQuantity != $currentItem['QTPRODPEFOS'] && $canceledQuantity > 0) {
						// se a quantidade mudou
						// se um ou mais itens (não todos) foram cancelados, updata a nova quantidade
						$currentItem['QTPRODPEFOS'] = $newQuantity;
						$this->KDSManagerService->changeItemQuantity($currentItem['CDFILIAL'], $currentItem['NRPEDIDOFOS'], $currentItem['NRITPEDIDOFOS'], $currentItem['QTPRODPEFOS']);
					}
					if ($canceledQuantity > 0) {
						// muda label para mostrar os itens cancelados dentro da mesma caixa
						if (empty($currentItem['TXPRODPED'])) {
							$currentItem['TXPRODPED'] = array();
						} else {
							$currentItem['TXPRODPED'] = json_decode($currentItem['TXPRODPED']);
						}

						array_push($currentItem['TXPRODPED'], array(
							'OBSERVACAO' => 'QTD CANCELADA: ' . $canceledQuantity,
							'NRCORSINAL' => 'EE0000'
						));

						$currentItem['TXPRODPED'] = json_encode($currentItem['TXPRODPED']);
					}
				} else {
					// recalcula quantidade do item pai quando cancelar os filhos
					$itemsRelations = $this->KDSManagerService->getItemsRelations($currentItem['CDFILIAL'], $currentItem['NRPEDIDOFOS'], $currentItem['NRITPEDIDOFOS']);
					$canceledQuantity = 0;
					foreach ($itemsRelations as $relation) {
						if ($relation['IDSITITPEDFOS'] == 'C') {
							$canceledQuantity = $canceledQuantity + $relation['QTPRODPEFOS'];
						}
					}
					if ($canceledQuantity > 0 && $canceledQuantity != $currentItem['QTPRODPEFOS']) {

						//NEWNRITPEDIDOFOS comentado por não ser utilizado
						//$NEWNRITPEDIDOFOS = $this->generateNewNRITPEDIDOFOS($currentItem['CDFILIAL'], $currentItem['NRPEDIDOFOS']);

						$this->KDSManagerService->changeItemQuantity($currentItem['CDFILIAL'], $currentItem['NRPEDIDOFOS'], $currentItem['NRITPEDIDOFOS'], $currentItem['QTPRODPEFOS'] - $canceledQuantity);
						$currentItem['QTPRODPEFOS'] = $canceledQuantity;
					}
				}
			}

			// remove o pai da promoção combinada no setor de produção
			$showComboProduction = $this->instanceManager->getParameter('showComboProduction');
			if (isset($currentItem['IDTIPOSETOR']) && $currentItem['IDTIPOSETOR'] == 'P' && !$showComboProduction){
				$currentItem['NMPROMOCAO'] = null;
				$currentItem['NRSEQPRODCOMKDS'] = null;
			}

			// se for venda balcão, o número do pedido será o mesmo impresso no cupom fiscal
			$currentItem['orderCode'] = $currentItem['CDFILIAL'] . $currentItem['NRPEDIDOFOS'] . $currentItem['NRITPEDIDOFOS'] . $currentItem['NRSEQLIBERADO'];
			$currentItem['label'] = $currentItem['CDSENHAPED'];
			// retira os zeros do número da mesa
			$currentItem['NRMESA'] = intval($currentItem['NRMESA']);
			// retira os zeros do número da mesa antiga
			$currentItem['NRMESAOLD'] = intval($currentItem['NRMESAOLD']);
			// retira os zeros da quantidade
			if (is_int($currentItem['QTPRODPEFOS'])) {
				$currentItem['QTPRODPEFOS'] = intval($currentItem['QTPRODPEFOS']);
			}
			// busca intervalos de produção do item
			$currentItem['INTERVALS'] = $this->KDSManagerService->getItemIntervals($currentItem['CDPRODUTO'], $currentItem['CDSETOR']);
			// busca origem do item
			if ($currentItem['IDORIGEMVENDA']) {
				// PROCURE AQUI
				$currentItem['ORIGEM'] = $this->getOrigins->manageOrigin($currentItem['IDORIGEMVENDA'], "system", "origin");
				if (isset($currentItem['IDRETBALLOJA']) && strpos($currentItem['ORIGEM'], 'DELIVERY') !== false) {
					$currentItem['ORIGEM'] = $currentItem['ORIGEM'] . $this->getOrigins->getToGo($currentItem['IDRETBALLOJA']);
				}
			}
			// substitui o setor do pedido pelo setor do produto (gambiarra) - WHY?!
			$currentItem['IDTIPOSETOR'] = 'P';
			// cria um array com somente um item
			$item = array();
			array_push($item, $currentItem);
			$currentItem['items'] = $item;

			if ($currentItem['IDSITITPEDFOS'] != 'X') {
				// coloca o pedido atual no array de retorno
				array_push($pedidos, $currentItem);
			}
		}

        if (count($pedidos) > 0) {
			$unePorPedido = count($this->KDSManagerService->verificaHabAgrup($pedidos[0]['CDFILIAL']));
			if ($unePorPedido > 0) {
				$setores = $this->mergeSetorProdutoMesa($pedidos);
				$setores = $this->verificaHabAgrupProd($setores);
				$pedidos = $this->agrupaPedidos($setores, $pedidos);
			}
		}
		// // echo "\nE gasto: ".(microtime(true)-// $start)."\n";
		return $pedidos;
	}

    function agrupaPedidos($setores, $pedidos) {
		$novosPedidos = array();
		foreach ($setores['CDSETOR'] as $cdsetor => $setor) {
			if ($setor['IDHABAGRUPPROD'] == 'S') {
				foreach ($setor['NRMESA'] as $nrmesa => $mesa) {
					foreach ($mesa['CDSENHAPED'] as $key => $cdsenhaped) {
						foreach ($pedidos as $key => $pedido) {
							if ($pedido['CDSENHAPED'] == $cdsenhaped && $pedido['CDSETOR'] == $cdsetor && $pedido['NRMESA'] == $nrmesa) {
								if ($pedido['NRATRAPRODITPE'] !== null) {
									if (!isset($pedidoMesaSegura)) {
										$pedidoMesaSegura = $pedido;
									} else {
										if (!in_array($pedido['items'][0], $pedidoMesaSegura['items'])) {
											array_push($pedidoMesaSegura['items'], $pedido['items'][0]);
										}
									}
								} else {
									if (!isset($pedidoMesa)) {
										$pedidoMesa = $pedido;
									} else {
										if (!in_array($pedido['items'][0], $pedidoMesa['items'])) {
											array_push($pedidoMesa['items'], $pedido['items'][0]);
										}
									}
								}
							}
						}
						if (isset($pedidoMesa)) {
							array_push($novosPedidos, $pedidoMesa);
						}
						if (isset($pedidoMesaSegura)) {
							array_push($novosPedidos, $pedidoMesaSegura);
						}
						unset($pedidoMesa);
						unset($pedidoMesaSegura);
					}
				}
			} else {
				foreach ($setor['NRMESA'] as $nrmesa => $mesa) {
					foreach ($mesa['CDSENHAPED'] as $key => $cdsenhaped) {
						foreach ($pedidos as $key => $pedido) {
							if ($pedido['CDSENHAPED'] == $cdsenhaped && $pedido['CDSETOR'] == $cdsetor && $pedido['NRMESA'] == $nrmesa) {
								array_push($novosPedidos, $pedido);
							}
						}
					}
				}
			}
		}

		return $novosPedidos;
	}

    function verificaHabAgrupProd($setores)
	{
		foreach ($setores['CDSETOR'] as $cdsetor => $setor) {
			$setores['CDSETOR'][$cdsetor]['IDHABAGRUPPROD'] = $this->KDSManagerService->verificaHabAgrupProd($cdsetor, $setores['CDFILIAL']);
		}
		return $setores;
	}

    function mergeSetorProdutoMesa($pedidos)
	{
		$setores['CDFILIAL'] = $pedidos[0]['CDFILIAL'];
		foreach ($pedidos as $key => $pedido) {
			if (!isset($setores['CDSETOR'])) {
				$setores['CDSETOR'][$pedido['CDSETOR']]['NRMESA'][$pedido['NRMESA']]['CDSENHAPED'] = array();
				array_push($setores['CDSETOR'][$pedido['CDSETOR']]['NRMESA'][$pedido['NRMESA']]['CDSENHAPED'], $pedido['CDSENHAPED']);
			}
			//Verifica se o setor ja existe, se nao existir ira montar a estrutura para aquele pedido
			if (!array_key_exists($pedido['CDSETOR'], $setores['CDSETOR'])) {
				$setores['CDSETOR'][$pedido['CDSETOR']]['NRMESA'][$pedido['NRMESA']]['CDSENHAPED'] = array();
				array_push($setores['CDSETOR'][$pedido['CDSETOR']]['NRMESA'][$pedido['NRMESA']]['CDSENHAPED'], $pedido['CDSENHAPED']);
			}
			//Verifica se a mesa ja existe, se nao existir ira montar a estrutura para aquele pedido dentro de um setor existente
			else if (!array_key_exists($pedido['NRMESA'], $setores['CDSETOR'][$pedido['CDSETOR']]['NRMESA'])) {
				$setores['CDSETOR'][$pedido['CDSETOR']]['NRMESA'][$pedido['NRMESA']]['CDSENHAPED'] = array();
				array_push($setores['CDSETOR'][$pedido['CDSETOR']]['NRMESA'][$pedido['NRMESA']]['CDSENHAPED'], $pedido['CDSENHAPED']);
			}
			//Verifica se o pedido ja existe, se nao existir ira inserir o pedido dentro de uma mesa existente
			else if (!in_array($pedido['CDSENHAPED'], $setores['CDSETOR'][$pedido['CDSETOR']]['NRMESA'][$pedido['NRMESA']]['CDSENHAPED'])) {
				array_push($setores['CDSETOR'][$pedido['CDSETOR']]['NRMESA'][$pedido['NRMESA']]['CDSENHAPED'], $pedido['CDSENHAPED']);
			}
		}
		return $setores;
	}

	function getLineProductionStatus(Request\Row $request, Response $response)
	{
		try {
			$params          = $request->getRow();
			$CDSETOR         = $params['CDSETOR'];
	
			$lineStatus = $this->environment->getSessionValue("lineStatus");
			if (!$lineStatus) {
				$this->setLineStatusOnSession($CDSETOR);
				$lineStatus = $this->environment->getSessionValue("lineStatus");
			}

			$response->addDataSet(new DataSet('checkLineProductionStatus', array('success' => true, 'lineStatus' => $lineStatus)));
		} catch (\Exception $e) {
			$response->addDataSet(new DataSet('checkLineProductionStatus', array('error' => true, 'message' => $e->getMessage())));
		}
	}

	function activateLineProduction(Request\Row $request, Response $response)
	{
		try {
			$params          = $request->getRow();
			$CDSETOR         = $params['CDSETOR'];
			$CDFILIAL        = $params['CDFILIAL'];

			//muda o status da linha atual (ativa/desativa)
			$updateLineStatus = $this->KDSManagerService->updateLineStatus($CDSETOR, $CDFILIAL);
			$this->setLineStatusOnSession($CDSETOR);
			
			$response->addDataSet(new DataSet('activateLine', array('success' => true, 'activateLine' => $updateLineStatus)));
		} catch (\Exception $e) {
			$response->addDataSet(new DataSet('activateLine', array('error' => true, 'message' => $e->getMessage())));
		}
	}

	function setLineStatusOnSession($CDSETOR) {
		$lineStatus = $this->KDSManagerService->getLineStatus($CDSETOR); 
		$this->environment->setSessionValue("lineStatus", $lineStatus);
	}

	function setLinePercentage(Request\Row $request, Response $response)
	{
		try {
			$params          = $request->getRow();
			$CDSETOR         = $params['CDSETOR'];
			$CDFILIAL         = $params['CDFILIAL'];
			$PERCENT         = $params['PERCENT'];
			$correlationatedSectorsCode = $this->KDSManagerService->getCorrelationatedSectorsCode($CDFILIAL, $CDSETOR);
			$countCorrelationated = sizeof($correlationatedSectorsCode);

			//atualiza a porcentagem da linha de produção dos setores correspondentes
			if($countCorrelationated > 0){
				$correlationatedPercent = (100 - $PERCENT)/$countCorrelationated;
				foreach($correlationatedSectorsCode as $correlationated){
					$this->KDSManagerService->setLinePercentage($correlationated['CDSETOR'], $CDFILIAL, $correlationatedPercent);
				}
			}
			
			//atualiza a porcentagem da linha de produção do setor por onde foi feita a requisição
			$setPercentage= $this->KDSManagerService->setLinePercentage( $CDSETOR, $CDFILIAL, $PERCENT );

			$response->addDataSet(new DataSet('setPercentage', array('success' => true, 'setPercentage' => $setPercentage)));
		} catch (\Exception $e) {
			$response->addDataSet(new DataSet('setPercentage', array('error' => true, 'message' => $e->getMessage())));
		}
	}

	function activateOverallStatistics(Request\Row $request, Response $response)
	{
		try {
			$params          = $request->getRow();
			$CDSETOR         = $params['CDSETOR'];
			$CDFILIAL        = $params['CDFILIAL'];
			$activatestatistics = $this->statisticsService->activateOverallStatistics($CDFILIAL, $CDSETOR);

			$response->addDataSet(new DataSet('activatestatistics', array('success' => true, 'activatestatistics' => $activatestatistics)));
		} catch (\Exception $e) {
			$response->addDataSet(new DataSet('activatestatistics', array('error' => true, 'message' => $e->getMessage())));
		}
	}



	function filterExpedition($item)
	{
		$result = $item['NRPEDIDOFOS'] != $this->oldOrder;
		$this->oldOrder = $item['NRPEDIDOFOS'];
		return $result;
	}

	function filterItems($item)
	{
		return $item['NRPEDIDOFOS'] == $this->order;
	}

	public function buildDataSourceExpedition($CDSETOR = null)
	{

		$returnArray = array();

		$returnToSector = $CDSETOR !== null;

		$sectorLists = $this->KDSManagerService->getExpeditionAndMountListFromItPedidofos();

		$resultArray = [];

		// $start = microtime(true);
		
		// // echo 'Início do Build Datasource Expedition'."\n";;
		// // echo '-------------------------------------------------'."\n";
		for($sId=0; $sId<sizeof($sectorLists); $sId++){
            
			$currentSector = $sectorLists[$sId];

            /**
             * Esta mesma função é chamada a partir de checkForUpdate e também quando é feito login/refresh em algum setor específico.
             * Quando a chamada vem de checkForUpdate, ela precisa verificar todos os setores em busca de alterações, mas quando
             * é feito login/refresh em algum setor, só precisa ser verificado aquele setor específico.
             * 
             * $CDSETOR === null indica que a chamada veio de checkForUpdate, nesse caso o código dentro do if deve ser executado
             * em todas as iterações do loop.
             * 
             * Caso $CDSETOR !== null, o código dentro do if deve ser executado apenas para o setor específico que foi passado por
             * parâmetro na chamada dessa função.
             */
            if ($CDSETOR === null || ($CDSETOR !== null && $currentSector['CDSETOR'] === $CDSETOR)){

                $expeditionArray = $this->KDSManagerService->getExpedition($currentSector['CDSETOR'], $returnToSector);
                $count = 0;
                $keyExpedition = 0;
                $lastIdx = 0;
                if (!empty($expeditionArray)) {
                    while ($keyExpedition < sizeof($expeditionArray)) {
                        $expedition = $expeditionArray[$keyExpedition];
                        $trueItemsExp = array();
                        $trueItemsMont = array();
                        // $this->order = $expedition['NRPEDIDOFOS'];
    
                        $itemsExpedition = array();
                        array_push($itemsExpedition, $expeditionArray[$keyExpedition]);
                        $lastIdx = $keyExpedition + 1;
                        if (isset($expeditionArray[$lastIdx])) {
                            $expeditionArray[$lastIdx]['orderCode'] = $expeditionArray[$lastIdx]['CDFILIAL'] . $expeditionArray[$lastIdx]['NRPEDIDOFOS'] . $expeditionArray[$lastIdx]['NRSEQLIBERADO'];
                        }
                        $expedition['orderCode'] = $expedition['CDFILIAL'] . $expedition['NRPEDIDOFOS'] . $expedition['NRSEQLIBERADO'];
    
                        while (isset($expeditionArray[$lastIdx]) && $expedition['orderCode'] == $expeditionArray[$lastIdx]['orderCode']) {
                            array_push($itemsExpedition,  $expeditionArray[$lastIdx]);
                            //unset($expeditionArray[$lastIdx]);
                            $lastIdx++;
                            if (isset($expeditionArray[$lastIdx])) {
                                $expeditionArray[$lastIdx]['orderCode'] = $expeditionArray[$lastIdx]['CDFILIAL'] . $expeditionArray[$lastIdx]['NRPEDIDOFOS'] . $expeditionArray[$lastIdx]['NRSEQLIBERADO'];
                            }
                        }
                        $keyExpedition = $lastIdx;
    
                        //nunca vai estar empty mas eu gosto deste bloco
                        if (!empty($itemsExpedition)) {
    
                            // se for venda balcão, o número do pedido será o mesmo impresso no cupom fiscal
                            $expedition['orderCode'] = $expedition['CDFILIAL'] . $expedition['NRPEDIDOFOS'];
                            $expedition['label'] = $expedition['CDSENHAPED'];
    
                            // retira os zeros do número da mesa
                            $expedition['NRMESA'] = intval($expedition['NRMESA']);
    
                            // retira os zeros do número da mesa antiga
                            $expedition['NRMESAOLD'] = intval($expedition['NRMESAOLD']);
    
                            // coloca o atraso
                            $expedition['DELAYTIME'] = $itemsExpedition[0]['DELAYTIME'];
    
                            // para tratar ordenação dos pedidos no frontend
                            $expedition['DTHREXIBICAOPRODISO'] = $itemsExpedition[0]['DTHREXIBICAOPRODISO'];
                            $expedition['DTHREXIBICAOPROD'] = $itemsExpedition[0]['DTHREXIBICAOPROD'];
                            $expedition['NRSEQLIBERADO'] = $itemsExpedition[0]['NRSEQLIBERADO'];
    
                            // verifica se todos os itens do pedido estão cancelados
                            $canCancel = 0;
                            // verifica se todos os itens do pedido estão cancelados
                            $doneItems = 0;
    
                            // maior tempo do pedido
                            $biggestTime = 0;
							$isPriority = true;
                            for($ii=0;$ii<sizeof($itemsExpedition);$ii++){
                            //foreach ($itemsExpedition as $key => $item) {
                                $key = $ii;
                                $item = $itemsExpedition[$ii];
    
                                if ($key == 0) {
                                    $expedition['orderKey'] = $item['CDFILIAL'] . '-' . $item['NRPEDIDOFOS'] . $item['NRITPEDIDOFOS'];
                                }
    
                                // trata cancelamento na venda mesa, comanda ou delivery
                                if (!empty($item['NRVENDAREST'])) {
                                    // se o item não está cancelado
                                    if ($item['IDSITITPEDFOS'] != 'C') {
                                        // relação com ITCOMANDAVEN
                                        $itemsRelations = $this->KDSManagerService->getItemsRelations($item['CDFILIAL'], $item['NRPEDIDOFOS'], $item['NRITPEDIDOFOS']);
    
                                        $newQuantity = 0;
                                        // for($i=0;$i<sizeof($itemsRelations);$i++){
                                        // 	$relation = $itemsRelations[$i];
                                        foreach ($itemsRelations as $relation) {
                                            if ($relation['IDSITITPEDFOS'] != 'C') {
                                                $newQuantity = $newQuantity + $relation['QTPRODPEFOS'];
                                            }
                                        }
                                        // se a nova quantidade for igual a zero, cancela o item pai (na ITPEDIDOFOS)
                                        if ($newQuantity <= 0) {
                                            $item['IDSITITPEDFOS'] = 'C';
                                            $this->KDSManagerService->cancelItem($item['CDFILIAL'], $item['NRPEDIDOFOS'], $item['NRITPEDIDOFOS']);
    
                                            /*if ($item['IDLIBERADO'] == 'N') {
                                                // expede se não tiver liberado para impedir que ele apareça na produção
                                                $item['IDSITITPEDFOS'] = 'X';
                                                $this->KDSManagerService->changeItemStatusReleased($item['CDFILIAL'], $item['NRPEDIDOFOS'], $item['NRITPEDIDOFOS']);
                                            }*/
                                        } else if ($newQuantity != $item['QTPRODPEFOS']) {
                                            // se a quantidade mudou
                                            // se um ou mais itens (não todos) foram cancelados, updata a nova quantidade
                                            $item['QTPRODPEFOS'] = $newQuantity;
                                            $this->KDSManagerService->changeItemQuantity($item['CDFILIAL'], $item['NRPEDIDOFOS'],  $item['NRITPEDIDOFOS'], $item['QTPRODPEFOS']);
                                        }
                                    }
                                }
    
                                //Solução Temporária do Combo
                                if ($this->util->getParameter('comboAlias') != '') {
                                    $item['NMPROMOCAO'] = preg_replace('/COMBO.*/', $this->util->getParameter('ComboAlias'), $item['NMPROMOCAO']);
                                }
    
                                // X - EXPEDIDO
                                // verifica se todos os itens filhos desse item já estão prontos
                                if ($item['IDSITITPEDFOS'] != 'X') {
                                    $item = $this->updateExpeditionStatus($item['CDFILIAL'], $item);
                                }
    
                                // se item está cancelado
                                if ($item['IDSITITPEDFOS'] == 'C') {
                                    $canCancel++;
                                }
                                // se item está cancelado
                                if ($item['IDSITITPEDFOS'] == 'C' || $item['IDSITITPEDFOS'] == 'F' || $item['IDSITITPEDFOS'] == 'X') {
                                    $doneItems++;
                                }
    
    
                                if ($item['IDSITITPEDFOS'] != 'X') {
                                    if ($item['IDTIPOSETOR'] == 'E') {
                                        // se tipo setor é expedição
                                        //if($item['CDSETOR'] == $expedition['CDSETOR']){
                                        array_push($trueItemsExp, $item);
                                        //}
                                    } else if ($item['IDTIPOSETOR'] == 'M') {
                                        // se tipo setor é montagem
                                        if ($item['CDSETOR'] == $expedition['CDSETOR']) {
                                            array_push($trueItemsMont, $item);
                                        }
                                    }
                                }
    
                                if ($item['TEMPOCORES'] > $biggestTime) {
                                    $biggestTime = $item['TEMPOCORES'];
                                }
								if ((!$item['IDPRODPRIORIDADE'] || $item['IDPRODPRIORIDADE'] == 'N') && $isPriority == true) {
									// Verificar se o CDPRODUTO é prioridade
									$isPriority = false;
								}
                            }
							
							$expedition['IDPRODPRIORIDADE'] = $isPriority;
    
                            if ($canCancel == count($itemsExpedition)) {
                                // se todos os itens do pedido estão cancelados
                                $expedition['IDSITITPEDFOS'] = "C";
    
                                /*if ($expedition['CDSENHAPED'] == '') {
                                    $expedition['label'] = $expedition['orderCode'];
                                }*/
                            } else if ($doneItems == count($itemsExpedition)) {
                                // se todos os itens do pedido estão prontos
                                $expedition['IDSITITPEDFOS'] = "F";
                            } else {
                                $expedition['IDSITITPEDFOS'] = "E";
                            }
    
                            $expedition['IDTIPOSETOR'] = $item['IDTIPOSETOR'];
                            $expedition['TEMPOCORES'] = $biggestTime;
                            //$expedition['NRTEMPOPROD'] = $biggestTime;
                            if ($expedition['IDORIGEMVENDA']) {
								// PROCURE AQUI
								$expedition['ORIGEM'] = $this->getOrigins->manageOrigin($expedition['IDORIGEMVENDA'], "system", "origin");
								if (isset($expedition['IDRETBALLOJA']) && strpos($expedition['ORIGEM'], 'DELIVERY') !== false) {
									$expedition['ORIGEM'] = $expedition['ORIGEM'] . $this->getOrigins->getToGo($expedition['IDRETBALLOJA']);
								}
                                // // echo 'Pega a origem do item: '.(microtime(true) - // $start).'s'."\n";

                            }

                            if (!empty($trueItemsExp)) {
                                // substitui o setor do pedido pelo setor do produto (gambiarra pra função de tempo funcionar corretamente)
                                $expedition['CDSETOR'] = $trueItemsExp[0]['CDSETOR'];
                                $expedition['items'] = $trueItemsExp;
                                array_push($returnArray, $expedition);
                            }
                            if (!empty($trueItemsMont)) {
                                // substitui o setor do pedido pelo setor do produto (gambiarra pra função de tempo funcionar corretamente)
                                $expedition['CDSETOR'] = $trueItemsMont[0]['CDSETOR'];
                                $expedition['items'] = $trueItemsMont;
                                array_push($returnArray, $expedition);
                            }
                        }
                    }
                }
                $resultArray = array_merge($resultArray, $returnArray);
            }
		}

		// $timer->end();
		// // echo '-------------------------------------------------'."\n";
		// // echo 'Fim do Build Datasource Expedition: '.(microtime(true) - // $start).'s'."\n";

		return $resultArray;
	}

	public function updateExpeditionStatus($CDFILIAL, $item)
	{
		// busca todos os sub-itens do produto pai
		$childProducts = $this->KDSManagerService->getChildProducts($CDFILIAL, $item['NRPEDIDOFOS'], $item['NRITPEDIDOFOS']);
		// busca todos os itens filhos desse pai que estão prontos
		$doneChildProducts = $this->KDSManagerService->checkChildDone($CDFILIAL, $item['NRPEDIDOFOS'], $item['NRITPEDIDOFOS'], $item['IDTIPOSETOR']);
		// busca todos os itens filhos desse pai que estão em produção
		$childProduction = $this->KDSManagerService->checkChildProduction($CDFILIAL, $item['NRPEDIDOFOS'], $item['NRITPEDIDOFOS']);
		$IDSITITPEDFOS = $item['IDSITITPEDFOS'];

		$NRPEDIDOFOS = $item['NRPEDIDOFOS'];

		$canSetHowDone = NULL;

		if ($item['IDTIPOSETOR'] == 'M') {
			$canSetHowDone = $this->KDSManagerService->checkIfCanDoneMount($NRPEDIDOFOS);
		}

		// verifica se item pai está cancelado (se esse if for removido, o sistema estrará em loop eterno)
		if ($IDSITITPEDFOS == 'C') {
			$item['IDSITITPEDFOS'] = 'C';

			// item não tem subprodutos já entra como feito na expedição
		} else if ((empty($childProducts)) && ($IDSITITPEDFOS != 'F')) {

			if ($item['IDTIPOSETOR'] != 'M' || ($canSetHowDone != NULL && $item['CDSETOR'] == $canSetHowDone[0]['CDSETOR'])) {
				$IDSITITPEDFOS = 'F';
				$this->KDSManagerService->changeStatusItemRequestConclude($CDFILIAL, $item['NRPEDIDOFOS'], $item['NRITPEDIDOFOS']);
			}


			// filhos concluídos, entra como feito
		} else if ((count($doneChildProducts) == count($childProducts)) && ($IDSITITPEDFOS != 'F')) {
			if (!empty($doneChildProducts)) {
				if ($item['IDTIPOSETOR'] != 'M' || ($canSetHowDone != NULL && $item['CDSETOR'] == $canSetHowDone[0]['CDSETOR'])) {
					$IDSITITPEDFOS = 'F';
					$this->KDSManagerService->changeStatusItemRequestConclude($CDFILIAL, $item['NRPEDIDOFOS'], $item['NRITPEDIDOFOS']);
				}
			}

			// produto em produção, mostra a engrenagem
		} else if ($IDSITITPEDFOS != 'P' && $IDSITITPEDFOS != 'F') {
			if (!empty($childProduction) || !empty($doneChildProducts)) {
				$IDSITITPEDFOS = 'P';
				$this->KDSManagerService->changeStatusItemRequestPlay($CDFILIAL, $item['NRPEDIDOFOS'], $item['NRITPEDIDOFOS']);
			}
		}

		$item['IDSITITPEDFOS'] = $IDSITITPEDFOS;
		if ($item['IDSITITPEDFOS'] == 'F' || $item['IDSITITPEDFOS'] == 'C' || $item['IDSITITPEDFOS'] == 'X') {
			$item['PRODUCTED'] = true;
		} else {
			$item['PRODUCTED'] = false;
		}
		return $item;
	}

	/* Comentado por não ser utilizado 
	public function updateTimeByObservationExpedition($item, $observations) {
		$detailObservation = self::getDetailObservation($observations);
		$maxTime = $detailObservation['maxtime']; // Maior tempo encontrado no array de observação
		$DSOCORR = $detailObservation['DSOCORR']; // String com todas as observações do pedido
		$item['DSOCORR'] = $DSOCORR;
		$item['NRTEMPOPROD'] = $maxTime;
		return $item;
	} */

	private function handleComposition($compositionArray)
	{
		foreach ($compositionArray as $currentComposition) {

			$this->fillComposition(
				$currentComposition['CDFILIAL'],
				$currentComposition['CDLOJA'],
				$currentComposition['CDPRODUTO'],
				$currentComposition['QTPRODPEFOS'],
				$currentComposition['NRPEDIDOFOS'],
				$currentComposition['CDCAIXA'],
				$currentComposition['NRSEQVENDA'],
				$currentComposition['NRSEQUITVEND'],
				$currentComposition['NRVENDAREST'],
				$currentComposition['NRCOMANDA'],
				$currentComposition['NRPRODCOMVEN'],
				$currentComposition['TXPRODPED'],
				$currentComposition['IDSITITPEDFOS'],
				$currentComposition['DSCHAVEPEDPAI'],
				$currentComposition['DTPEDIDOFOS'],
				$currentComposition['NRITPEDIDOFOS'],
				$currentComposition['CHAVEPEDIDO'],
				$currentComposition['CDOCORR'],
				$currentComposition['CDGRPOCOR'],
				$currentComposition['ARRAYPAI'],
				$currentComposition['NRATRAPROD'],
				$currentComposition['CHAVEITEM'],
				$currentComposition['RELITCOMANDAVEN'],
				$currentComposition['OBSDIGITADA'],
				$currentComposition['NRLUGARMESAIT'],
				$currentComposition['IDORIGEMVENDA'],
				$currentComposition['IDITORIGEMVENDA'],
				$currentComposition['NRTEMPOPROD'],
				$currentComposition['NRTEMPOEXIB'],
				$currentComposition['TXPRODCOMVENEST'],
				$currentComposition['DSTIPOPEDFOS'],
				$currentComposition['NRMESA'],
				$currentComposition['NRSEQITVENDAEST'],
				$currentComposition['NRSEQCOMB']
			);
		}
	}

	public function fillComposition(
		$CDFILIAL,
		$CDLOJA,
		$CDPRODUTOPAI,
		$QTPRODPEFOS,
		$NRPEDIDOFOS,
		$CDCAIXA,
		$NRSEQVENDA,
		$NRSEQUITVEND,
		$NRVENDAREST,
		$NRCOMANDA,
		$NRPRODCOMVEN,
		$TXPRODPED,
		$IDSITITPEDFOS,
		$DSCHAVEPEDPAI,
		$DTPEDIDOFOS,
		$NRITPEDIDOFOSPAI,
		$CHAVEPEDIDO,
		$CDOCORR,
		$CDGRPOCOR,
		$ARRAYPAIS,
		$NRATRAPROD,
		$CHAVEITEM,
		$RELITCOMANDAVEN,
		$OBSDIGITADA,
		$NRLUGARMESAIT,
		$IDORIGEMVENDA,
		$IDITORIGEMVENDA,
		$NRTEMPOPRODPAI, 
		$NRTEMPOEXIBPAI,
		$TXPRODCOMVENEST = NULL,
		$DSTIPOPEDFOS = NULL,
		$NRMESA = NULL,
		$NRSEQITVENDAEST = NULL,
		$NRSEQCOMB = NULL
	) {

		// busca composição local
		$composition = $this->KDSManagerService->searchLocalComposition($CDFILIAL, $CDPRODUTOPAI, $CDLOJA);
		// se não tiver composição local, procura a padrão
		if (empty($composition)) {
			$composition = $this->KDSManagerService->searchPatternComposition($CDPRODUTOPAI, $CDFILIAL, $CDLOJA);
		}

		// se tiver composição local ou padrão
		if (!empty($composition)) {
			$DSCHAVEPEDPAI = $CDFILIAL . $NRPEDIDOFOS . $NRITPEDIDOFOSPAI;

			//correlationatedSectors comentado por não ser utilizado
			//$correlationatedSectors = $this->KDSManagerService->getCorrelationatedSectors($CDFILIAL);
			

			foreach ($composition as $prodComposition) {
				if ($this->validateProduct($prodComposition['CDSUBPRODUTO'])) {

					$CDSUBPRODUTO  = $prodComposition['CDSUBPRODUTO'];
					$DSAPELIDOKDS = null;

					if (!empty($prodComposition['DSAPELIDOKDS'])) {
						$DSAPELIDOKDS = $prodComposition['DSAPELIDOKDS'];
					}
					if ($CDPRODUTOPAI != $CDSUBPRODUTO) { // somente de subproduto for diferente do produto de origem

						$QTDSUBPRD = $prodComposition['QUANTIDADE'];
						$QTDSUBPRD = ($QTDSUBPRD * $QTPRODPEFOS);
						$QTDSUBPRD = number_format($QTDSUBPRD, 3);

						// quantidade usada na ITPEDIDOFOSREL
						$QTCOMPOSICAO = $prodComposition['QUANTIDADE'];

						$productSectors = $this->KDSManagerService->getProductSector($CDSUBPRODUTO, $CDFILIAL, $CDLOJA);

						// distribuição de itens de composição em setores de produção - desativado
						// $productSectors = $this->distributeItems($CDFILIAL, $productSectors);
						//Redirecionemento do pedido por Origem da venda
						$productSectors = $this->KDSManagerService->redirectOrderSector($CDFILIAL, $productSectors, $IDORIGEMVENDA);

						$productSectors = array_values(
							array_filter($productSectors, function($productSector) {
								return $productSector['CDLINHAPROD'] != '2';
							})
						);
						
							foreach ($productSectors as $currentSector) {
									
									$TXPRODPED = null;
									if($currentSector['IDSOMRESUMO'] == 'S'){
										$NRTEMPOPROD = $NRTEMPOPRODPAI;
										$NRTEMPOEXIB = $NRTEMPOEXIBPAI;
									} else {
										$NRTEMPOPROD = $currentSector['NRTEMPOPROD'];
										$NRTEMPOEXIB = $currentSector['NRTEMPOEXIB'];
									}
									$CDSETOR = $currentSector['CDSETOR'];
									if (is_array($CHAVEITEM)) {
										$key = $CHAVEITEM[0]['CHAVEITEM'];
									} else {
										$key = $CHAVEITEM;
									}
	
									// buildTxProd aqui
									$itemBuildTxProd = [
										'TXPRODPED' => $TXPRODPED,
										'NRTEMPOPROD' => $NRTEMPOPROD,
										'NRTEMPOEXIB' => $NRTEMPOEXIB,
										'TXPRODCOMVENEST' => $DSTIPOPEDFOS == 'RODIZIO' ? $TXPRODCOMVENEST : NULL,
										'VENDA' => $DSTIPOPEDFOS
									];

									// OBS: KEY = CHAVEITEM | CDPRODUTOPAI = CDPRODUTO 
									$TXPRODPED = $this->buildTxProd($key, $CDSETOR, $CDSUBPRODUTO, $OBSDIGITADA, $itemBuildTxProd);

									if (!empty($TXPRODPED)) {
										// soma o tempo de produção das observações com o tempo de produção do item
										$TIMEPRODUCTION = $this->calcTimeObs($NRTEMPOPROD, $NRTEMPOEXIB, $TXPRODPED);
										$NRTEMPOPROD = $TIMEPRODUCTION['NRTEMPOPROD'];
										$NRTEMPOEXIB = $TIMEPRODUCTION['NRTEMPOEXIB'];
									}

									//Nova implementação para viagem
									if (!is_null($IDITORIGEMVENDA) && strpos(' ' . $IDITORIGEMVENDA, 'TGO') > 0) {
										$NRCORVGM = $currentSector['NRCORVGM'];
										if (!isset($TXPRODPED)) {
											$TXPRODPED = array();
										}
										$has = false;
										foreach ($TXPRODPED as $key => $value) {
											if (strpos('  ' . $value['OBSERVACAO'], 'VIAGEM') != '') {
												//REMOVE OBSERVAÇÃO;
												$TXPRODPED[$key]['NRCORSINAL'] = $NRCORVGM;
												$has = true;
											}
										}
	
										if ($has == false) {
											array_push($TXPRODPED, array(
												'OBSERVACAO' => 'VIAGEM',
												'NRCORSINAL' => $NRCORVGM
											));
										}
									}
	
									// substitui a quantidade da composição (na ITPEDIDOFOSREL)
									$RELITCOMANDAVENcomp = $this->changeRelationsQuantity($RELITCOMANDAVEN, $QTCOMPOSICAO);
	
									if (!$this->groupCompositionFromDifferentProducts(
										$CDFILIAL,
										$NRPEDIDOFOS,
										$CDSUBPRODUTO,
										$DSAPELIDOKDS,
										$TXPRODPED,
										$QTDSUBPRD,
										$NRATRAPROD,
										$ARRAYPAIS,
										$RELITCOMANDAVENcomp,
										$CDSETOR,
										$CDPRODUTOPAI,
										$NRLUGARMESAIT
									)) {	
	
										// insere novo item no array
										$NRITPEDIDOFOS = $this->generateNewNRITPEDIDOFOS($CDFILIAL, $NRPEDIDOFOS);
										if ($currentSector['IDTIPOSETOR'] == 'P') {
											array_push($this->itemsForInsertion, array(
												'CDFILIAL'        => $CDFILIAL,
												'NRPEDIDOFOS'     => $NRPEDIDOFOS,
												'NRITPEDIDOFOS'   => $NRITPEDIDOFOS,
												'CDPRODUTO' 	  => $CDSUBPRODUTO,
												'QTPRODPEFOS' 	  => $QTDSUBPRD,
												'CDCAIXA' 		  => $CDCAIXA,
												'NRSEQVENDA' 	  => $NRSEQVENDA,
												'NRSEQUITVEND'    => $NRSEQUITVEND,
												'NRVENDAREST' 	  => $NRVENDAREST,
												'NRCOMANDA' 	  => $NRCOMANDA,
												'NRPRODCOMVEN'    => $NRPRODCOMVEN,
												'TXPRODPED' 	  => $TXPRODPED,
												'IDSITITPEDFOS'   => $IDSITITPEDFOS,
												'CDSETOR'         => $CDSETOR,
												'CDGRPOCOR'       => $CDGRPOCOR,
												'CDOCORR'         => $CDOCORR,
												'DSCHAVEPEDPAI'   => $DSCHAVEPEDPAI,
												'CHAVEPEDIDO'     => $CHAVEPEDIDO,
												'DTPEDIDOFOS'     => $DTPEDIDOFOS,
												'CDPRODUTOPAI'    => $CDPRODUTOPAI,
												'IDTIPOSETOR'     => $currentSector['IDTIPOSETOR'],
												'CDPRODPROMOKDS'  => null,
												'NRSEQPRODCOMKDS' => null,
												'NRTEMPOPROD'     => $NRTEMPOPROD,
												'NRTEMPOEXIB'     => $NRTEMPOEXIB,
												'NRATRAPROD'      => $NRATRAPROD,
												'DSAPELIDOKDS'    => $DSAPELIDOKDS,
												'RELITCOMANDAVEN' => $RELITCOMANDAVENcomp,
												'NRLUGARMESAIT'   => $NRLUGARMESAIT,
												'IDORIGEMVENDA'   => $IDORIGEMVENDA,
												'NRMESA' 		  => $NRMESA,
												'NRSEQITVENDAEST' => $NRSEQITVENDAEST,
												'NRSEQCOMB' 	  => $NRSEQCOMB
											));
										}
	
										// insere uma linha para cada pai dessa composição na ITPEDIDOFOSPAI
										// uma composição pode ter vários pais, pois será agrupada com outros itens
										foreach ($ARRAYPAIS as $currentPai) {
											$NRSEQITPEDPAI = $this->generateNewNRSEQITPEDPAI($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS);
											array_push($this->itemsRelations, array(
												'CDFILIAL'         => $CDFILIAL,
												'NRPEDIDOFOS'      => $NRPEDIDOFOS,
												'NRITPEDIDOFOS'    => $NRITPEDIDOFOS,
												'NRSEQITPEDPAI'    => $NRSEQITPEDPAI,
												'CDFILIALPAI'      => $currentPai['CDFILIAL'],
												'NRPEDIDOFOSPAI'   => $currentPai['NRPEDIDOFOS'],
												'NRITPEDIDOFOSPAI' => $currentPai['NRITPEDIDOFOS']
											));
										}
									}
								//}
							}

						}
					}
				}
			}
		}
	

	private function changeRelationsQuantity($RELITCOMANDAVEN, $QTDSUBPRD)
	{
		$RELITCOMANDAVENcomp = array();
		foreach ($RELITCOMANDAVEN as $currentRelation) {
			$currentRelation['QTPRODPEFOS'] = $QTDSUBPRD;
			array_push($RELITCOMANDAVENcomp, $currentRelation);
		}
		return $RELITCOMANDAVENcomp;
	}

	private function groupCompositionFromDifferentProducts(
		$CDFILIAL,
		$NRPEDIDOFOS,
		$CDSUBPRODUTO,
		$DSAPELIDOKDS,
		$TXPRODPED,
		$QTPRODPEFOS,
		$NRATRAPROD,
		$ARRAYPAIS,
		$RELITCOMANDAVEN,
		$NRLUGARMESAIT
	) {
		foreach ($this->itemsForInsertion as &$currentItem) {
			if (($CDFILIAL == $currentItem['CDFILIAL']) && ($NRPEDIDOFOS == $currentItem['NRPEDIDOFOS']) &&
				($CDSUBPRODUTO == $currentItem['CDPRODUTO']) && ($TXPRODPED == $currentItem['TXPRODPED']) &&
				($DSAPELIDOKDS == $currentItem['DSAPELIDOKDS']) &&
				($NRATRAPROD == $currentItem['NRATRAPROD']) && ($NRLUGARMESAIT == $currentItem['NRLUGARMESAIT'])
			) { 

                $currentItem['QTPRODPEFOS'] += $QTPRODPEFOS;
                //$RELITCOMANDAVENcomp = $this->changeRelationsQuantity($RELITCOMANDAVEN, $QTPRODPEFOS);
                $currentItem['RELITCOMANDAVEN'] = array_merge($currentItem['RELITCOMANDAVEN'], $RELITCOMANDAVEN);

                // insere uma linha para cada pai dessa composição na ITPEDIDOFOSPAI
                // uma composição pode ter vários pais, pois será agrupada com outros itens
                foreach ($ARRAYPAIS as $currentPai) {
                    $NRSEQITPEDPAI = $this->generateNewNRSEQITPEDPAI($currentItem['CDFILIAL'], $currentItem['NRPEDIDOFOS'], $currentItem['NRITPEDIDOFOS']);
                    array_push($this->itemsRelations, array(
                        'CDFILIAL'         => $currentItem['CDFILIAL'],
                        'NRPEDIDOFOS'      => $currentItem['NRPEDIDOFOS'],
                        'NRITPEDIDOFOS'    => $currentItem['NRITPEDIDOFOS'],
                        'NRSEQITPEDPAI'    => $NRSEQITPEDPAI,
                        'CDFILIALPAI'      => $currentPai['CDFILIAL'],
                        'NRPEDIDOFOSPAI'   => $currentPai['NRPEDIDOFOS'],
                        'NRITPEDIDOFOSPAI' => $currentPai['NRITPEDIDOFOS']
                    ));
                }
				return true;
			}
		}
		return false;
	}

	public function concludeRequest(Request\Filter $request, Response $response)
	{
		try {
			$params = $request->getFilterCriteria()->getConditions();

			$CDFILIAL      = $params[0]['value'];
			$NRPEDIDOFOS   = $params[1]['value'];
			$NRITPEDIDOFOS = $params[2]['value'];
			$CDLOJA        = $params[3]['value'];
			$CDSETOR       = $params[4]['value'];

			//F de feito
			$IDSITITPEDFOS = 'F';
			$this->KDSManagerService->changeStatusItemRequestConclude($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS, $IDSITITPEDFOS);
			$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('RequestsRepo', array()));
		} catch (\Exception $e) {
			$this->util->logException($e->getMessage());
			$response->addMessage(new Message($e->getMessage()));
		}
	}

	public function getTimersForBoxes($pedidos)
	{
		foreach ($pedidos as &$pedido) {
			$item = $pedido['items'][0];

			$data = $item['DTHREXIBICAOPROD'];
			if (($pedido['IDTIPOSETOR'] == 'E') || ($pedido['IDTIPOSETOR'] == 'M')) {
				// IDTIPOSETOR = E
				// setor de expedição, cada caixa tem N itens
				// busca o maior tempo de produção do pedido
				$maxTime = $this->KDSManagerService->getMaxProductionTime($pedido['CDFILIAL'], $pedido['NRPEDIDOFOS'], ($pedido['NRSEQLIBERADO'] != ''));



				$NRTEMPOPROD = $maxTime[0]['TEMPO'];
			} else {
				// IDTIPOSETOR = P
				// setor de produção, cada caixa tem somente um item
				$NRTEMPOPROD = $item['NRTEMPOPROD'];
			}
			$pedido['NRTEMPOPROD'] = $NRTEMPOPROD;

			// coloca o timer no pedido
			$now = new \Datetime($item['NOW'], new \DateTimeZone(date_default_timezone_get()));
			$data = new \Datetime($data, new \DateTimeZone(date_default_timezone_get()));

			if ($now < $data) {
				$timer = 'wait';
				if (!isset($pedido['NRATRAPRODITPE']) || $pedido['NRATRAPRODITPE'] == 0) {
					$pedido['waiting'] = true;
				} else {
					$pedido['waiting'] = false;
				}
				$pedido['IDLIBERADO'] = "N";
			} else {
				//libera o treco.
				$pedido['IDLIBERADO'] = "S";

				$timer = $now->diff($data);
				$timer = $timer->format('1/1/1 %H:%i:%s');
				$pedido['waiting'] = false;
			}
			$pedido['timer'] = $timer;
		}

		return $pedidos;
	}

	public function getColors(Request $request, Response $response)
	{
		try {
			$colorsParams = $this->KDSManagerService->getColourParams();
			$response->addDataSet(new DataSet('getColorsReponse', array('success' => true, 'colors' => $colorsParams)));
		} catch (\Exception $e) {
			$this->util->logException($e->getMessage());
			$response->addDataSet(new DataSet('getColorsReponse', array('error' => true, 'message' => $e->getMessage())));
		}
	}

	public function generateITPEDIDOFOS(Request $request, Response $response)
	{
		$inputJSON = file_get_contents('php://input');
		$input = json_decode($inputJSON, TRUE);
		$this->util->beginTransaction();
		try {
			if (empty($input)) {
				throw new \Exception('Não foram recebidos parâmetros');
			}

			//CHAVE, Chave unica do pedido a ser gerado.  VE.CDFILIAL + VE.CDCAIXA + VE.NRSEQVENDA  se for venda, VE.CDFILIAL + IT.NRSEQPRODCUP se for pedido
			//VENDA, Identificador de onde veio a venda. 'ODHENAPI' caso gerado pela api, VENDA caso balcão, COMANDA caso pedido
			//CDFILIAL, Filial relacionada ao item
			//NRPEDIDO, Chave unica do pedido a ser gerado. pode ser a CHAVE
			//DATA, data do pedido. "agora" geralmente
			//NOW, agora, mesmo
			//CDPRODUTO, codigo do produto
			//QTDEPROD, quantidade
			//CDCAIXA, caso VENDA
			//NRSEQVENDA, caso VENDA
			//NRMESA do pedido, caso haja
			//NRCOMANDAVND, NRCOMANDA caso venda.  NRCOMANDA caso pedido
			//DSCOMANDAVND, DSCOMANDAVND caso venda. DSCOMANDA caso pedido
			//NRSEQUITVEND, caso venda
			//NRCOMANDA, caso pedido. retundancia a gente ve por aqui
			//DSCOMANDA, caso pedido
			//NRVENDAREST, caso pedido
			//NRPRODCOMVEN, caso pedido
			//QTMAXPEDFOS, não sei o que é isso @lua
			//CDLOJA, codigo da loja do pedido
			//OBSDIGITADA, obeservação personalizada do pedido. DSOBSPEDDIGITA caso venda. DSOBSPEDDIGCMD caso pedido.
			//CDSENHAPED, codigo do pedido na tela do kds.
			//NMCONSUMIDOR, nome do consumidor do pedido
			//CDGRPOCOR, caso o item seja uma observação ?
			//CDOCORR, caso o item seja uma observação ?
			//CHAVEPEDIDO, CHAVE? DE NOVO!?
			//IDORIPED, K! (O kds do fos insere F)
			//NRPEDIDOFOS, null
			//NRITPEDIDOFOS, null
			//NRSEQPRODCOM, NRSEQPRODCOMIT caso venda, NRSEQPRODCOM caso pedido
			//CDPRODPROMOCAO,
			//CHAVEITEM, chave unica do item a ser inserido. IT.CDFILIAL + IT.CDCAIXA + IT.NRSEQVENDA + IT.NRSEQUITVEND caso venda,IT.CDFILIAL + IT.NRVENDAREST + IT.NRCOMANDA + IT.NRPRODCOMVEN caso pedido
			//NRATRAPROD, tempo de atraso no item, famigerado "SEGURA". NRATRAPRODITVE caso venda, NRATRAPRODCOVE caso pedido
			//NRLUGARMESA, posição na mesa


			$items = $input['items'];
			if (empty($items)) {
				throw new \Exception('Não foram recebidos items');
			}
			$requiredFields = array(
				'CHAVE',
				'VENDA',
				'CDFILIAL',
				'DATA',
				'NOW',
				'CDPRODUTO',
				'QTDEPROD',
				'CDLOJA',
				'CDSENHAPED',
				'CHAVEITEM'
			);
			$nullableFields = array(
				'CDCAIXA',
				'NRSEQVENDA',
				'NRMESA',
				'NRCOMANDAVND',
				'DSCOMANDAVND',
				'NRSEQUITVEND',
				'NRCOMANDA',
				'DSCOMANDA',
				'NRVENDAREST',
				'NRPRODCOMVEN',
				'OBSDIGITADA',
				'NMCONSUMIDOR',
				'CDGRPOCOR',
				'CDOCORR',
				'NRSEQPRODCOM',
				'CDPRODPROMOCAO',
				'NRATRAPROD',
				'NRLUGARMESA',
				'IDORIGEMVENDA',
				'IDITORIGEMVENDA',
				'TXPRODPED'
			);

			foreach ($items as &$item) {
				$item['IDORIPED'] = 'K';
				$item['NRPEDIDOFOS'] = null;
				$item['NRITPEDIDOFOS'] = null;
				try {
					$data = new \DateTime();
					$data = $data->createFromFormat('H:i:s d/m/Y', $item['DATA']);
					$item['DATA'] = $data->format('c');
					$item['DATA'] = substr($item['DATA'], 0, -6);
				} catch (\Exception $e) {
				}
				try {
					$data = new \DateTime();
					$data = $data->createFromFormat('H:i:s d/m/Y', $item['NOW']);
					$item['NOW'] = $data->format('H:i:s m/d/Y');
				} catch (\Exception $e) {
				}

				foreach ($requiredFields as $requiredField) {
					if (!array_key_exists($requiredField, $item) || !isset($item[$requiredField])) {
						throw new \Exception('Item {$item}, sem o campo {$requiredField}. Por favor revise os dados.');
					}
				}
				foreach ($nullableFields as $nullableField) {
					if (!array_key_exists($nullableField, $item)) {
						$item[$nullableField] = null;
					}
				}
			}

			$itemsForInsertion = $this->pedidoFosGenerator($items);
			if (!empty($itemsForInsertion)) {
				// ordena, define hora de exibição e insere na ITPEDIDOFOS
				$this->handleProductsForDisplay($itemsForInsertion);
			}

			$message = 'Pedidos inseridos com sucesso.';
			$response->addMessage(new Message($message, self::SUCCESS_MESSAGE));
			$this->util->commit();
		} catch (\Exception $e) {
			$this->util->rollBack();
			$message = 'Erro ao inserir pedidos. ' . $e->getMessage();
			$response->addMessage(new Message($message, self::ERROR_MESSAGE));
		}
	}
	public function generateDumbITPEDIDOFOS(Request $request, Response $response)
	{
		$inputJSON = file_get_contents('php://input');
		$input = json_decode($inputJSON, TRUE);
		$this->util->beginTransaction();
		try {
			if (empty($input)) {
				throw new \Exception('Não foram recebidos parâmetros');
			}

			//CHAVE, Chave unica do pedido a ser gerado.  VE.CDFILIAL + VE.CDCAIXA + VE.NRSEQVENDA  se for venda, VE.CDFILIAL + IT.NRSEQPRODCUP se for pedido
			//VENDA, Identificador de onde veio a venda. 'ODHENAPI' caso gerado pela api, VENDA caso balcão, COMANDA caso pedido
			//CDFILIAL, Filial relacionada ao item
			//NRPEDIDO, Chave unica do pedido a ser gerado. pode ser a CHAVE
			//DATA, data do pedido. "agora" geralmente
			//NOW, agora, mesmo
			//CDPRODUTO, codigo do produto
			//QTDEPROD, quantidade
			//CDCAIXA, caso VENDA
			//NRSEQVENDA, caso VENDA
			//NRMESA do pedido, caso haja
			//NRCOMANDAVND, NRCOMANDA caso venda.  NRCOMANDA caso pedido
			//DSCOMANDAVND, DSCOMANDAVND caso venda. DSCOMANDA caso pedido
			//NRSEQUITVEND, caso venda
			//NRCOMANDA, caso pedido. retundancia a gente ve por aqui
			//DSCOMANDA, caso pedido
			//NRVENDAREST, caso pedido
			//NRPRODCOMVEN, caso pedido
			//QTMAXPEDFOS, não sei o que é isso @lua
			//CDLOJA, codigo da loja do pedido
			//OBSDIGITADA, obeservação personalizada do pedido. DSOBSPEDDIGITA caso venda. DSOBSPEDDIGCMD caso pedido.
			//CDSENHAPED, codigo do pedido na tela do kds.
			//NMCONSUMIDOR, nome do consumidor do pedido
			//CDGRPOCOR, caso o item seja uma observação ?
			//CDOCORR, caso o item seja uma observação ?
			//CHAVEPEDIDO, CHAVE? DE NOVO!?
			//IDORIPED, K! (O kds do fos insere F)
			//NRPEDIDOFOS, null
			//NRITPEDIDOFOS, null
			//NRSEQPRODCOM, NRSEQPRODCOMIT caso venda, NRSEQPRODCOM caso pedido
			//CDPRODPROMOCAO,
			//CHAVEITEM, chave unica do item a ser inserido. IT.CDFILIAL + IT.CDCAIXA + IT.NRSEQVENDA + IT.NRSEQUITVEND caso venda,IT.CDFILIAL + IT.NRVENDAREST + IT.NRCOMANDA + IT.NRPRODCOMVEN caso pedido
			//NRATRAPROD, tempo de atraso no item, famigerado "SEGURA". NRATRAPRODITVE caso venda, NRATRAPRODCOVE caso pedido
			//NRLUGARMESA, posição na mesa

			$items = $input['items'];
			if (empty($items)) {
				throw new \Exception('Não foram recebidos items');
			}
			$requiredFields = array(
				"CDFILIAL",
				"CHAVE",
				"CDSETOR",
				"CDLOJA",
				"CDSENHAPED",
				"CDPRODUTO",
				"QTPRODPEFOS",
				"NRTEMPOPROD",
				"NRTEMPOEXIB"
			);
			$nullableFields = array(
				'NRMESA',
				"NRLUGARMESAIT",
				'NMCONSUMIDOR',
				'NRSEQVENDA',
				'NRSEQPRODCOMKDS',
				'CDPRODPROMOKDS',
				'CDPRODUTOPAI',
				'DSCHAVEPEDPAI',
				'CDOCORR',
				'CDGRPOCOR',
				'NRPRODCOMVEN',
				'NRCOMANDA',
				'NRVENDAREST',
				'NRSEQUITVEND',
				'NRSEQVENDA',
				'CDCAIXA',
				'CDPRODPROMOCAO',
				'QTMAXPEDFOS',
				"TXPRODPED",
				"DSCOMANDA"
			);

			foreach ($items as &$item) {
				foreach ($requiredFields as $requiredField) {
					if (!array_key_exists($requiredField, $item) || !isset($item[$requiredField])) {
						if ($requiredField == 'CHAVE') {
							throw new \Exception("Um pedido sem chave foi recebido. Por favor revise os dados.");
						} else {
							throw new \Exception("Item {$item['CHAVE']}, sem o campo $requiredField. Por favor revise os dados.");
						}
					}
					if ($requiredField == 'QTPRODPEFOS' && $item[$requiredField] <= 0) {
						throw new \Exception("Quantidade do pedido inserido deve ser maior que zero.");
					}
				}
				foreach ($nullableFields as $nullableField) {
					if (!array_key_exists($nullableField, $item)) {
						$item[$nullableField] = null;
					}
				}
			}

			$itemsForInsertion = $this->dumbPedidoFosGenerator($items);
			if (!empty($itemsForInsertion)) {
				// ordena, define hora de exibição e insere na ITPEDIDOFOS
				$this->handleProductsForDisplay($itemsForInsertion);
			}

			$message = 'Pedidos inseridos com sucesso.';
			$response->addMessage(new Message($message, self::SUCCESS_MESSAGE));
			$this->util->commit();
		} catch (\Exception $e) {
			$this->util->rollBack();
			$message = 'Erro ao inserir pedidos. ' . $e->getMessage();
			$response->addMessage(new Message($message, self::ERROR_MESSAGE));
		}
	}

	private function dumbPedidoFosGenerator($items)
	{
		foreach ($items as $item) {
			$CDFILIAL = $item['CDFILIAL'];
			$CDPRODUTO = $item['CDPRODUTO'];
			$QTPRODPEFOS = $item['QTPRODPEFOS'];
			$CDCAIXA = $item['CDCAIXA'];
			$NRSEQVENDA = $item['NRSEQVENDA'];
			$NRSEQUITVEND = $item['NRSEQUITVEND'];
			$NRVENDAREST = $item['NRVENDAREST'];
			$NRCOMANDA = $item['NRCOMANDA'];
			$NRPRODCOMVEN = $item['NRPRODCOMVEN'];
			$TXPRODPED = !empty($item['TXPRODPED']) ? $item['TXPRODPED'] :  null;
			$CDSETOR = $item['CDSETOR'];
			$CDGRPOCOR = $item['CDGRPOCOR'];
			$CDOCORR = $item['CDOCORR'];
			$DSCHAVEPEDPAI = $item['DSCHAVEPEDPAI'];
			$CDPRODUTOPAI = $item['CDPRODUTOPAI'];
			$CDPRODPROMOKDS = $item['CDPRODPROMOKDS'];
			$NRSEQPRODCOMKDS = $item['NRSEQPRODCOMKDS'];
			$NRTEMPOPROD = $item['NRTEMPOPROD'];
			$NRTEMPOEXIB = $item['NRTEMPOEXIB'];
			$NRLUGARMESAIT = 0;
			$IDLIBERADO = 'N';
			$IDSTATUSPEDFOS = 'E';
			$DSTIPOPEDFOS = 'ODHENAPI';
			$NRMESA		  = $item['NRMESA'];
			$DSCOMANDA    = $item['DSCOMANDA'];
			$QTMAXPEDFOS  = $item['QTMAXPEDFOS'];
			$CDSENHAPED   = $item['CDSENHAPED'];
			$NMCONSUMIDOR = $item['NMCONSUMIDOR'];
			$IDSITITPEDFOS = '1';
			$IDATIVO       = 'S';
			$DSPEDIDOFOS   = $item['CHAVE'];
			$DSCHAVEPEDFOS = $item['CHAVE'];
			$CHAVEPEDIDO   = $item['CHAVE'];
			$NRMAIORTEMPO  = $item['NRTEMPOPROD'];
			$CDLOJA = $item['CDLOJA'];
			$DTPEDIDOFOS   = new \Datetime();
			$DTHREXIBICAOPROD = new \Datetime();
			$NRATRAPROD  = 0;
			$this->util->newCode('PEDIDOFOS' . $CDFILIAL);
			$NRPEDIDOFOS = $this->KDSManagerService->getNewCode('PEDIDOFOS' . $CDFILIAL);
			$NRPEDIDOFOS = $NRPEDIDOFOS[0]['NRSEQUENCIAL'];
			$NRPEDIDOFOS = substr($NRPEDIDOFOS, -10);
			$NRPEDIDOFOSMSDE = $NRPEDIDOFOS;
			$NRPEDIDOFOSAUX  = $NRPEDIDOFOS;
			$IDORIGEMVENDA = isset($item['IDORIGEMVENDA']) ? $item['IDORIGEMVENDA'] : 'ODH_API';
			$this->KDSManagerService->insertOrder($CDFILIAL, $NRPEDIDOFOS, $DSPEDIDOFOS, $DSCHAVEPEDFOS, $DTPEDIDOFOS, $IDSTATUSPEDFOS, $DSTIPOPEDFOS, $NRMESA, $NRCOMANDA, $DSCOMANDA, $QTMAXPEDFOS, $CDSETOR, $NRPEDIDOFOSAUX, $CDLOJA, $CDSENHAPED, $NRPEDIDOFOSMSDE,	$NMCONSUMIDOR, $IDATIVO);
			$NRITPEDIDOFOS = $this->generateNewNRITPEDIDOFOS($CDFILIAL, $NRPEDIDOFOS);
			$this->KDSManagerService->insertOrderItem($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS, $CDPRODUTO, $QTPRODPEFOS, $CDCAIXA, $NRSEQVENDA, $NRSEQUITVEND, $NRVENDAREST, $NRCOMANDA, $NRPRODCOMVEN, $TXPRODPED, $IDSITITPEDFOS, $CDSETOR, $CDGRPOCOR, $CDOCORR, $DTHREXIBICAOPROD, $DSCHAVEPEDPAI, $CDPRODUTOPAI, $CDPRODPROMOKDS, $NRSEQPRODCOMKDS, $NRTEMPOPROD, $NRTEMPOEXIB, $NRATRAPROD, $NRLUGARMESAIT, $IDLIBERADO, $NRMAIORTEMPO, $IDORIGEMVENDA);
		}
	}

	public function getOrders($CDSETOR = null)
	{
		if($CDSETOR != null){
			$sectorType = $this->KDSManagerService->getSectorType($CDSETOR)[0];
            /**
             * E = Expedição
             * M = Montagem
             * C = Checagem
             */
			if($sectorType['IDTIPOSETOR'] == 'E' || $sectorType['IDTIPOSETOR'] == 'M' || $sectorType['IDTIPOSETOR'] == 'C'){			
                $orders = $this->buildDataSourceExpedition($CDSETOR);
		
			} else {
				$orders = $this->buildDataSource($CDSETOR);
			}
			
		} else {

			$productionOrders = $this->buildDataSource($CDSETOR);
			$expeditionOrders = $this->buildDataSourceExpedition($CDSETOR);
			$orders = array_merge($productionOrders, $expeditionOrders);
	
		}
		// $start = microtime(true);
		// // echo "\n\n\n".'Iniciando o processo de geração das caixas dos pedidos'."\n";
		// // echo '-------------------------------------------------------------'."\n";
		// busca os tempos de troca de cor para cada pedido
		$orders = $this->getTimersForBoxes($orders);

		// PROCURAR OBS COM DÓLAR E REMOVER

		return $orders;
	}

	public function getStatisticsFromOneClient($CDFILIAL, $CDSETOR, $socketClient)
	{
		$paramCDFILIAL = array (
			':CDFILIAL' => $CDFILIAL
		);
		$getExibEstatics = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_IDEXIBESTATIS, $paramCDFILIAL);

		// $getExibEstatics vai verificar na query se está habilitado a exibição das estatíscas.
		if (isset($getExibEstatics[0]) && $getExibEstatics[0]["IDEXIBESTATIS"] == "S") {
			$newStatistics = $this->KDSManagerService->getStatisticsFromOneClient($CDFILIAL, $CDSETOR);

			if (!empty($newStatistics)) {
				$packageToSend = array(
					'event' => 'sendStatistics',
					'data'  => $newStatistics
				);
				$socketClient->send(json_encode($packageToSend));
			}
		}
	}

	public function getDataFromOneClient($CDFILIAL, $CDSETOR, $IDTIPOSETOR, $socketClient) {	
		//limpa a sessão ao logar ou recarregar a pagina
        $this->environment->clearSession();
		$dateTimeParam = $this->environment->getSessionValue('showDateTime');
        if (!$dateTimeParam) {
            $this->environment->setSessionValue('showDateTime', $this->showDateTime);
        }

		// busca os pedidos para enviar para o client que acabou de logar
		$orders = $this->getOrders($CDSETOR);

		$arrayToSend = array();

		// $start = microtime(true);

		// // echo 'Processando as vendas trazidas do banco de dados'.'\n\n\n';
		// // echo '---------------------------------------------------------';

		foreach ($orders as $item) {
			$orderKey =	$item['CDFILIAL'] . '-' . $item['NRPEDIDOFOS'] . $item['items'][0]['NRITPEDIDOFOS'];
			$item['orderKey'] = $orderKey;
			$item['clientKey'] = $item['CDFILIAL'] . '-' . $item['items'][0]['CDSETOR'];
			if ($item['clientKey'] == $CDFILIAL . '-' . $CDSETOR) {
				if (empty($arrayToSend[$item['clientKey']])) {
					// se ainda não existe o array de itens para este cliente, cria ele
					$orderArray = array(
						$item['orderKey'] => $item
					);
					$arrayToSend[$item['clientKey']] = $orderArray;
				} else {
					// se já existe o array, somente adiciona o próximo
					$arrayToSend[$item['clientKey']][$item['orderKey']] = $item;
				}
			}
		}

		if (!empty($arrayToSend)) {
			$packageToSend = array(
				'event' => 'sendChanges',
				'data'  => $arrayToSend
			);
			$socketClient->send(json_encode($packageToSend));
		}

		// $getExibEstatics vai verificar na query se está habilitado a exibição das estatíscas.
		$paramCDFILIAL = array (
			':CDFILIAL' => $CDFILIAL
		);
		$getExibEstatics = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_IDEXIBESTATIS, $paramCDFILIAL);
		$params = array(
			':CDFILIAL' => $CDFILIAL,
			':CDSETOR'  => $CDSETOR,
			':IDTIPOSETOR' => $IDTIPOSETOR
		);
		if(isset($getExibEstatics[0]) && $getExibEstatics[0]["IDEXIBESTATIS"] == "S") {
			// busca as estatísticas para enviar para o client que acabou de logar
			$newStatistics = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_STATISTICS_FROM_ONE_CLIENT, $params);
			//$newStatisticsGeral = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_STATISTICS_GERAL, $params);
			$arrayToSend = array();
			foreach ($newStatistics as $current) {
				$clientKey = $current['CDFILIAL'] . '-' . $current['CDSETOR'];
				if ($clientKey == $CDFILIAL . '-' . $CDSETOR) {
					$arrayToSend[$clientKey] = $current;
				}
			}

			// foreach ($newStatisticsGeral as $current) {
			// 	$clientKey = $current['CDFILIAL'];
			// 	if ($clientKey == $CDFILIAL) {
			// 		$arrayToSend[$clientKey] = $current;
			// 	}
			// }

			if (!empty($arrayToSend)) {
				$packageToSend = array(
					'event' => 'sendStatistics',
					'data'  => $arrayToSend
				);

				// Daqui que saem os dados para o getStatistics do Socket
				$socketClient->send(json_encode($packageToSend));
			}
		}
	}
}