<?php
namespace tests\Zeedhi\Framework\DataSource\Manager\Mongo\Types;

use Zeedhi\Framework\DataSource\Manager\Mongo\Types\DateType;

class DateTypeTest extends \PHPUnit_Framework_TestCase {

    protected function setUp() {
        $extensionLoaded = extension_loaded('mongo') || extension_loaded('mongodb');
        if (!$extensionLoaded) {
            $this->markTestSkipped('The tests require mongo extension');
        }

        parent::setUp();
    }

    public function testGetDateTimePassingDateTime() {
        $dateTime = new \DateTime();
        $newDateTime = DateType::getDateTime($dateTime);

        $this->assertEquals($dateTime, $newDateTime);
    }

    public function testGetDateTimePassingMongoDate() {
        $dateTime = new \MongoDate();
        $newDateTime = DateType::getDateTime($dateTime);

        $expected = new \DateTime('now', new \DateTimeZone("UTC"));

        $this->assertEquals($expected, $newDateTime);
    }

    public function testGetDateTimePassingNumber() {
        $expected = new \DateTime('now', DateType::getUTC());

        $dateTime = time() + 0.852;
        $newDateTime = DateType::getDateTime($dateTime);

        $this->assertSame('852000', $newDateTime->format('u'));
        $this->assertEquals($expected->format('d/m/Y H:i:s'), $newDateTime->format('d/m/Y H:i:s'));
    }

    public function testGetDateTimePassingString() {
        $dateTime = 'now';
        $newDateTime = DateType::getDateTime($dateTime);

        $expected = new \DateTime('now', DateType::getUTC());

        $this->assertEquals($expected, $newDateTime);
    }

    public function testGetDateTimePassingInvalidString() {
        $this->setExpectedException('\InvalidArgumentException');
        DateType::getDateTime('invalid date');
    }

    public function testGetDateTimePassingFalse() {
        $this->setExpectedException('\InvalidArgumentException');
        DateType::getDateTime(false);
    }

    public function testConvertToDatabaseValue() {
        $dateTimeType = new DateType();
        $dateTime = time();
        $newDateTime = $dateTimeType->convertToDatabaseValue($dateTime);

        $this->assertInstanceOf('MongoDate', $newDateTime);
    }

    public function testConvertToDatabaseValuePassingNull() {
        $dateTimeType = new DateType();
        $dateTime = null;
        $newDateTime = $dateTimeType->convertToDatabaseValue($dateTime);
        $this->assertNull($newDateTime);
    }

    public function testConvertToDatabaseValuePassingMongoDate() {
        $dateTimeType = new DateType();
        $dateTime = new \MongoDate();
        $newDateTime = $dateTimeType->convertToDatabaseValue($dateTime);
        $this->assertEquals($dateTime, $newDateTime);
    }

    public function testConvertToPHPValue() {
        $dateTimeType = new DateType();
        $newDateTime = $dateTimeType->convertToPHPValue(time());
        $this->assertInstanceOf('DateTime', $newDateTime);
        $this->assertEquals(new \DateTime(), $newDateTime);
    }

    public function testConvertToPHPValuePassingMongoDate() {
        $dateTimeType = new DateType();
        $newDateTime = $dateTimeType->convertToPHPValue(new \MongoDate());
        $this->assertInstanceOf('DateTime', $newDateTime);
        $this->assertEquals(new \DateTime(), $newDateTime);
    }

    public function testConvertToPHPValuePassingNull() {
        $dateTimeType = new DateType();
        $newDateTime = $dateTimeType->convertToPHPValue(null);
        $this->assertNull($newDateTime);
    }

}