function ColorController($scope, metaDataFactory, templateManager, ScreenService) {

	var self = this;

	this.__defineGetter__('field', function() {
		return $scope.field;
	});

	this.__defineGetter__('widget', function() {
		return $scope.widget;
	});

	function focusFieldElement() {
		if(self.field && self.field.element) {
			self.field.element.focus();
		}
	}

	function setSelectedColor() {
		var field = self.field;
		var widget = self.field.widget;
		var colorPickerWidget = self.field.colorPickerWidget;

		widget.currentRow[field.name] = tinycolor(Util.clone(colorPickerWidget.currentRow));

		if(field.change) {
			field.change();
		}
	}

	function setSelectedColors() {
		var field = self.field;
		var widget = self.field.widget;
		var colorPaletteWidget = self.field.colorPaletteWidget;

		widget.currentRow[field.name] = Util.clone(colorPaletteWidget.currentRow).map(function(color) {
			return tinycolor(color._originalInput);
		});

		if(field.change) {
			field.change();
		}
	}

	function addSelectedColor() {
		var widget = self.widget;
		var colorPickerWidget = widget.colorPickerWidget;

		var colorRow = {};
		var newColor = tinycolor(colorPickerWidget.currentRow);
		colorRow[widget.valueField] = newColor.toHexString();

		if(!!!widget.dataSource.data) {
			widget.dataSource.data = [];
		}

		widget.dataSource.data.push(colorRow);
		widget.currentRow.push(newColor);

		if(widget.onNewColor) {
			widget.onNewColor();
		}
	}

	function getSelectedColors() {
		var selectedColors;
		var field = self.field;
		var widget = self.field.widget;

		if(widget && widget.currentRow && widget.currentRow[field.name]) {
			selectedColors = widget.currentRow[field.name].map(function(color) {
				return tinycolor(color).toHslString();
			});
		} else {
			selectedColors = [];
		}

		return selectedColors;
	}

	function getPlural(amount) {
		return amount > 1 ? " colors" : " color";
	}

	function validadeNumberOfColors() {
		var field = self.field;
		var widget = self.field.colorPaletteWidget;
		var minColors = field.restrictions.minColors;

		if(widget.currentRow.length >= minColors) {
			ScreenService.closePopup();
		} else {
			ScreenService.showMessage(
				ScreenService.i18n("Please select at least " + minColors + getPlural(minColors)) + "!"
			);
		}
	}

	function getSugestedColors(unrestricted) {
		var widget = $scope.widget;
		var sugestedColors = [];


		if(widget.showSugestedColors !== false) {

			sugestedColors = sugestedColors.concat([
				"#3cd7e7", "#7ac95b", "#f2ca27", "#ff7921", "#e74c3c",
				"#1ac3d6", "#5eb43c", "#f39c12", "#d35400", "#d62d1a",
				"#3498db", "#9b59b6", "#ff4c65", "#34495e", "#afb6bb",
				"#217dbb", "#804399", "#ff334f", "#22303d"
			]);
		}

		if(!!widget.dataSource && !!widget.dataSource.data) {

			sugestedColors = sugestedColors.concat(
				widget.dataSource.data.map(function(item) {
					return item[widget.valueField];
				})
			);
		}

		// Transforms in HEX color and
		sugestedColors = sugestedColors
			.map(function(color) {
				return tinycolor(color).toHexString();
			})
			.filter(function(color, pos) {
	    	return sugestedColors.indexOf(color) == pos;
			});

		// Only returns a maimum of five sugested colors
		return unrestricted ? sugestedColors : sugestedColors.slice(0,5);
	}

	function hasSugestedColors() {
		return !!getSugestedColors().length;
	}

	$scope.getSugestedColors = getSugestedColors;
	$scope.hasSugestedColors = hasSugestedColors;
	$scope.getSelectedColors = getSelectedColors;

	this.openColorPicker = function() {
		var component = self.field || self.widget;

		var dataSource = {};

		if(!component.colorPickerWidget) {
			component.colorPickerWidget = metaDataFactory.widgetFactory({
				"isVisible": true,
				"label": component.label,
				"name": component.name + "_color_picker_widget",
				"template": "widget/color-picker.html",
				"valueField": component.valueField,
				"dataSource": dataSource,
				"container": component.container || component.widget.container,
				"showSugestedColors": component.showSugestedColors,
				"events": [],
				"actions": [
					{
            "isVisible": true,
            "label": "Cancel",
            "showAsAction": "back",
            "hideIcon": true,
            "events": [
              {
                "name": "ActionEvent",
                "code": "ScreenService.closePopup(); args.owner.widget.focusFieldElement();"
              }
            ]
          },
          {
          	"isVisible": true,
            "label": "Ok",
            "showAsAction": "never",
            "events": [
              {
                "name": "ActionEvent",
                "code": "ScreenService.closePopup(); args.owner.widget.setSelectedColor(); args.owner.widget.focusFieldElement();"
              }
            ]
          },
				]
			});

			if(component instanceof Field) {
				component.colorPickerWidget.setSelectedColor = setSelectedColor;
				component.colorPickerWidget.focusFieldElement = focusFieldElement;
			} else {
				component.colorPickerWidget.showSugestedColors = false;
				component.colorPickerWidget.setSelectedColor = addSelectedColor;
			}
		}

		if(component instanceof Field) {
			component.colorPickerWidget.oldColor = component.widget.currentRow[component.name];
		}

		ScreenService.openPopup(component.colorPickerWidget);
	};

	this.openColorPalette = function() {
		if(this.field && this.field.template == "field/color-multiple.html") {
			var field = self.field;

			if(!field.colorPaletteWidget) {
				field.colorPaletteWidget = metaDataFactory.widgetFactory({
					"isVisible": true,
					"label": field.label,
					"name": field.name + "_color_palette_widget",
					"template": "widget/color-palette.html",
					"valueField": field.valueField,
					"dataSource": field.dataSource || {},
					"container": field.widget.container,
					"parentField": field,
					"restrictions": field.restrictions,
					"events": field.events,
					"actions": [
						{
	            "isVisible": true,
	            "label": "Cancel",
	            "showAsAction": "back",
	            "hideIcon": true,
	            "events": [
	              {
	                "name": "ActionEvent",
	                "code": "ScreenService.closePopup(); args.owner.widget.focusFieldElement();"
	              }
	            ]
	          },
	          {
	          	"isVisible": true,
	            "label": "Ok",
	            "showAsAction": "never",
	            "events": [
	              {
	                "name": "ActionEvent",
	                "code": "args.owner.widget.validadeNumberOfColors(); args.owner.widget.setSelectedColors(); args.owner.widget.focusFieldElement();"
	              }
	            ]
	          },
					]
				});

				field.colorPaletteWidget.validadeNumberOfColors = validadeNumberOfColors;
				field.colorPaletteWidget.setSelectedColors = setSelectedColors;
				field.colorPaletteWidget.focusFieldElement = focusFieldElement;
			}

			field.colorPaletteWidget.oldColor = field.widget.currentRow[field.name];
			ScreenService.openPopup(field.colorPaletteWidget);
		}
	};

	if(this.field && this.field.template == "field/color.html") {
		this.field.openColorPicker = self.openColorPicker;
	}

	if(this.field && this.field.template == "field/color-multiple.html") {
		this.field.openColorPalette = self.openColorPalette;
	}

}
