function MetaDataLoader(ContainerLoader, metaDataFactory, ScreenService, ZHPromise, templateManager,
	restEngine, ApplicationContext, $rootScope, eventAggregator, urlEngine) {


	var getFileInAllDirectories = function(fileName, urls) {
		return ZHPromise.all(urls.map(function(url) {
			return restEngine.requestMetaData(fileName, {}, undefined, url).then(function(file) {
				return file.data;
			}, function() {
				return {};
			});
		}));
	};

	this.loadContainer = function(name, maintainWindowFilter) {
		if (name) {
			if (!templateManager.containers[name]) {
				return ContainerLoader.loadContainer(name).then(function(containerJson) {
					if (containerJson) {
						if (containerJson.stages) {
							templateManager.containers[name] = metaDataFactory.wizardFactory(containerJson);
						} else {
							templateManager.containers[name] = metaDataFactory.containerFactory(containerJson, maintainWindowFilter, false);
						}
					}
					if (templateManager.project.containersTimeToLive) {
						activeTimeToLive(name);
					}
					return templateManager.containers[name];
				});
			} else {
				if (maintainWindowFilter && templateManager.containers[name].reloadLazyDataSources) {
					templateManager.containers[name].reloadLazyDataSources(maintainWindowFilter);
				}
				return ZHPromise.when(templateManager.containers[name]);
			}
		} else {
			return ZHPromise.reject({
				err: ScreenService.i18n("Invalid arguments")
			});
		}
	};

	this.loadMessageByCode = function() {
		var messagesPromise = restEngine.requestMetaData("json/messages.json").then(function(response) {
			return response.data;
		});
		return messagesPromise;
	};

	this.loadData = function() {
		var urls = templateManager.getCommonFilesSource().data;
		return getFileInAllDirectories("json/data.json", urls).then(function(files) {
			var data = files.reduce(function(data, file) {
				return Util.extend(data, file);
			});
			templateManager.setDataSources(data);
		});
	};

	this.loadProjectMetadata = function() {
		var containers = restEngine.requestMetaData("json/containers.json").then(function(data) {
			var windows = data.data;
			templateManager.containers = windows;
			templateManager.project = metaDataFactory.projectFactory(windows.zeedhi_project);
			templateManager.project.__indexPath = data.url;
			templateManager.updateURL();
			templateManager.updateTitle(templateManager.project.label);
			templateManager.updateVersionData();
			templateManager.historyItems = [];
			if (ApplicationContext.preferencesDecorator) {
				ApplicationContext.preferencesDecorator.syncRepositoryData();
			}
			if (!Util.inDevToolsMode()) {
				eventAggregator.publish("beforeProjectStartup");
			}
			urlEngine.init(templateManager.project);
		});

		var environment = restEngine.requestMetaData("json/environment.json").then(function(response) {
			return response.data;
		}, function() {});

		return ZHPromise.all([containers, environment]).then(function(response) {
			var parameters = response[1];
			for (var key in parameters) {
				templateManager.project[key] = parameters[key];
			}
			templateManager.projectPromise.resolve(templateManager.containers);
		});
	};

	this.loadAcl = function() {
		var urls = templateManager.getCommonFilesSource().acl;
		getFileInAllDirectories("json/acl.json", urls).then(function(files) {
			templateManager.aclRules = files.reduce(function(acl, file) {
				return Util.extend(acl, file);
			});
		});
	};

	function activeTimeToLive(containerName) {
		setTimeout(function() {
			delete templateManager.containers[containerName];
			var containerIndexOnHistoryItems = templateManager.historyItems.indexOf(containerName);
			if (~containerIndexOnHistoryItems) {
				templateManager.historyItems.splice(containerIndexOnHistoryItems, 1);
			}
		}, templateManager.project.containersTimeToLive * 1000);
	}

}