describe("Util", function() {
	var zhPromise = new ZHPromise(Q);

	window.functions = {
		"string": function(){ return 'teste'; },
		"number": function(){ return 23; },
		"array": function(){ return [1,2,3]; },
		"undefined": function(){ return undefined; },
		"null": function(){ return null; },
		"boolF": function(){ return false; },
		"boolT": function(){ return true; },
		"object": function(){ return {a: 'aa'}; }
	};

	var zhUtil = new ZhUtil({ getArgsToFunction: function() { return { keys: ['functions'], args: [window.functions] }; } });

	var _simpleValueAsserts = function(util) {
		expect(util.evaluate('teste')).toEqual('teste');
		expect(util.evaluate(123)).toEqual(123);
		expect(util.evaluate([1,2,3])).toEqual([1,2,3]);
		expect(util.evaluate(false)).toEqual(false);
		expect(util.evaluate(true)).toEqual(true);
		expect(util.evaluate(null)).toEqual(null);
		expect(util.evaluate(undefined)).toEqual(undefined);
		expect(util.evaluate({a: 'a'})).toEqual({a: 'a'});
	};

	var _functionsValueAsserts = function(util) {
		expect(util.evaluate(functions.string)).toEqual('teste');
		expect(util.evaluate(functions.number)).toEqual(23);
		expect(util.evaluate(functions.array)).toEqual([1,2,3]);
		expect(util.evaluate(functions.undefined)).toEqual(undefined);
		expect(util.evaluate(functions.null)).toEqual(null);
		expect(util.evaluate(functions.boolF)).toEqual(false);
		expect(util.evaluate(functions.boolT)).toEqual(true);
		expect(util.evaluate(functions.object)).toEqual({a: 'aa'});
	};

	var _evaluatedFunctionsAsserts = function(util){
		expect(util.evaluate('@functions.string()@')).toEqual('teste');
		expect(util.evaluate('@functions.number()@')).toEqual(23);
		expect(util.evaluate('@functions.number() + functions.number()@')).toEqual(46);
		expect(util.evaluate('@functions.array()@')).toEqual([1,2,3]);
		expect(util.evaluate('@functions.undefined()@')).toEqual(undefined);
		expect(util.evaluate('@functions.null()@')).toEqual(null);
		expect(util.evaluate('@functions.boolF()@')).toEqual(false);
		expect(util.evaluate('@functions.boolT()@')).toEqual(true);
		expect(util.evaluate('@functions.object()@')).toEqual({a: 'aa'});
	};

	var _evaluatedFunctionsWithPrefixAsserts = function(util){
		expect(util.evaluate('My value is: @functions.string()@')).toEqual('My value is: teste');
		expect(util.evaluate('My value is: @functions.number()@')).toEqual('My value is: 23');
		expect(util.evaluate('My value is: @functions.number() + functions.number()@')).toEqual('My value is: 46');
		expect(util.evaluate('My value is: @functions.array()@')).toEqual('My value is: 1,2,3');
		expect(util.evaluate('My value is: @functions.undefined()@')).toEqual('My value is: undefined');
		expect(util.evaluate('My value is: @functions.null()@')).toEqual('My value is: null');
		expect(util.evaluate('My value is: @functions.boolF()@')).toEqual('My value is: false');
		expect(util.evaluate('My value is: @functions.boolT()@')).toEqual('My value is: true');
		expect(util.evaluate('My value is: @functions.object()@')).toEqual('My value is: [object Object]');

		expect(util.evaluate('@functions.number()@_@functions.string()@')).toEqual('23_teste');
		expect(util.evaluate('@functions.number()@_@functions.number()@')).toEqual('23_23');
		expect(util.evaluate('@functions.number()@_@functions.array()@')).toEqual('23_1,2,3');
		expect(util.evaluate('@functions.number()@_@functions.undefined()@')).toEqual('23_undefined');
		expect(util.evaluate('@functions.number()@_@functions.null()@')).toEqual('23_null');
		expect(util.evaluate('@functions.number()@_@functions.boolF()@')).toEqual('23_false');
		expect(util.evaluate('@functions.number()@_@functions.boolT()@')).toEqual('23_true');
		expect(util.evaluate('@functions.number()@_@functions.object()@')).toEqual('23_[object Object]');
	};

	it("Util.mergeSort: should be stable", function(){
		var arrayToSort = [{v:10,t:"t1"},{v:20,t:"t2"},{v:20,t:"t3"},{v:40,t:"t4"},{v:30,t:"t5"}];
		var expected = [{v:10,t:"t1"},{v:20,t:"t2"},{v:20,t:"t3"},{v:30,t:"t5"},{v:40,t:"t4"}];
		var sorted = Util.mergeSort(arrayToSort, function(a,b) 	{ return a.v - b.v;});
		expect(sorted).toEqual(expected);
	});
	
	it("Util.mergeSort: should be stable when using reverse order", function() {
		var arrayToSort = [{v:10,t:"t1"},{v:20,t:"t2"},{v:20,t:"t3"},{v:40,t:"t4"},{v:30,t:"t5"}];
		var expected = [{v:40,t:"t4"}, {v:30,t:"t5"}, {v:20,t:"t2"}, {v:20,t:"t3"}, {v:10,t:"t1"}];
		var sorted = Util.mergeSort(arrayToSort, function(a,b) 	{ return b.v - a.v;});
		expect(sorted).toEqual(expected);
	});
	
	it("Util.mergeSort: should maintain every item in the same order when all items have the same value", function() {
		var arrayToSort = [{v:10,t:"t4"}, {v:10,t:"t5"}, {v:10,t:"t2"}, {v:10,t:"t3"}, {v:10,t:"t1"}];
		var expected = [{v:10,t:"t4"}, {v:10,t:"t5"}, {v:10,t:"t2"}, {v:10,t:"t3"}, {v:10,t:"t1"}];
		var sorted = Util.mergeSort(arrayToSort, function(a,b) 	{ return a.v - b.v;});
		expect(sorted).toEqual(expected);
	});
	
	it("Util.mergeSort: should maintain every item in the same order when all items have the same value and the order is reverse", function() {
		var arrayToSort = [{v:10,t:"t4"}, {v:10,t:"t5"}, {v:10,t:"t2"}, {v:10,t:"t3"}, {v:10,t:"t1"}];
		var expected = [{v:10,t:"t4"}, {v:10,t:"t5"}, {v:10,t:"t2"}, {v:10,t:"t3"}, {v:10,t:"t1"}];
		var sorted = Util.mergeSort(arrayToSort, function(a,b) 	{ return b.v - a.v;});
		expect(sorted).toEqual(expected);
	});

	it('Util.evaluate: if the value is simple, always must be return a origin value', function() {
		_simpleValueAsserts(Util);
	});

	it('Util.evaluate: if the value is a function, always must be return a function return value', function() {
		_functionsValueAsserts(Util);
	});

	it('Util.evaluate: if the value is a string with one function, always must be return a function return value', function() {
		_evaluatedFunctionsAsserts(Util);
	});

	it('Util.evaluate: if the value is a string with a function and prefix, or other function, always must be return a function return value in string format', function() {
		_evaluatedFunctionsWithPrefixAsserts(Util);
	});

	it("DoSync", function(done) {
		var processed = false;
		var lasExecuted = 0;
		var promises = [];
		var first = function() {
			var myDefered = zhPromise.defer();
			myDefered.then(function() {
				lasExecuted = 1;
			});
			setTimeout(function() {
				myDefered.resolve(1);
			}, 1000);
			return myDefered;
		};
		promises.push(first);
		var second = function() {
			var myDefered = zhPromise.defer();
			myDefered.then(function() {
				expect(lasExecuted).toBe(1);
				lasExecuted = 2;
			});
			setTimeout(function() {
				myDefered.resolve(2);
			}, 10);
			return myDefered;
		};
		promises.push(second);
		var third = function() {
			var myDefered = zhPromise.defer();
			myDefered.then(function() {
				expect(lasExecuted).toBe(2);
				lasExecuted = 3;
			});
			setTimeout(function() {
				myDefered.resolve(2);
			}, 5);
			return myDefered;
		};
		promises.push(third);
		Util.doSync(promises, function() {
			processed = true;
			expect(lasExecuted).toBe(3);
			done();
		});

	});

	it('Util.isDateGt: success without format', function() {
		expect(Util.isDateGt("01/02/2015", "01/01/2015")).toEqual(true);
	});

	it('Util.isDateGt: error without format', function() {
		expect(Util.isDateGt("01/01/2015", "01/02/2015")).toEqual(false);
	});

	it('Util.isDateGt: success with format', function() {
		expect(Util.isDateGt("01/02/2015", "01/01/2015", "MM/DD/YYYY")).toEqual(true);
	});

	it('Util.isDateGt: error with format', function() {
		expect(Util.isDateGt("01/01/2015", "01/02/2015", "MM/DD/YYYY")).toEqual(false);
	});

	it('Util.isDateGte: success without format', function() {
		expect(Util.isDateGte("01/01/2015", "01/01/2015")).toEqual(true);
	});

	it('Util.isDateGte: error without format', function() {
		expect(Util.isDateGte("01/01/2015", "01/02/2015")).toEqual(false);
	});

	it('Util.isDateGte: success with format', function() {
		expect(Util.isDateGte("01/01/2015", "01/01/2015", "MM/DD/YYYY")).toEqual(true);
	});

	it('Util.isDateGte: error with format', function() {
		expect(Util.isDateGte("01/01/2015", "01/02/2015", "MM/DD/YYYY")).toEqual(false);
	});

	it('Util.isDateLt: success without format', function() {
		expect(Util.isDateLt("01/01/2015", "01/02/2015")).toEqual(true);
	});

	it('Util.isDateLt: error without format', function() {
		expect(Util.isDateLt("01/02/2015", "01/01/2015")).toEqual(false);
	});

	it('Util.isDateLt: success with format', function() {
		expect(Util.isDateLt("01/01/2015", "01/02/2015", "MM/DD/YYYY")).toEqual(true);
	});

	it('Util.isDateLt: error with format', function() {
		expect(Util.isDateLt("01/02/2015", "01/01/2015", "MM/DD/YYYY")).toEqual(false);
	});

	it('Util.isDateLte: success without format', function() {
		expect(Util.isDateLte("01/01/2015", "01/01/2015")).toEqual(true);
	});

	it('Util.isDateLte: error without format', function() {
		expect(Util.isDateLte("01/02/2015", "01/01/2015")).toEqual(false);
	});

	it('Util.isDateLte: success with format', function() {
		expect(Util.isDateLte("01/01/2015", "01/01/2015", "MM/DD/YYYY")).toEqual(true);
	});

	it('Util.isDateLte: error with format', function() {
		expect(Util.isDateLte("01/02/2015", "01/01/2015", "MM/DD/YYYY")).toEqual(false);
	});

	it('Util.isDateEqual: success without format', function() {
		expect(Util.isDateEqual("01/01/2015", "01/01/2015")).toEqual(true);
	});

	it('Util.isDateEqual: error without format', function() {
		expect(Util.isDateEqual("01/02/2015", "01/01/2015")).toEqual(false);
	});

	it('Util.isDateEqual: success with format', function() {
		expect(Util.isDateEqual("01/01/2015", "01/01/2015", "MM/DD/YYYY")).toEqual(true);
	});

	it('Util.isDateEqual: error with format', function() {
		expect(Util.isDateEqual("01/02/2015", "01/01/2015", "MM/DD/YYYY")).toEqual(false);
	});

	it('Util.isHourGt: success without format', function() {
		expect(Util.isHourGt("10:34", "09:34")).toEqual(true);
	});

	it('Util.isHourGt: error without format', function() {
		expect(Util.isHourGt("09:34", "10:34")).toEqual(false);
	});

	it('Util.isHourGt: success with format', function() {
		expect(Util.isHourGt("10:34", "09:34", "HH:mm")).toEqual(true);
	});

	it('Util.isHourGt: error with format', function() {
		expect(Util.isHourGt("09:34", "10:34", "HH:mm")).toEqual(false);
	});

	it('Util.isHourGte: success without format', function() {
		expect(Util.isHourGte("10:34", "10:34")).toEqual(true);
	});

	it('Util.isHourGte: error without format', function() {
		expect(Util.isHourGte("09:34", "10:34")).toEqual(false);
	});

	it('Util.isHourGte: success with format', function() {
		expect(Util.isHourGte("10:34", "10:34", "HH:mm")).toEqual(true);
	});

	it('Util.isHourGte: error with format', function() {
		expect(Util.isHourGte("09:34", "10:34", "HH:mm")).toEqual(false);
	});

	it('Util.isHourLt: success without format', function() {
		expect(Util.isHourLt("09:34", "10:34")).toEqual(true);
	});

	it('Util.isHourLt: error without format', function() {
		expect(Util.isHourLt("10:34", "09:34")).toEqual(false);
	});

	it('Util.isHourLt: success with format', function() {
		expect(Util.isHourLt("09:34", "10:34", "HH:mm")).toEqual(true);
	});

	it('Util.isHourLt: error with format', function() {
		expect(Util.isHourLt("10:34", "09:34", "HH:mm")).toEqual(false);
	});

	it('Util.isHourLte: success without format', function() {
		expect(Util.isHourLte("09:34", "09:34")).toEqual(true);
	});

	it('Util.isHourLte: error without format', function() {
		expect(Util.isHourLte("10:34", "09:34")).toEqual(false);
	});

	it('Util.isHourLte: success with format', function() {
		expect(Util.isHourLte("09:34", "09:34", "HH:mm")).toEqual(true);
	});

	it('Util.isHourLte: error with format', function() {
		expect(Util.isHourLte("10:34", "09:34", "HH:mm")).toEqual(false);
	});

	it('Util.isHourEqual: success without format', function() {
		expect(Util.isHourEqual("10:34", "10:34")).toEqual(true);
	});

	it('Util.isHourEqual: error without format', function() {
		expect(Util.isHourEqual("10:34", "09:34")).toEqual(false);
	});

	it('Util.isHourEqual: success with format', function() {
		expect(Util.isHourEqual("10:34", "10:34", "HH:mm")).toEqual(true);
	});

	it('Util.isHourEqual: error with format', function() {
		expect(Util.isHourEqual("10:34", "09:34", "HH:mm")).toEqual(false);
	});

	it('Util.isValidDate: success without format', function() {
		expect(Util.isValidDate("12/12/2015")).toEqual(true);
	});

	it('Util.isValidDate: error without format', function() {
		expect(Util.isValidDate("13/13/2015")).toEqual(false);
	});

	it('Util.isValidDate: success with format', function() {
		expect(Util.isValidDate("12/12/2015", "MM/DD/YYYY")).toEqual(true);
	});

	it('Util.isValidDate: error with format', function() {
		expect(Util.isValidDate("13/13/2015", "MM/DD/YYYY")).toEqual(false);
	});

	it('Util.isValidTime: success without format', function() {
		expect(Util.isValidTime("10:34")).toEqual(true);
	});

	it('Util.isValidTime: error without format', function() {
		expect(Util.isValidTime("26:00")).toEqual(false);
	});

	it('Util.isValidTime: success with format', function() {
		expect(Util.isValidTime("10:34", "HH:mm")).toEqual(true);
	});

	it('Util.isValidTime: error with format', function() {
		expect(Util.isValidTime("26:00", "HH:mm")).toEqual(false);
	});

	it('Util.isValidDateTime: success without format', function() {
		expect(Util.isValidDateTime("01/01/2015 10:20:30")).toEqual(true);
	});

	it('Util.isValidDateTime: error without format', function() {
		expect(Util.isValidDateTime("01/01/2015 26:20:30")).toEqual(false);
	});

	it('Util.isValidDateTime: success with format', function() {
		expect(Util.isValidDateTime("01/01/2015 10:20:30", "MM/DD/YYYY HH:mm:ss")).toEqual(true);
	});

	it('Util.isValidDateTime: error with format', function() {
		expect(Util.isValidDateTime("01/01/2015 26:20:30", "MM/DD/YYYY HH:mm:ss")).toEqual(false);
	});

	it('Util.isBetweenDate', function(){
		expect(Util.isBetweenDate("03/03/2015", "27/11/2015", "05/08/2015", "DD/MM/YYYY")).toEqual(true);
		expect(Util.isBetweenDate("Jan-2014", "Feb-2015", "Mar-2015", "MMM-YYYY")).toEqual(false);
		expect(Util.isBetweenDate("03/03/2012", "04/04/2015", "02/02/2012", "MM/DD/YYYY")).toEqual(false);
		expect(Util.isBetweenDate("11/03/2015", "10/11/2015", "13/08/2015", "MM/DD/YYYY")).toEqual(false);
	});
	
	it('Util.buildDebounceMethod: should call a function after specified milliseconds', function (done) {
		var callCount = 0;
		
		var method = Util.buildDebounceMethod(function () {
			callCount++;
		}, 10);
		
		method();
		
		setTimeout(function () {
			expect(callCount).toBe(1);
			done();
		}, 10);
	});
	
	/**
	 * I could not make this test. It work in chrome but not in phamtomjs. Need to check this later, maybe updating phamtomjs could work
	 */
	
	// it('Util.buildDebounceMethod: should call the function only once if it was called again before the time cap', function (done) {
	// 	var callCount = 0;
		
	// 	var method = Util.buildDebounceMethod(function () {
	// 		callCount++;
	// 	}, 10);
		
	// 	method();
		
	// 	setTimeout(function () {
	// 		method();
	// 	}, 5);
		
	// 	setTimeout(function () {
	// 		expect(callCount).toBe(1);
	// 		done();
	// 	}, 50);
	// });
	
	it('Util.buildDebounceMethod: should call the function twice if it was called again after the time cap', function (done) {
		var callCount = 0;
		
		var method = Util.buildDebounceMethod(function () {
			callCount++;
		}, 10);
		
		method();
		
		setTimeout(function () {
			method();
		}, 11);
		
		setTimeout(function () {
			expect(callCount).toBe(2);
			done();
		}, 50);
	});
	
	it('Util.buildDebounceMethod: should call the function twice too if the third parameter is true', function (done) {
		var callCount = 0;
		
		var method = Util.buildDebounceMethod(function () {
			callCount++;
		}, 5, true);
		
		method();
		
		setTimeout(function () {
			method();
		}, 10);
		
		setTimeout(function () {
			expect(callCount).toBe(2);
			done();
		}, 50);
	});
	
	it('Util.isEmptyObject: should return true if the property have no own key', function () {
		expect(Util.isEmptyObject({})).toBeTruthy();
		expect(Util.isEmptyObject({a: "foo"})).toBeFalsy();
	});
	
	it('Util.isNumber: should return true if the type of the value is number', function () {
		expect(Util.isNumber(1)).toBeTruthy();
		expect(Util.isNumber("a")).toBeFalsy();
		expect(Util.isNumber("42")).toBeFalsy();
	});
	
	it('Util.roundDecimal: should round the number to a specified number of decimals', function () {
		expect(Util.roundDecimal(1.69, 1)).toBe(1.7);
		expect(Util.roundDecimal(1.654, 2)).toBe(1.65);
		expect(Util.roundDecimal(1.655, 2)).toBe(1.66);
	});
	
	it('Util.getRightPath: should return an url with the component path', function () {
		expect(Util.getRightPath("test#uabafet.json", "json/datasources/")).toEqual("test#json/datasources/uabafet.json");
	});
	
	it('Util.getRightPath: should return an url with prefix plus componentName if there is no hash', function () {
		expect(Util.getRightPath("uabafet.json", "json/datasources/")).toEqual("json/datasources/uabafet.json");
	});
	
	it('Util.getPackage: should return the package from an uri', function () {
		expect(Util.getPackage("test#uabafet.json")).toEqual("test");
	});
	
	it('Util.getPackage: should return an empty string if there is no hash', function () {
		expect(Util.getPackage("uabafet.json")).toEqual("");
	});
	
	it('Util.getPackageUrl: should return the packageUrl from an uri', function () {
		expect(Util.getPackageUrl("test#uabafet.json")).toEqual("uabafet.json");
	});
	
	it('Util.getPackageUrl: should return an empty string if there is no hash in the uri', function () {
		expect(Util.getPackageUrl("uabafet.json")).toEqual("");
	});
	
	it('Util.range: should return from 0 to the specified end, if there is just one parameter', function () {
		expect(Util.range(2)).toEqual([0, 1]);
	});
	
	it('Util.range: should return from the start to the end', function () {
		expect(Util.range(5, 10)).toEqual([5, 6, 7, 8, 9]);
	});
	
});
