
// FILE: js/util/Utilities.js
function Utilities() {

	var self = this;

	this.frontVersion = "1.0.0";

	this.zerofillNumberFormat = function(number, length) {
		if(typeof number == 'number') {
			number = number.toString(); 
		}
		var baseZeroFill = "0";
		var zeroFilledStringLength = self.stringRepeat(baseZeroFill, length);
		var concatenadBasePlusNumber =  zeroFilledStringLength + number;
		return concatenadBasePlusNumber.slice(-length);
	};

	this.stringRepeat = function(string, qtd) { //the same as str.repeat() ES6
		var finalString = [];
		while(finalString.length < qtd){
			finalString.push(string);
		}
		return finalString.join('');
	};

	this.clearAllTimeOutsAndIntervals = function() {
        var lastTimeOutId = window.setTimeout(function(){}, 0);
        while (lastTimeOutId > 0) {
            clearTimeout(lastTimeOutId);
            clearInterval(lastTimeOutId);
            lastTimeOutId --;
        }
	};

	this.capitalizeFirstLetter = function(string) {
	    return string.charAt(0).toUpperCase() + string.slice(1);
	};

	this.uniqueArray = function(toBecomeUniqueArray) {
		return toBecomeUniqueArray.filter(function(item, pos) {
		    return toBecomeUniqueArray.indexOf(item) == pos;
		});
	};

	this.groupArrayByProperty = function(toGroupArray, property) {
		var groupedObject = {};
		toGroupArray.forEach(function(item) {
			if(!item[property]) {
				item[property] = null;
			}
			if(!groupedObject.hasOwnProperty(item[property])) {
				groupedObject[item[property]] = [];
			}
			groupedObject[item[property]].push(item);
		});
		return groupedObject;
	};

	String.prototype.moneyToFloat = function () {
		var treated = this.replace('R$', '');
		treated = treated.replace(' ', '');
		treated = treated.replace(',', '.');
		return parseFloat(treated);
	};

	Number.prototype.moneyToFloat = function () {
		return parseFloat(this);
	};

	String.prototype.replaceAt = function(index, character) {
	    return this.substr(0, index) + character + this.substr(index+character.length);
	};

	Number.prototype.toMoney = function () {
		var treated = "";
		var splited = this.toString().split('.');
		if (parseFloat(this).isInt()) {
			treated = this + ',00';
		} else if (parseFloat(splited[1]) < 10) {
			splited[1] += '0';
			treated = splited[0] + ',' + splited[1];
		} else {
			treated = this;
		}

		treated = treated.replace('.', ',');
		treated = 'R$ ' + treated;
		return treated;
	};

	Number.prototype.isInt = function () {
		return this % 1 === 0;
	};

	this.idealTextColor = function(bgColor) {
		// bgColor = tinycolor(bgColor);
		// var red = bgColor._r;
		// var green = bgColor._g;
		// var blue = bgColor._b;
		// var textColor;
		// if ((red*0.299 + green*0.587 + blue*0.114) > 203) {
		// 	textColor = '#000000';
		// } else {
		// 	textColor = '#ffffff';
		// }
		// return textColor;
		// return tinycolor.mostReadable(bgColor, ['#FFF', '#000']).toHexString();

		let textColor;
		const luminance = getLuminance(bgColor);
		if (luminance > 0.5) {
			textColor = '#000000'; // Set text color to black
		} else {
			textColor = '#FFFFFF'; // Set text color to white
  		}

		return textColor;
	};

	function getLuminance(color) {
		// Convert hex color to RGB
		const rgb = hexToRgb(color);
		// Calculate relative luminance
		const r = adjustGamma(rgb.r / 255);
		const g = adjustGamma(rgb.g / 255);
		const b = adjustGamma(rgb.b / 255);
		const luminance = 0.2126 * r + 0.7152 * g + 0.0722 * b;
		return luminance;
	  }
	  
	  // Convert a hex color to an RGB object
	function hexToRgb(hex) {
		const shorthandRegex = /^#?([a-f\d])([a-f\d])([a-f\d])$/i;
		hex = hex.replace(shorthandRegex, (m, r, g, b) => r + r + g + g + b + b);
		const result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
		return result ? { r: parseInt(result[1], 16), g: parseInt(result[2], 16), b: parseInt(result[3], 16) } : null;
	}
	  
	  // Adjust the gamma of a color component
	function adjustGamma(component) {
		if (component <= 0.03928) {
			return component / 12.92;
		} else {
			return Math.pow((component + 0.055) / 1.055, 2.4);
		}
	}

}

Configuration(function(ContextRegister) {
	ContextRegister.register('Utilities', Utilities);
});

// FILE: js/services/Environment.js
function Environment(localStorageEngine, RestEngine) {
    
    var self = this;    
    var ENVIRONMENT_KDS_FLAG = 'ENVIRONMENT_KDS';
    var ENVIRONMENT_LOCAL_ID = 'LOCAL_ID';

    self.projectConfig = null;
    self.kdsWidget = null;

	let localID = localStorage.getItem(ENVIRONMENT_LOCAL_ID);
	if (localID) {
		window.localID = localID;
		localStorage.removeItem(ENVIRONMENT_LOCAL_ID);
		ENVIRONMENT_KDS_FLAG = ENVIRONMENT_KDS_FLAG + window.localID;
	}

	window.changeLocalEnvironment = function () {
		if (window.localID) {
			let oldValue = localStorageEngine.getLocalVar(ENVIRONMENT_KDS_FLAG);
			ENVIRONMENT_KDS_FLAG = ENVIRONMENT_KDS_FLAG + window.localID;
			localStorageEngine.setLocalVar(ENVIRONMENT_KDS_FLAG, oldValue);	
		}
	};

    window.customKDSReload = function () {
        if (!window.localID) {
            location.reload();
        } else {
			let savedID = localStorage.getItem(ENVIRONMENT_LOCAL_ID);
			if (savedID && (savedID != window.localID)) {
				setTimeout(() => {
					window.customKDSReload();
				}, 1000);
			} else {
				localStorage.setItem(ENVIRONMENT_LOCAL_ID, window.localID);
				location.reload();
			}
			
        }
    };

    this.setKDSWidgetOnEnvironment = function(widget) {
        self.kdsWidget = widget;
    };

    this.getKDSWidget = function() {
        return self.kdsWidget;
    };
	
    this.setLocalEnvironment = function(localEnvironment) {
    	localEnvironment = JSON.stringify(localEnvironment);
        localStorageEngine.setLocalVar(ENVIRONMENT_KDS_FLAG, localEnvironment);
    };
    
	this.getLocalEnvironment = function() {
		var localEnvironment = localStorageEngine.getLocalVar(ENVIRONMENT_KDS_FLAG);
		if(localEnvironment) {
	    	return JSON.parse(localEnvironment);
		}
	};

    this.setProjectConfig = function() {
        return RestEngine.requestMetaData('config.json').then(function(configJSONData) {
            self.projectConfig = configJSONData.data;
            return configJSONData.data;
        });
    };

    this.getProjectConfig = function() {
        return self.projectConfig;
    };

    this.unsetLocalEnvironment = function() {
        localStorage.removeItem(ENVIRONMENT_KDS_FLAG);
    };

	this.getBranch = function(){
        var params = {
            requestType: "DataSet",
            dataset: {}
        };
        var getBranchRoute = '/branch/get';
        return RestEngine.post(getBranchRoute, params).then(function(response) {
            return response.dataset.branch;
        });
    };

    this.getBranchSectors = function(){
        var params = {
            requestType: "DataSet",
            dataset: {}
        };
        var getBranchSectorsRoute = '/sector/get';
        return RestEngine.post(getBranchSectorsRoute, params).then(function(response) {
            return response.dataset.sectors;
        });
    };

    this.getRegisterControlParams = function(CDSETOR){
        var params = {
            requestType: "Row",
            row: {
                'CDSETOR': CDSETOR
            }
        };
        var getControlParametersRoute = '/register/getControlParameters';
        return RestEngine.post(getControlParametersRoute, params).then(function(response) {
            return response.dataset.productionControlParams;
        });
    };
}

Configuration(function(ContextRegister) {
	ContextRegister.register('environment', Environment);
});

// FILE: js/services/SummaryItemsService.js
function SummaryItemsService(environment, RestEngine, ZHPromise) {

	var self = this;
	var summaryItems = [];
	var fixedItems = null;

	this.setFixedItems = function(fixedItemsValue) {
		fixedItems = fixedItemsValue;
	};

	this.setSummaryItems = function(summaryItemsValue){
		summaryItems = summaryItemsValue;
	};

	this.getSummaryItems = function(){
		return summaryItems;
	};

	this.sortOrders = function(order){
		if(order && order.IDRESUMOPRODKDS){
			return (order.IDRESUMOPRODKDS == 'FIXED' ? 0 : 1);
		} else {
			return -1;
		}
	};

	this.updateSummaryItems = function() {
		let widget = environment.getKDSWidget();
		getFixedItems().then( fixedItemsTreated =>{
			summaryItems = angular.copy(fixedItemsTreated);
			widget.dataSource.data.forEach(order => {
				if(order[widget.orderTimeField] !== 'wait' && ( order.IDCONDPEDPAG == 'N' || (order.IDCONDPEDPAG == 'S' &&  !!order.NRSEQMOVDLV && order.IDSITUVENDA == 'O'))) {
					if(order[widget.orderStatusField] !== 'C'){
						order[widget.orderItemsField].forEach(function(item) {
							let newSumItem = getSummaryItem(item, widget);
							if (!newSumItem) {
								newSumItem = {};
								newSumItem.DSAPELIDORESUMO = item.DSAPELIDORESUMO;
								newSumItem.CDPRODUTO = item.CDPRODUTO;
								newSumItem.IDRESUMOPRODKDS = item.IDRESUMOPRODKDS;
								newSumItem.QTDESPERA = 0;
								newSumItem.QTDPRODUCAO = 0;
								newSumItem.IDAUMENTARESUMO = item.IDAUMENTARESUMO;
								// newSumItem.IDSOMRESUMO = item.IDSOMRESUMO;
								summaryItems.push(newSumItem);
							}
							if (item[widget.orderItemStatusField] === widget.itemNotStartedStatus) {
								newSumItem.QTDESPERA += parseFloat(item[widget.orderItemQttyField]);
							} else {
								newSumItem.QTDPRODUCAO += parseFloat(item[widget.orderItemQttyField]);
							}
						});
					}
				}
			});
		});
	};

	function getSummaryItem (item, widget) {
		return summaryItems.filter(sumItem => {
			return sumItem.CDPRODUTO == item.CDPRODUTO;
		}).shift();
	}

	function _getFixedItems(CDFILIAL, CDSETOR){
		var params = {
			requestType: "Row",
			row: {
					'CDSETOR': CDSETOR,
					'CDFILIAL': CDFILIAL
			}
		};
		var getSummaryRoute = '/getFixedSummaryItems';
		return RestEngine.post(getSummaryRoute, params).then(function(response) {
			return response.dataset.summary.fixedItems;
		});
	}

	function getFixedItems() {
		var deferred = ZHPromise.defer();
		let localEnvironment = environment.getLocalEnvironment();
		if(fixedItems === null && localEnvironment){
			self.setFixedItems(_getFixedItems(localEnvironment.CDFILIAL, localEnvironment.CDSETOR));
		}
 		deferred.resolve(fixedItems);
 		return deferred.promise;
	}

}

Configuration(function(ContextRegister) {
	ContextRegister.register('SummaryItemsService', SummaryItemsService);
});

// FILE: js/services/SoundService.js
function SoundService(environment) {

	var audioArray = {};
	var beepPath;
	var self = this;
	var initialInfo = environment.getLocalEnvironment();
	var continuosBeepArray = {};

	this.configureBeep = function() {
		var audio;
		let SOMBEEP;
		if (initialInfo && initialInfo.SOMBEEP) {
			let idx = initialInfo.SOMBEEP.indexOf('kds') !== -1 ? initialInfo.SOMBEEP.indexOf('kds') : initialInfo.SOMBEEP.indexOf('KDS');
			if (idx !== -1) {
				SOMBEEP = initialInfo.SOMBEEP.substr(idx);
				beepPath = `http://${window.location.hostname}:${window.location.port}/${SOMBEEP}`;
			} else {
				beepPath = "assets/beep.mp3";
			}
		} else {
			beepPath = "assets/beep.mp3";
		}
		audio = new Audio(beepPath);
		audioArray[beepPath] = audio;
		return audioArray;
	};

	this.beep = function(continuosBeep) {
		var audio = audioArray[beepPath];
		var widget = environment.getKDSWidget();
		if(!widget.beeping && continuosBeep) {
			widget.beeping = setInterval(function() {
				if(isSomeOrderBeeping(widget)) {
					try{
						audio.play();
					}catch(error){
						console.log(error);
					}
				}
			}, 1000);
		} else if(!continuosBeep) {
			try{
				audio.play();
			}catch(error){
				console.log(error);
			}
		}
	};

	function isSomeOrderBeeping(widget) {
		var orders = widget.dataSource.data;
		return orders.some(function(order) {
			return order.isOrderBeeping;
		});
	}

	this.playSound = function(audioLink, continuosBeepName){
		var soundPath;
		if (audioLink) {
			let idx = audioLink.indexOf('kds') !== -1 ? audioLink.indexOf('kds') : audioLink.indexOf('KDS');
			if (idx !== -1) {
				audioLink = audioLink.substr(idx);
				soundPath = `http://${window.location.hostname}:${window.location.port}/${audioLink}`;
			} else {
				soundPath = "assets/beep.mp3";
			}
		} else {
			soundPath = "assets/beep.mp3";
		}

		if (!audioArray[soundPath]) {
			var audio =  new Audio(soundPath);
			audioArray[soundPath] = audio;
		}
		try{
			if(continuosBeepName && !continuosBeepArray[continuosBeepName]){
				audioArray[soundPath].play();
				continuosBeepArray[continuosBeepName] = setInterval(function() {
					if(someOrderStillBeeping(continuosBeepName)){
						try{
							audioArray[soundPath].play();
						}catch(error){
							console.log(error);
						}
					}else{
						clearInterval(continuosBeepArray[continuosBeepName]);
						continuosBeepArray[continuosBeepName] = null;
					}
				}, 1000);
			}
		}catch(error){
			console.log(error);
		}

		function someOrderStillBeeping(continuosBeepName) {
			var orders = environment.getKDSWidget().dataSource.data;
			return orders.some(function(order) {
				return order[continuosBeepName];
			});
		}
	};

}

Configuration(function(ContextRegister) {
	ContextRegister.register('SoundService', SoundService);
});

// FILE: js/services/ScreenMessagesService.js
function ScreenMessagesService(environment, SoundService, $timeout) {

    var self = this;
    self.messages = [];
    self.waitingMessages = [];
    self.deletedMessages = [];
    self.messageOnScreen = null;
    self.timeoutFunction = null;

    // NOW THE TIMEOUTS WALK TOGETHER
    this.getTimeUntilNextMinute = function (time){
        var timeUntilNextMinute = 0;
        if(time > 0){
            timeUntilNextMinute = moment();
            time = time * 60 * 1000;
            timeUntilNextMinute = (time - ((timeUntilNextMinute.seconds() * 1000) + timeUntilNextMinute.milliseconds()));
        }
        return timeUntilNextMinute;
    };
    this.clearMessages = function(){
        self.waitingMessages.forEach(function(message){
            if(message && message.timeoutFunction && typeof message.timeoutFunction.cancel === 'function') {
                message.timeoutFunction.cancel();
            }
        });
        self.messages = [];
        self.waitingMessages = [];
    };

    this.onNewMessage = function(messageData) {
        if(messageData){
            messageData.forEach(function(currentMessage){
                $timeout(function(){
                    if(currentMessage.DSMENSAGEM){
                        var messageToUpdate = self.messages.filter(function(filtered){
                            return currentMessage.CDMENSAGEM == filtered.CDMENSAGEM;
                        }).shift();
                        if(messageToUpdate){
                            messageToUpdate = currentMessage;

                            messageToUpdate = self.waitingMessages.filter(function(filtered){
                                return currentMessage.CDMENSAGEM == filtered.CDMENSAGEM;
                            }).shift();
                            if(messageToUpdate){
                                messageToUpdate.timeoutFunction.cancel();
                                self.waitingMessages.splice(self.waitingMessages.indexOf(messageToUpdate), 1);
                                self.doomOrSaveMessage(currentMessage, currentMessage.DSMENSAGEM.length/3 > 30 ? currentMessage.DSMENSAGEM.length/3 : 30);
                            }
                        }else{
                            self.doomOrSaveMessage(currentMessage, currentMessage.DSMENSAGEM.length/3 > 30 ? currentMessage.DSMENSAGEM.length/3 : 30);
                            self.messages.push(currentMessage);
                        }
                        if(!self.messagePlayed){
                            self.playNextMessage();
                        }
                    }
                }, self.getTimeUntilNextMinute(currentMessage.timeout), true);
            });
        }
    };

    this.onDeleteMessage = function(messageData) {
        self.deletedMessages = messageData;
        if(messageData){
            messageData.forEach(function(currentMessage){
                var messageToDelete = self.messages.filter(function(filtered){
                        return currentMessage == filtered.CDMENSAGEM;
                    }).shift();
                if(messageToDelete){
                    this._eraseMessage(messageToDelete);
                }
                messageToDelete = self.waitingMessages.filter(function(filtered){
                        return currentMessage == filtered.CDMENSAGEM;
                    }).shift();
                if(messageToDelete){
                    self.waitingMessages.splice(self.waitingMessages.indexOf(messageToDelete), 1);
                }
                deletedMessages.splice(self.deletedMessages.indexOf(messageToDelete.CDMENSAGEM), 1);
            });
        }
    };
    this.getMessage = function(){
        return self.messageOnScreen != null ? [self.messageOnScreen] : [];
    };

    this.playNextMessage = function(){
        self.messageOnScreen = null;
        var kdsWidget = environment.getKDSWidget();
        if(!kdsWidget.frozen){
            self.messagePlayed = true;
            if(self.messages.length > 0){
                var temp = null;
                if(self.messages[0].timeoutFunction){
                    temp = self.messages[0].timeoutFunction;
                    delete self.messages[0].timeoutFunction;
                }
                var beforeClone = self.messages.shift();
                self.messageOnScreen = Util.clone(beforeClone);
                beforeClone.timeoutFunction = temp;
                var now = moment();
                self.messageOnScreen.DSMENSAGEM = self.messageOnScreen.DSMENSAGEM + ' ' + String('00'+now.hours()).slice(-2) +":"+ String('00'+now.minutes()).slice(-2);
                //timeout que roda a fila
                var time = self.messageOnScreen.DSMENSAGEM.length/3 > 30 ? self.messageOnScreen.DSMENSAGEM.length/3 : 30;
                if(self.messageOnScreen.sound){
                    SoundService.playSound(self.messageOnScreen.sound);
                }
                self.timeoutFunction = $timeout(function(){
                    self.playNextMessage();
                }, time*1000, true);
            }else{
                self.messagePlayed = false;
            }
        }
    };

    this._eraseMessage = function(message){
        if(self.messages.indexOf(message) > -1){
            self.messages.splice(self.messages.indexOf(message), 1);
        }
    };


    this.doomOrSaveMessage = function(message, animationTime){
        if(message.NRREPTINTERV && (message.NRQTDREPTINTERV > 0)){
            //uma delicia campo varchar pra int
            if(typeof message.NRQTDREPTINTERV == "number" || (typeof message.NRQTDREPTINTERV == "string" && parseInt(message.NRQTDREPTINTERV)!= "NaN")){
                message.NRQTDREPTINTERV = parseInt(message.NRQTDREPTINTERV);
            }else{
                message.NRQTDREPTINTERV = 1;
            }
            if(typeof message.NRREPTINTERV == "number" || (typeof message.NRREPTINTERV == "string" && parseInt(message.NRREPTINTERV)!= "NaN")){
                message.NRREPTINTERV = parseInt(message.NRREPTINTERV);
            }else{
                message.NRREPTINTERV = animationTime/60;
            }
            message.NRQTDREPTINTERV--;
            //timeout de repetição
            message.timeoutFunction = $timeout(function() {
                self.messages.push(message);
                self.doomOrSaveMessage(message, message.DSMENSAGEM.length/3 > 30 ? message.DSMENSAGEM.length/3 : 30);

                if(!self.messagePlayed){
                    self.playNextMessage();
                }
            }.bind(message), (message.NRREPTINTERV*60) * 1000, true);

            var messageToUpdate = self.waitingMessages.filter(function(filtered){
                return message.CDMENSAGEM == filtered.CDMENSAGEM;
            }).shift();
            if(!messageToUpdate){
                self.waitingMessages.push(message);
            }
        }else{
            if(~self.waitingMessages.indexOf(message)){
                self.waitingMessages.splice(self.waitingMessages.indexOf(message), 1);
            }
        }
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('ScreenMessagesService', ScreenMessagesService);
});

// FILE: js/services/NavigatorService.js
function NavigatorService(environment) {

	var self = this;
	var teste = 1;

	this.setOrderAsSelected = function(orderIndex, keepOrMove, upDownFlag) {
        var widget = environment.getKDSWidget();
        self.unSelectAllOrders();
		const ordersOnDS = this.filterOrders(widget.dataSource.data);
        if(~orderIndex) {
        	var isOrderTheLastOfDataSource = orderIndex == this.filterOrders(widget.dataSource.data).length - 1;
        	if(upDownFlag){
    			if(upDownFlag == 'down'){
	        		if (isOrderTheLastOfDataSource){
	        			self.setFirstOrderAsSelected();
	        			orderIndex = -1;
	        		}else{
		        		orderIndex++;
	        		}
        		}else{
        			if(orderIndex === 0){
        				self.setLastOrderAsSelected();
	        			orderIndex = -1;
        			}else{
	        			orderIndex--;
        			}
	        	}
        	}else{
        		if (isOrderTheLastOfDataSource){
	        		if(keepOrMove =='move'){
		        		orderIndex--;
		        		upDownFlag = 'up';
	        		}
	        	}else{
	        		orderIndex++;
	        		upDownFlag = 'down';        	
	        	}
        	}
            while(~orderIndex && ordersOnDS[orderIndex] && ordersOnDS[orderIndex].isInConclusionQueue) {
            	isOrderTheLastOfDataSource = orderIndex == ordersOnDS.length - 1;
            	if(isOrderTheLastOfDataSource) {
        			upDownFlag = 'up';
        			orderIndex--;
            	} else {
					if(upDownFlag == 'up') {
						orderIndex--;
					} else {
						orderIndex++;
					}
            	}
            }
        }
        if(~orderIndex && ordersOnDS[orderIndex]){
        	ordersOnDS[orderIndex].isSelected = true;
        }
        widget.updateKDSTemplate();
        self.scrollAfterSelectOtherOrder();

    };

    this.setPreviousOrderAsSelected = function(lastSelectedOrderIndex) {
		var currentOrder = self.getSelectedOrder();
		var currentOrderIndex = lastSelectedOrderIndex || self.getOrderIndex(currentOrder);
		var widget = environment.getKDSWidget();
		if(~currentOrderIndex) {
			self.setOrderAsSelected(currentOrderIndex, 'keep', 'up');
		} else {
			self.setFirstOrderAsSelected();
		}
	};

	this.setLastOrderAsSelected = function() {
		var widget = environment.getKDSWidget();
		self.setOrderAsSelected(this.filterOrders(widget.dataSource.data).length - 1, 'keep');
	};

	this.setFirstOrderAsSelected = function() {
		var widget = environment.getKDSWidget();
		if(this.filterOrders(widget.dataSource.data)[0]){
			if(this.filterOrders(widget.dataSource.data)[0].isInConclusionQueue){
				self.setOrderAsSelected(0, 'keep');
			} else {
				this.filterOrders(widget.dataSource.data)[0].isSelected = true;
			}
		}
		
	};

	this.setNextOrderAsSelected = function(lastSelectedOrderIndex = null) {
		var widget = environment.getKDSWidget();
		var currentOrder = self.getSelectedOrder();
		var currentOrderIndex = lastSelectedOrderIndex || self.getOrderIndex(currentOrder);
		if(~currentOrderIndex) {
			self.setOrderAsSelected(currentOrderIndex, 'keep', 'down');
		} else {
			self.setFirstOrderAsSelected();
		}

		if(!self.getSelectedOrder()) {
			self.setFirstOrderAsSelected();
		}

	};

	this.unSelectAllOrders = function() {
		var widget = environment.getKDSWidget();
		this.filterOrders(widget.dataSource.data).forEach(function(order) {
			self.unSelectOrder(order);
		});
	};

	this.unSelectOrder = function(order) {
		order.isSelected = false;
	};

	this.hasSelectOrder = function() {
		var widget = environment.getKDSWidget();
		return this.filterOrders(widget.dataSource.data).some(function(order){
			return order.isSelected;
		});
	};

	this.getSelectedOrder = function() {
		var widget = environment.getKDSWidget();
		return this.filterOrders(widget.dataSource.data).find(function(order){
			return order.isSelected;
		});
	};

	this.getOrderIndex = function(order) {
		var widget = environment.getKDSWidget();
		return this.filterOrders(widget.dataSource.data).indexOf(order);
	};

	this.scrollAfterSelectOtherOrder = function(){

		let orderContainerPriority = $('#priority-orders');
		let orderContainerdefault = $('#default-orders');
		var kdsContainer = null;
		var selectedOrder = null;
		if(orderContainerdefault.hasClass("selected-container")) {
			kdsContainer = orderContainerdefault;
			selectedOrder = kdsContainer.find('.selected');
			if(selectedOrder.length > 0) {
				let leftOrderOffset = selectedOrder.offset().left;
				let orderWidth = selectedOrder.outerWidth();		
				let rigthOrderOffset = leftOrderOffset + orderWidth;
				let containerWidth = kdsContainer.outerWidth();
	
				let gapBetweenOrders = 10;
				let containerLeftPadding = 12;
				let toScrollValue;
	
				// Rules to check when the scroll should move to the right
				if (rigthOrderOffset > containerWidth || selectedOrder.find('.kds-item').last().offset().left + selectedOrder.outerWidth() > containerWidth) {
					toScrollValue = rigthOrderOffset - containerWidth + gapBetweenOrders;
					// If the order haves items on the other side the size of the box is added to the scroll distante
					if (selectedOrder.find('.kds-item-group').last().offset().left + selectedOrder.outerWidth() > containerWidth) {
						toScrollValue = selectedOrder.find('.kds-item').last().offset().left + selectedOrder.outerWidth() - containerWidth + gapBetweenOrders;
					}
	
				} else if(leftOrderOffset < containerLeftPadding || selectedOrder.offset().top < 0) {
					toScrollValue =  leftOrderOffset - gapBetweenOrders;
					if (selectedOrder.offset().top < 0) {
						toScrollValue -= selectedOrder.width() - gapBetweenOrders;
					}
				}
	
				if(toScrollValue) {
					kdsContainer.animate({
						scrollLeft: '+=' + toScrollValue
					}, 100);
				}
			}
		} else if (orderContainerPriority.hasClass("selected-container")) {
			kdsContainer = orderContainerPriority;
			selectedOrder = kdsContainer.find('.selected');
			if(selectedOrder.length > 0) {
				let topOrderOffset = selectedOrder.offset().top;
				let orderHeight = selectedOrder.outerHeight();		
				let downOrderOffset = topOrderOffset + orderHeight;
				let containerHeight = kdsContainer.outerHeight();
	
				let gapBetweenOrders = 10;
				let containerTopPadding = 12;
				let toScrollValue;
	
				// Rules to check when the scroll should move to the right
				if (downOrderOffset > containerHeight || selectedOrder.find('.kds-item').last().offset().top + selectedOrder.outerHeight() > containerHeight) {
					toScrollValue = topOrderOffset - containerHeight + gapBetweenOrders;
					// If the order haves items on the other side the size of the box is added to the scroll distante
					if (selectedOrder.find('.kds-item-group').last().offset().top + selectedOrder.outerHeight() > containerHeight) {
						toScrollValue = selectedOrder.find('.kds-item').last().offset().top + selectedOrder.outerHeight() - containerHeight + gapBetweenOrders;
					}
	
				} else if(downOrderOffset < containerTopPadding || selectedOrder.offset().top < 0) {
					toScrollValue =  topOrderOffset - gapBetweenOrders;
					if (selectedOrder.offset().top < 0) {
						toScrollValue -= selectedOrder.height() - gapBetweenOrders;
					}
				}
	
				if(toScrollValue) {
					kdsContainer.animate({
						scrollTop: '+=' + toScrollValue
					}, 100);
				}
			}
		}
	};

	//------------------------------------------Zoom navigation

	this.setFirstItemAsSelected = function(order) {
		var widget = environment.getKDSWidget();
		unSelectAllSelectedItems(widget.zoomItem);
		widget.zoomItem[0][0].isZoomSelected = true;
		widget.zoomSelectedOrderKey = widget.zoomItem[0][0].orderKey;
	};

	this.getSelectedOrderItem = function(order) {
		var widget = environment.getKDSWidget();
		var selectedItem;
		if(order && Util.isArray(order)) {
			order.forEach(function(position) {
				position.forEach(function(item) {
					if(item.isZoomSelected) {
						selectedItem = item;
					}else{
						if(item.items) {
							item.items.forEach(function(comboItem) {
								if(comboItem.isZoomSelected) {
									if(item.items.length == 1){
										selectedItem = item;
									}else{
										selectedItem = comboItem;
									}
								}
							});
						}
					}
				});
			});
		}
		return selectedItem;
	};

	this.setInConclusionOnDataSource = function (item){
		var widget = environment.getKDSWidget();
		var currentOrder = self.getSelectedOrder();
		var CDPRODPROMOKDS;
		if(item.items){
			CDPRODPROMOKDS = item.CDPRODPROMOKDS;
		}
		currentOrder.items.forEach(function(dataItem){
			if(item.orderKey == dataItem.orderKey || (CDPRODPROMOKDS && CDPRODPROMOKDS == dataItem.CDPRODPROMOKDS)){
				dataItem.isInConclusionQueue = true;
			}
		});
	};

	function unSelectAllSelectedItems(order) {
		if(order) {
			order.forEach(function(position) {
				position.forEach(function(item) {
					item.isZoomSelected = false;
					if(item.items) {
						item.items.forEach(function(comboItem) {
							comboItem.isZoomSelected = false;
						});
					}
				});
			});
		}
	}

	function setZoomItemAsSelected(subZoomItem, zoomItem, orderKey, next){
		subZoomItem.isZoomSelected = true;
		if(!subZoomItem.isInConclusionQueue){
			var widget = environment.getKDSWidget();
			widget.zoomSelectedOrderKey = subZoomItem.orderKey;
		}else{
			self.setSelectedByOrderKeyZoom(zoomItem, subZoomItem.orderKey, next);
		}
	}

	this.setSelectedByOrderKeyZoom = function(zoomItem, orderKey, next){
		//next is true if is the next, false if is the prev
		zoomItem.forEach(function(position, positionIndex, positions){
			position.forEach(function(item, itemIndex, positionItems){
				//pra baixo
				if(next){
					if(orderKey == item.orderKey && item.isZoomSelected){
						item.isZoomSelected = false;
						if(item.items){
							setZoomItemAsSelected(item.items[0], zoomItem, orderKey, next);
						}else{
							if(positionItems[itemIndex + 1]){
								setZoomItemAsSelected(positionItems[itemIndex + 1], zoomItem, orderKey, next);
							}else if(!positions[positionIndex + 1]){
								setZoomItemAsSelected(positions[0][0], zoomItem, orderKey, next);
							}else{
								setZoomItemAsSelected(positions[positionIndex + 1][0], zoomItem, orderKey, next);
							}
						}
					}else{
						if(item.items) {
							item.items.forEach(function(compItem, compItemIndex, items){
								if(orderKey == compItem.orderKey && compItem.isZoomSelected){
									compItem.isZoomSelected = false;
									if(items[compItemIndex+1]){
										setZoomItemAsSelected(items[compItemIndex+1], zoomItem, orderKey, next);
									}else{
										if(positionItems[itemIndex + 1]){
											setZoomItemAsSelected(positionItems[itemIndex + 1], zoomItem, orderKey, next);
										}else if(!positions[positionIndex + 1]){
											setZoomItemAsSelected(positions[0][0], zoomItem, orderKey, next);
										}else{
											setZoomItemAsSelected(positions[positionIndex + 1][0], zoomItem, orderKey, next);
										}
									}
								}
							});
						}
					}
				}else{
				//pra cima
					if(orderKey == item.orderKey && item.isZoomSelected){
						item.isZoomSelected = false;
						var prevItem = positionItems[itemIndex - 1];
						if(prevItem){
							if(prevItem.items){
								setZoomItemAsSelected(prevItem.items[prevItem.items.length-1], zoomItem, orderKey, next);
							}else{
								setZoomItemAsSelected(prevItem, zoomItem, orderKey, next);
							}
						}else if(!positions[positionIndex - 1]){
							prevItem = positions[positions.length - 1][positions[positions.length - 1].length - 1];
							if(prevItem.items){
								setZoomItemAsSelected(prevItem.items[prevItem.items.length-1], zoomItem, orderKey, next);
							}else{
								setZoomItemAsSelected(prevItem, zoomItem, orderKey, next);
							}
						}else{
							prevItem = positions[positionIndex - 1][positions[positionIndex - 1].length -1];
							if(prevItem.items){
								setZoomItemAsSelected(prevItem.items[prevItem.items.length-1], zoomItem, orderKey, next);
							}else{
								setZoomItemAsSelected(prevItem, zoomItem, orderKey, next);
							}
						}
					}else{
						if(item.items){
							item.items.forEach(function(compItem, compItemIndex, items){
								if(orderKey == compItem.orderKey && compItem.isZoomSelected){
									compItem.isZoomSelected = false;
									if(items[compItemIndex - 1]){
										setZoomItemAsSelected(items[compItemIndex - 1], zoomItem, orderKey, next);
									}else{
										setZoomItemAsSelected(item, zoomItem, orderKey, next);
									}
								}
							});
						}
					}
				}
			});
		});
	
	};


	this.unselectAllOrdersItems = function(order) {
		var widget = environment.getKDSWidget();
		order = widget.zoomItem;
		order.forEach(function(position){
			self.unselectItemOrder(position, false);
		});
	};

	this.unselectItemOrder = function(item, selectNext) {
		if(selectNext) {
			item.isZoomSelected = true;
		} else {
			if(item.isZoomSelected) {
				item.isZoomSelected = false;
				self.this.unselectItemOrder(comboItem, item.isZoomSelected);
			}
			if(item.items) {
				item.items.forEach(function(comboItem) {
					self.this.unselectItemOrder(comboItem, comboItem.isZoomSelected);
				});
			} else {
				item.isZoomSelected = false;
			}
		}
	};

	this.handleScrollbarOnZoomDown = function () {
		var zoomContainer = $('.kds-zoom');
		var zoomContainerBottomOffset = zoomContainer.offset().top + zoomContainer.outerHeight(true);
		var selectedElement = zoomContainer.find('.selected');
		if (selectedElement[0]) {
			var selectedElementBottomOffset = selectedElement.offset().top + selectedElement.outerHeight(true);
			if (selectedElementBottomOffset > zoomContainerBottomOffset) {
				zoomContainer.scrollTop(zoomContainer.scrollTop() + selectedElementBottomOffset - zoomContainerBottomOffset);
			}
			if (selectedElement.offset().top - zoomContainer.offset().top < 0) {
				zoomContainer.scrollTop(0);
			}
		}
	};

	this.filterOrders = function(orders){
		let orderContainerPriority = document.getElementById('priority-orders') || false;
		let orderContainerdefault = document.getElementById('default-orders') || false;
		let IDHABPRIORIDADE = environment.getLocalEnvironment().IDHABPRIORIDADE || 'N';
		if (IDHABPRIORIDADE == 'S') {
			if(orderContainerdefault && orderContainerdefault.classList.contains("selected-container")) {
				return this.filterDefaultOrders(orders);
			} else if (orderContainerPriority && orderContainerPriority.classList.contains("selected-container")) {
				let filteredOrders = this.filterPriorityOrders(orders);
				if (filteredOrders && filteredOrders.length > 0) {
					return filteredOrders;
				} else {
					orderContainerdefault && orderContainerPriority.classList.remove("selected-container"); // jshint ignore:line
					orderContainerdefault && orderContainerdefault.classList.add("selected-container"); // jshint ignore:line
					return this.filterDefaultOrders(orders);
				}
			} else {
				orderContainerdefault && orderContainerdefault.classList.add("selected-container"); // jshint ignore:line
				return this.filterVisibleOrders(orders);
			}
		} else {
			return this.filterVisibleOrders(orders);
		}
	};

	this.filterVisibleOrders = function(orders){
		let newOrders = [];
		let filtredOrders = orders.filter( order => {
			return order.hide === false; //Alterado devido aos pedidos não visiveis estarem interferindo na navegação
		});
		filtredOrders.forEach(order => {
			if(newOrders.indexOf(order) < 0) {
				newOrders.push(order);
			}
		});
	
		if(filtredOrders.length != newOrders.length){
		  console.log('..');
		}
		return newOrders;
	};

	this.filterPriorityOrders = function (orders) {
		let newOrders = [];
		let filtredOrders = orders.filter( order => {
			return order.hide === false && this.isOrderPriority(order); //Alterado devido aos pedidos não visiveis estarem interferindo na navegação
		});
		filtredOrders.forEach(order => {
			if(newOrders.indexOf(order) < 0) {
				newOrders.push(order);
			}
		});
	
		if(filtredOrders.length != newOrders.length){
		  console.log('..');
		}
		return newOrders;
	};
	this.filterDefaultOrders = function (orders) {
		let newOrders = [];
		let filtredOrders = orders.filter( order => {
			return order.hide === false && !this.isOrderPriority(order); //Alterado devido aos pedidos não visiveis estarem interferindo na navegação
		});
		filtredOrders.forEach(order => {
			if(newOrders.indexOf(order) < 0) {
				newOrders.push(order);
			}
		});
	
		if(filtredOrders.length != newOrders.length){
		  console.log('..');
		}
		return newOrders;
	};
	this.isOrderPriority = function(order) {
		if (order && order.IDPRODPRIORIDADE == false) {
			return false;
		}
		return true;
	};

	this.handleScrollbarOnZoomUp = function () {
		var zoomContainer = $('.kds-zoom');
		var zoomContainerTopOffset = zoomContainer.offset().top;
		var selectedElement = zoomContainer.find('.selected');
		if (selectedElement[0]) {
			var selectedElementTopOffset = selectedElement.offset().top;
			if (selectedElementTopOffset < zoomContainerTopOffset) {
				zoomContainer.scrollTop(zoomContainer.scrollTop() - (zoomContainerTopOffset - selectedElementTopOffset));
			} 
			if (selectedElementTopOffset + selectedElement.outerHeight(true) >= $('.kds-zoom').outerHeight(true) + zoomContainerTopOffset) {
				zoomContainer.scrollTop(selectedElementTopOffset + selectedElement.outerHeight(true));
			}
		}
	};

}

Configuration(function(ContextRegister) {
	ContextRegister.register('NavigatorService', NavigatorService);
});

// FILE: js/services/SocketService.js
function SocketService(ZhSocketFactory, ScreenService, ApplicationContext, environment, eventAggregator, ScreenMessagesService, localStorageEngine) {
	var self = this;
	// var host = 'ws://odh-pdt014:8080';
	var host = 'ws://' + window.location.hostname + ':';
	var webSocket = null;
	var securityKey = null;
	var socketEvents = {
		LOGIN_BROWSER      : 'loginBrowser',
		DELETE_ORDERS      : 'deleteOrders',
		LOGOFF_BROWSER     : 'logoffBrowser',
		PLAY_REQUEST       : 'playRequest',
		CONCLUDE_REQUEST   : 'concludeRequest',
		EXPEDITION_REQUEST : 'expeditionRequest',
		CLEAR_SCREEN       : 'clearScreen',
		GET_DATA           : 'getData',
		PING_EVENT         : 'ping',
		FREEZE             : 'freezeUpdates',
		UNFREEZE           : 'unfreezeUpdates',
		PRODUCTIONLINE     : 'productionLine'

	};

    var mutex = true;
    var intervalPing;
    var pingTime = 10000;
    var NUMBER_OF_PINGS_TO_RESYNCRONIZE = 2;
    var inLogoff = false;
    self.lastPong = null;
    var isConnected = false;

	this.startWebSocket = function() {
		var text = null;
		var port = null;
		var xhr = new XMLHttpRequest();
		var getipAddress = localStorageEngine.getLocalVar('kdsserverdns');
		var path = getipAddress ? (getipAddress.ip + ':' + getipAddress.port) : ("http://" + window.location.hostname + ':' + window.location.port);
		var assistant_updater = window.location.href.indexOf('/main') === -1 ? '' : '/main';
		xhr.open("GET", path + "/kds" + assistant_updater + "/backend/environment.xml");
		xhr.responseType = "text";//force the HTTP response, response-type header to be blob
		xhr.onload = function()
		{
		    text = xhr.response;//xhr.response is now a blob object
	     	if (~text.indexOf('wsport')) {
		     	port = text.substr(text.indexOf('wsport') + 8);//+ 'wsport">'.length
		     	port = port.substr(0, port.indexOf('</'));
	     	} else {
	     		port = '8080';
			}
			path = getipAddress ? (getipAddress.ip.replace('http://','').replace('https://','')) : (window.location.hostname);
			host = 'ws://' + path + ':' + port;
			createWebSocket();
			addWebSocketListeners();
		};
		xhr.send();

		/*if (intervalPing == undefined) {
			intervalPing = setInterval(self.ping, pingTime);
		}*/
	};

	function createWebSocket() {
		webSocket = ZhSocketFactory.create(socketClientFactory, host, ApplicationContext);
	}

	function socketClientFactory() {
		return new SocketResponseHandler(eventAggregator);
	}

	function addWebSocketListeners() {
		webSocket.addListener({
			connectionLost : function(){
				if (!inLogoff) {
					setTimeout(()=> {
						if (!inLogoff) {
							ScreenService.notificationMessage("Sem conexão com o servidor.", 'error', 4000);
						}
					}, 5000);
				} else {
					inLogoff = !inLogoff;
				}
				self.lastPong = null;
				isConnected = false;
			},
			onOpen :  function(){
				isConnected = true;
				var environmentParams = environment.getLocalEnvironment();
				if(environmentParams && environmentParams.CDFILIAL && environmentParams.CDSETOR){
					self.loginBrowser(environmentParams.CDFILIAL, environmentParams.CDSETOR, environmentParams.IDTIPOSETOR);
					ScreenMessagesService.clearMessages();
					if(!environmentParams.cleanRequestsOnLogin){
						setTimeout(function(){self.getData(environmentParams.CDFILIAL, environmentParams.CDSETOR, environmentParams.IDTIPOSETOR);}, 1000);
					}
				}
			}
		});
	}

	this.isConnected = function() {
		return isConnected;
	};

	this.isOpen = function() {
		return webSocket.isOpen();
	};

	this.loginBrowser = function(CDFILIAL, CDSETOR, IDTIPOSETOR) {
		var date = new Date();
		securityKey = CDFILIAL + '-' + CDSETOR + '-' + date.getMilliseconds() + Math.floor((Math.random() * 100) + 1);
		this.send({
			event: socketEvents.LOGIN_BROWSER,
			data: {
				'CDFILIAL': CDFILIAL,
				'CDSETOR': CDSETOR,
				'IDTIPOSETOR': IDTIPOSETOR
			}
		});
	};

	this.send = function send(parameters){
		try {
			parameters.data.SECURITYKEY = securityKey;
			webSocket.send(JSON.stringify(parameters));
		} catch(erro){
			setTimeout(function(){self.send(parameters);console.log(erro);}, 1000);
		}
	};

	this.logoffBrowser = function() {
		var environmentParams = environment.getLocalEnvironment();
		this.send({
			event: socketEvents.LOGOFF_BROWSER,
			data: {
				'CDFILIAL': environmentParams.CDFILIAL,
				'CDSETOR': environmentParams.CDSETOR
			}
		});
		webSocket.close();
		/*clearInterval(intervalPing);
		intervalPing = null;*/
		inLogoff = true;
	};

	this.playRequest = function(CDFILIAL, NRPEDIDOFOS, NRITPEDIDOFOS, CDSETOR){
		this.send({
			event: socketEvents.PLAY_REQUEST,
			data: {
				'CDFILIAL': CDFILIAL,
				'NRPEDIDOFOS': NRPEDIDOFOS,
				'NRITPEDIDOFOS': NRITPEDIDOFOS,
				'CDSETOR': CDSETOR
			}
		});
	};

	this.concludeRequest = function(CDFILIAL, NRPEDIDOFOS, NRITPEDIDOFOS, orderKey, clientKey, CDSETOR, isZoom){
		this.send({
			event: socketEvents.CONCLUDE_REQUEST,
			data: {
				'CDFILIAL': CDFILIAL,
				'NRPEDIDOFOS': NRPEDIDOFOS,
				'NRITPEDIDOFOS': NRITPEDIDOFOS,
				'orderKey' :orderKey,
				'clientKey' :clientKey,
				'CDSETOR': CDSETOR,
				'isZoom': isZoom
			}
		});
	};

	this.concludeExpedition = function(CDFILIAL, itemsArray, orderKey, clientKey, CDSETOR){
		this.send({
			event: socketEvents.EXPEDITION_REQUEST,
			data: {
				'CDFILIAL': CDFILIAL,
				'CDSETOR': CDSETOR,
				'itemsArray': JSON.stringify(itemsArray),
				'orderKey': orderKey,
				'clientKey': clientKey
			}
		});
	};

	this.clearScreen = function(CDFILIAL, CDSETOR, hoursNumber = 0){
		this.send({
			event: socketEvents.CLEAR_SCREEN,
			data: {
				'CDFILIAL': CDFILIAL,
				'CDSETOR': CDSETOR,
				'clearScreen': true,
				'hoursNumber': hoursNumber
			}
		});
	};

	this.productionLine = function(CDFILIAL, CDSETOR, hoursNumber = 0){
		this.send({
			event: socketEvents.PRODUCTIONLINE,
			data: {
				'CDFILIAL': CDFILIAL,
				'CDSETOR': CDSETOR,
				'clearScreen': true,
				'hoursNumber': hoursNumber
			}
		});
	};

	this.freezeUpdates = function(){
		this.send({
			event: socketEvents.FREEZE,
			data: {}
		});
	};

	this.unfreezeUpdates = function(){
		this.send({
			event: socketEvents.UNFREEZE,
			data: {}
		});
	};

	this.getData = function(CDFILIAL, CDSETOR, IDTIPOSETOR){
		this.send({
			event: socketEvents.GET_DATA,
			data: {
				'CDFILIAL': CDFILIAL,
				'CDSETOR': CDSETOR,
				'IDTIPOSETOR': IDTIPOSETOR,
				'clearScreen': true
			}
		});
	};

    this.ping = function() {
        var environmentParams = environment.getLocalEnvironment();
        if (mutex) {
            try {
                mutex = false;
                if (isConnected) {
                    webSocket.send({
                        event: socketEvents.PING_EVENT,
                        data: {
                            'CDFILIAL': environmentParams.CDFILIAL,
                            'CDSETOR': environmentParams.CDSETOR
                        }
                    });
                    if (self.isServerNotResponding()) {
                        webSocket.close();
                    }
                }
                mutex = true;
            } catch(err) {
                console.log('SocketService - ping error:');
                console.log(err);
                mutex = true;
            }
        } else {
            console.log('SocketService - mutex false');
        }
    };

    this.onPong = function() {
        self.lastPong = new Date();
    };

    this.isServerNotResponding = function() {
        var now = new Date();
        return self.lastPong && (now - self.lastPong) > (NUMBER_OF_PINGS_TO_RESYNCRONIZE * pingTime);
    };
}

Configuration(function(ContextRegister) {
	ContextRegister.register('SocketServiceKDS', SocketService);
});

// FILE: js/services/ConcludeQueue.js
function ConcludeQueue(environment, SocketServiceKDS, $timeout, $interval, SummaryItemsService) {

	var self = this;

    this.queue = [];

    this.insert = function(order, isZoom) {
        var rollbackLimitTime = environment.getLocalEnvironment().NRROLLBTIME;
        order.keepOrderColor = true;
        if (environment.getLocalEnvironment().IDTIPOSETOR === 'E'){
            order.expeditionDelayBeep = false;
        }
        var toConcludeOrder = {
            order: order,
            isZoom: isZoom,
            timer : $timeout(timeoutConclusionQueue, rollbackLimitTime, true)
        };
        self.queue.push(toConcludeOrder);
    };

    this.pop = function() {
        return self.queue.pop();
    };

    this.isQueueEmpty = function() {
        return self.queue.length === 0;
    };

    this.get = function() {
        return self.queue;
    };

    this.checkItemExistence = function(orderKey) {
        return self.queue.some(function(item) {
            return orderKey && item.order.orderKey == orderKey;
        });
    };

    function timeoutConclusionQueue() {
        var localConclusionOrder = self.queue.shift();
        handleConcludeOrder(localConclusionOrder.order, localConclusionOrder.isZoom);
    }

    function handleConcludeOrder(order, isZoom) {
        var widget = environment.getKDSWidget();
        var localEnvironment = environment.getLocalEnvironment();
        var sectorType  = localEnvironment.IDTIPOSETOR;
        var CDFILIAL    = localEnvironment.CDFILIAL;
        var NRPEDIDOFOS = order.NRPEDIDOFOS;
        var orderKey    = order.orderKey;
        var CDSETOR     = localEnvironment.CDSETOR;
        var clientKey   = CDFILIAL + '-' + CDSETOR;

        if ((sectorType === 'E' || sectorType === 'M') && !isZoom) {
            order.CDFILIAL = CDFILIAL;
            var itemsArray  = order.items ? order.items : [order];
            SocketServiceKDS.concludeExpedition(CDFILIAL, itemsArray, orderKey, clientKey, CDSETOR);
            if(order.items){
                if(order.orderCode){
                    //se for uma ordem
                    clearOrderTimeControl(order);
                    widget.dataSource.data.splice(widget.dataSource.data.indexOf(order), 1);
                } else {
                    //se for um grupo
                    widget.dataSource.data.forEach(function(dataOrder){
                        dataOrder.items.forEach(function(dataItem, index, items){
                            if(dataItem.orderKey == order.orderKey){
                                items.splice(index);
                            }
                        });
                    });
                }
                widget.dataSource.hasPriorityOrders = hasPriorityOrders(widget);
            }
        } else if((sectorType === 'E' || sectorType === 'M') && isZoom){
            let NRITPEDIDOFOS = order.NRITPEDIDOFOS;
            SocketServiceKDS.concludeRequest(CDFILIAL, NRPEDIDOFOS, NRITPEDIDOFOS, orderKey, clientKey, CDSETOR, isZoom);
            widget.dataSource.hasPriorityOrders = hasPriorityOrders(widget);
        } else if(sectorType === 'P'){
            //var NRITPEDIDOFOS = order.items[0].NRITPEDIDOFOS;
            var orderItens = order.items;
            var NRITPEDIDOFOS;
            //Para dois itens ou mais a serem concluidos no mesmo card
            if(orderItens.length >= 2){
                NRITPEDIDOFOS = [];
                order.items.forEach(function(item){
                    NRITPEDIDOFOS.push(item.NRITPEDIDOFOS);
                });
            }else{
                NRITPEDIDOFOS = order.items[0].NRITPEDIDOFOS;
            }
            SocketServiceKDS.concludeRequest(CDFILIAL, NRPEDIDOFOS, NRITPEDIDOFOS, orderKey, clientKey, CDSETOR);
            widget.dataSource.data.splice(widget.dataSource.data.indexOf(order), 1);
            SummaryItemsService.updateSummaryItems();
        }
    }

    function clearOrderTimeControl(order) {
        if(order.timerInterval) {
            $interval.cancel(order.timerInterval);
            delete order.timerInterval;
        }
        if(order.colorTimeout) {
			$timeout.cancel(order.colorTimeout);
			delete order.colorTimeout;
		}
    }

    function hasPriorityOrders (widget) {
        if (!widget.IDHABPRIORIDADE || widget.IDHABPRIORIDADE !== 'S') {
            return false;
        }
        if (widget.IDTIPOSETOR === 'P') {
            return false;
        }
        if (widget.dataSource.data.length === 0) {
            return false;
        }
        return !!widget.dataSource.data.find((p) => {
            return p.IDPRODPRIORIDADE === true && ((p.IDCONDPEDPAG === 'S' && p.IDSITUVENDA === 'O') || p.IDCONDPEDPAG === 'N');
        });
    }
}

Configuration(function(ContextRegister) {
	ContextRegister.register('ConcludeQueue', ConcludeQueue);
});

// FILE: js/services/OrderActionService.js
function OrderActionService($interval, $timeout, environment, NavigatorService, SocketServiceKDS, ConcludeQueue, SoundService, ScreenService) {

    this.startOrderProduction = function (order) {
        var widget = environment.getKDSWidget();
        if (SocketServiceKDS.isOpen() && SocketServiceKDS.isConnected()) {
            var index = NavigatorService.filterOrders(
                widget.dataSource.data
            ).indexOf(order);
            /* var itemNotCancelled = order.items.find((item) => {
                return item.IDSITITPEDFOS != 'C';
            });
            NRITPEDIDOFOS = itemNotCancelled && itemNotCancelled.NRITPEDIDOFOS;
            SocketServiceKDS.playRequest(order.CDFILIAL, order.NRPEDIDOFOS, NRITPEDIDOFOS, order.CDSETOR);
                widget.dataSource.data.forEach((summaryOrder, idx) => {
                if (summaryOrder.IDSOMRESUMO == 'S' && summaryOrder.NRPEDIDOFOS == order.NRPEDIDOFOS) {
                    SocketServiceKDS.playRequest(summaryOrder.CDFILIAL, summaryOrder.NRPEDIDOFOS, summaryOrder.NRITPEDIDOFOS, summaryOrder.CDSETOR);
                }
            }); */
            let localEnvironment = environment.getLocalEnvironment();
            if (localEnvironment.IDHABAGRUPPROD == "S") {
                // quando estiver agrupado INICIA todos os itens desse pedido
                SocketServiceKDS.playRequest(order.CDFILIAL, order.NRPEDIDOFOS, null, order.CDSETOR);
            } else {
                let itemNotCancelled = order.items.find((item) => {
                    return item.IDSITITPEDFOS != 'C';
                });
                let NRITPEDIDOFOS = itemNotCancelled && itemNotCancelled.NRITPEDIDOFOS;
                SocketServiceKDS.playRequest(order.CDFILIAL, order.NRPEDIDOFOS, NRITPEDIDOFOS, order.CDSETOR);
            }
            // SocketServiceKDS.playRequest(order.CDFILIAL, order.NRPEDIDOFOS, NRITPEDIDOFOS, order.CDSETOR)
            NavigatorService.setOrderAsSelected(index, 'keep');
        } else {
            ScreenService.notificationMessage("Não foi possível iniciar o item, falha de conexão com o servidor.", 'error', 4000);
        }
    };

    this.concludeOrderProduction = function (order) {
        const widget = environment.getKDSWidget();
        order = ungroupItems(order);
        if (widget && !widget.completedOrders) {
            // array utilizado para guardar chaves dos pedidos ja removidos da tela
            widget.completedOrders = {};
        }

        let lastSelectedOrder = NavigatorService.filterOrders(widget.dataSource.data).findIndex(of => of.isSelected);

        if (SocketServiceKDS.isOpen() && SocketServiceKDS.isConnected()) {

            let indexOfSelectedOrder = NavigatorService.filterOrders(widget.dataSource.data).indexOf(order);
            let selecteOrderOnDataSource = NavigatorService.filterOrders(widget.dataSource.data)[indexOfSelectedOrder];
            
            if(selecteOrderOnDataSource && order) {
                ConcludeQueue.insert(order, false);
                selecteOrderOnDataSource.isInConclusionQueue = true;

                let key = selecteOrderOnDataSource.CDSETOR + selecteOrderOnDataSource.NRPEDIDOFOS + selecteOrderOnDataSource.NRITPEDIDOFOS;
                widget.completedOrders[key] = moment().format('HH:mm'); // Armazena a chave do pedido para impedir que ele volte pra tela
                widget.dataSource.data.forEach((summaryOrder, idx) => {
                    if (summaryOrder && (summaryOrder.IDSOMRESUMO == 'S' && summaryOrder.NRPEDIDOFOS == NavigatorService.filterOrders(widget.dataSource.data)[indexOfSelectedOrder].NRPEDIDOFOS)) {
                        ConcludeQueue.insert(summaryOrder, false);
                        summaryOrder.isInConclusionQueue = true;
                    }
                });

                //widget.redrawKDSContainer
                if (!lastSelectedOrder) {
                    let toGo = indexOfSelectedOrder || lastSelectedOrder;
                    toGo = widget.dataSource.data.length == toGo ? toGo -1 : toGo;
                    NavigatorService.setOrderAsSelected(toGo, 'move');
                } else {
                    NavigatorService.setOrderAsSelected(lastSelectedOrder, 'move');
                }
            }
        } else {
            ScreenService.notificationMessage("Sem conexão no momento, tente novamente mais tarde!", 'error', 3000);
        }
    };

    this.expediteOrder = function (order) {
        var widget = environment.getKDSWidget();
        if (SocketServiceKDS.isOpen() && SocketServiceKDS.isConnected()) {
            // verifica se os itens da expedição foram agrupados
           order = ungroupItems(order);

            ConcludeQueue.insert(order, widget.isZoom);
            var indexOfSelectedOrder = NavigatorService.filterOrders(
                widget.dataSource.data
            ).indexOf(order);
            let filteredOrders = NavigatorService.filterOrders(widget.dataSource.data);
            if (filteredOrders[indexOfSelectedOrder]){
                NavigatorService.filterOrders(
                    widget.dataSource.data
                )[indexOfSelectedOrder].isInConclusionQueue = true;
            }
            //widget.redrawKDSContainer
            NavigatorService.setOrderAsSelected(indexOfSelectedOrder, 'move');

        } else {
            ScreenService.notificationMessage("Sem conexão no momento, tente novamente mais tarde!", 'error', 3000);
        }
    };

    this.continueOrder = function (order) {
        order.isPausedByInterval = false;
        order.updateAvailableAction();
        order.startOrderTimer();
    };

    // Controla se as ações disparadas no Front pelo usuário podem ser executadas naquele momento
    // ou se devem aguardar o tempo de delay definido no setTimeout dentro de orderOnClick
    this.isInDelayTime = false;

    this.orderOnClick = function (order, isAutoClick) {
        var widget = environment.getKDSWidget(); 
        
        // Verifica se o click foi aplicado sobre um pedido cinza. Em caso afirmativo, o click deve ser ignorado.
        if(order && order.availableAction !== null){
            
            // Esse caso será executado apenas para os orderOnClick disparados automaticamente devido ao 
            // parametro PLAYANDCONCLUDE (inicia direto na producao)
            if (order && isAutoClick){
                let currentOrder = NavigatorService.getSelectedOrder();
                let lastSelectedOrderIndex = NavigatorService.getOrderIndex(currentOrder);

                if(!(order.processing === true && order.availableAction && order.availableAction.actionLabel.toLowerCase() == 'iniciar')) {
                    order.processing = true;
                    widget.updateKDSTemplate();
                        order.clearBeeps();
                    if (order.availableAction) {

                        if (order.IDSITITPEDFOS == 'E') {
                            clearOrderTimer(order);
                            order.dot = false;
                        }
                        order.availableAction.actionCallback(order);
                        NavigatorService.unSelectAllOrders();
                        NavigatorService.setPreviousOrderAsSelected(++lastSelectedOrderIndex);
                        widget.updateKDSTemplate();
                    }
                }
            // Esse caso será executado para todos os demais cliques efetuados pelo usuário
            } else if (order && !this.isInDelayTime) {

                if(!(order.processing === true && order.availableAction && order.availableAction.actionLabel.toLowerCase() == 'iniciar')) {
                order.processing = true;
                widget.updateKDSTemplate();
                    order.clearBeeps();
                    if (order.availableAction) {

                        this.isInDelayTime = true;
                        setTimeout(()=>{
                            if (order.IDSITITPEDFOS == 'E') {
                                clearOrderTimer(order);	
                                order.dot = false;
                            }
                            order.availableAction.actionCallback(order);
                            this.isInDelayTime = false;
                        }, 100);
                    }
                }
            }
            widget.updateKDSTemplate();
        }
    };

    function clearOrderTimer(order) {
        if (order.timerInterval) {
            $interval.cancel(order.timerInterval);
            delete order.timerInterval;
        }
    }

    function ungroupItems (order) {
        // desfaz o agrupamento para enviar ao backend
        let ungroupedItem = [];
        let itemsToRemove = [];
        if (!order.items) {
            return order;
        }

        order.items.forEach((e)=> {
            if (e.groupedItens) {
                e.groupedItens.forEach((element) => {
                    ungroupedItem.push(element);
                });
                itemsToRemove.push(e);
            }
        });

        itemsToRemove.forEach((e) => {
            order.items.splice(order.items.indexOf(e), 1);
        });

        order.items = order.items.concat(ungroupedItem);
        return order;
    }

}

Configuration(function (ContextRegister) {
    ContextRegister.register('OrderActionService', OrderActionService);
});


// FILE: js/services/ClientNotificationService.js
function ClientNotificationService(ScreenService, RestEngine) {

    var self = this;

    this.notifyClient = function (currentOrder) {
        if (currentOrder && currentOrder.items.length > 0) {         

            dispatchClientNotification(currentOrder.CDFILIAL, currentOrder.items[0].NRVENDAREST, currentOrder.items[0].NRCOMANDA, currentOrder.CDSENHAPED,
                currentOrder.CDSETOR, currentOrder.NRPEDIDOFOS, currentOrder.items[0].NRSEQVENDA, currentOrder.items[0].CDCAIXA, currentOrder.dot, currentOrder.items[0].NRPRODCOMVEN).then(function (response) {
                    if (!response.error) {
                        if(response.pager != undefined){
                            sweetNotify(response.pager);
                        }
                        if(response.voz != undefined){
                            sweetNotify(response.voz);
                        }
                        if(response.painel != undefined){
                            sweetNotify(response.painel);
                        }
                        if(response.pagerDigital != undefined){
                            sweetNotify(response.pagerDigital);
                        }
                        if(response.error === false){
                            ScreenService.notificationMessage(response.message, 'success', 3000);
                        }
                    } else {
                        currentOrder.DTHRPRIEXPED = null;
                        ScreenService.notificationMessage('Falha ao notificar consumidor.', 'error', 5000);
                    }
                });
        }
    };

    function sweetNotify(notifyType){
        if(notifyType.error && notifyType.message){
            ScreenService.notificationMessage(notifyType.message, 'error', 5000);
        } else if (!notifyType.error && notifyType.message) {
            ScreenService.notificationMessage(notifyType.message, 'success', 3000);
        }
    }

    function dispatchClientNotification(CDFILIAL, NRVENDAREST, NRCOMANDA, CDSENHAPED, CDSETOR, NRPEDIDOFOS, NRSEQVENDA, CDCAIXA, DOT, NRPRODCOMVEN) {
        var params = {
            requestType: "Row",
            row: {
                'CDFILIAL': CDFILIAL || null,
                'NRVENDAREST': NRVENDAREST || null,
                'NRCOMANDA': NRCOMANDA || null,
                'CDSENHAPED': CDSENHAPED || null,
                'CDSETOR': CDSETOR || null,
                'NRPEDIDOFOS': NRPEDIDOFOS || null,
                'NRSEQVENDA': NRSEQVENDA || null,
                'CDCAIXA': CDCAIXA || null,
                'DOT': DOT || null,
                'NRPRODCOMVEN': NRPRODCOMVEN || null
            }
        };
        var notifyClientRoute = '/notification/client';
        return RestEngine.post(notifyClientRoute, params).then(function (response) {
            return response.dataset.notifyClient;
        });
    }

}

Configuration(function (ContextRegister) {
    ContextRegister.register('ClientNotificationService', ClientNotificationService);
});

// FILE: js/services/ColorService.js
function ColorService(RestEngine, localStorageEngine) {

	var self = this;
	var colors = null;

	this.setControlColors = function(controlColors) {
		colors = controlColors;
    };
    
	this.getControlColors = function() {
		return colors;
	};

    this.unsetControlColors = function() {
    	colors = null;
    };

    this.getColors = function(){
        var params = {
            requestType: "Empty"
        };
        var getColorsRoute = '/colors/get';
        return RestEngine.post(getColorsRoute, params).then(function(response) {
            return response.dataset.getColorsReponse;
        });
    };

    self.getColors().then(fillColorsOnBegin);

    function fillColorsOnBegin(response) {
    	if(response.success) {
    		self.setControlColors(response.colors);
    	}
    }
}

Configuration(function(ContextRegister) {
	ContextRegister.register('ColorService', ColorService);
});

// FILE: js/services/KeyboardEventsHandler.js
function KeyboardEventsHandler($timeout, environment, NavigatorService, OrderActionService, ClientNotificationService,  ConcludeQueue, ScreenService, RestEngine, ApplicationContext, templateManager, SocketServiceKDS, SummaryItemsService, Utilities) {

	var self = this;

	var justExecuted = false;

	var lastKeyOne = false;

	this.executeDownEvent = function(event, key) {
		lastKeyOne = false;
		NavigatorService.setNextOrderAsSelected();
	};

	this.tempList = [];

	this.executeUpEvent = function(event, key) {
		lastKeyOne = false;
		let confirmar = 0, canc = 1, libe = 1;
		NavigatorService.setPreviousOrderAsSelected();
	};

	this.executeLeftEvent = function(event, key) {
		lastKeyOne = false;
		var jumps = 1, NRMULTIPULO = environment.getLocalEnvironment().NRMULTIPULO;
		if(NRMULTIPULO && NRMULTIPULO > 1){
			jumps = NRMULTIPULO;
		}

		for (jumps; jumps > 0; jumps--) {
			self.executeUpEvent();
		}
	};

	this.executeRightEvent = function(event, key) {
		lastKeyOne = false;
		var jumps = 1, NRMULTIPULO = environment.getLocalEnvironment().NRMULTIPULO;
		if(NRMULTIPULO && NRMULTIPULO > 1){
			jumps = NRMULTIPULO;
		}
		
		for (jumps; jumps > 0; jumps--) {
			self.executeDownEvent();
		}
	};

	this.executeZoomEvent = function(event, key) {
		lastKeyOne = false;
		var widget = environment.getKDSWidget();
		var projectConfig = environment.getProjectConfig();
		if(widget.kdsType == "expedition" && projectConfig.partialRelease){
			var currentOrder = NavigatorService.getSelectedOrder();
			var tempOrder = widget.getItemsArray(currentOrder);
			if (currentOrder && hasItemOnOrder(tempOrder, 1)) {
				widget.zoomItem = tempOrder;
				NavigatorService.setFirstItemAsSelected(widget.zoomItem);
			}else{
				ScreenService.notificationMessage('Não há itens para serem expedidos separadamente', 'alert', 5200);
			}
			widget.updateKDSTemplate();
		}
	};

	this.executeBackspaceEvent = function() {
		lastKeyOne = false;
		var widget = environment.getKDSWidget();
		widget.completedOrders = {};
		if (environment.getLocalEnvironment().IDHABROLLBKDS == 'S') {
			if (!ConcludeQueue.isQueueEmpty()) {
				var conclusionOrder = ConcludeQueue.pop();
				$timeout.cancel(conclusionOrder.timer);
				conclusionOrder.order.isInConclusionQueue = false;
				conclusionOrder.order.isNewOrder = true;
				if(widget.zoomItem){
					NavigatorService.handleScrollbarOnZoomUp();
				}
				if(!NavigatorService.getSelectedOrder()){
					NavigatorService.setFirstOrderAsSelected();
				}
				widget.updateKDSTemplate();
				conclusionOrder.order.keepOrderColor = false;
			} else {
				ScreenService.alertNotification('Não há pedidos na fila de conclusão!');
			}
		}
	};

	this.confirmExpedition = function(currentOrder){
            let lastSelectedOrderIndex = NavigatorService.getOrderIndex(currentOrder);
			OrderActionService.concludeOrderProduction(currentOrder);
			OrderActionService.orderOnClick(currentOrder);
			ClientNotificationService.notifyClient(currentOrder);
			NavigatorService.unSelectAllOrders();
			ApplicationContext.ShortcutKeyService.bindShortcutsOrderPanel();
			setTimeout(() => {
                NavigatorService.setPreviousOrderAsSelected(lastSelectedOrderIndex);
            }, 300);
	};

	this.executeEnterEvent = function (event, key) {
		lastKeyOne = false;
		if (justExecuted) {
			setTimeout(() => {
				justExecuted = false;
			}, 500);
		}
		else if (!justExecuted) {
			var widget = environment.getKDSWidget();
			var currentOrder = NavigatorService.getSelectedOrder();
			if (currentOrder != undefined) {
				let loader = document.querySelector('aside.zh-background-loading');
				if(loader && loader.style && loader.style.display != "block") {
					if (currentOrder.isCallClientAvailable() && currentOrder.status == 'expedite') {
						currentOrder.dot = false;
						ClientNotificationService.notifyClient(currentOrder);
					}else if(currentOrder.status == undefined && widget.IDHABLIBDIRETOEXP == 'S'){
						ApplicationContext.ShortcutKeyService.bindPopUpKeys();
						ScreenService.confirmMessage(
							'Este pedido ainda está em produção. Tem certeza que deseja expedi-lo?',
							'question', 
							function(){
								ApplicationContext.ShortcutKeyService.unbindPopUpKeys();
								delete currentOrder.dot;
								self.confirmExpedition(currentOrder);
								justExecuted = true;
							},
							function (){ 
								ApplicationContext.ShortcutKeyService.unbindPopUpKeys();
								ApplicationContext.ShortcutKeyService.bindShortcutsOrderPanel();
								justExecuted = true;
							}
						);
					}
				} else{
					ApplicationContext.ShortcutKeyService.bindPopUpKeys();
				}
			}
			OrderActionService.orderOnClick(currentOrder);
		}
        
    };

	this.logoff = function() {
		if(ConcludeQueue.isQueueEmpty()) {
			logoffConfirmation();
		} else {
			openLastOpenedWindow();
			ScreenService.showMessage('Ainda existem pedidos aguardando para serem liberados, favor aguardar');
		}
	};

	function logoffConfirmation() {
		clearKDSWidgetDataSource();
		ApplicationContext.ShortcutKeyService.unbindMappedKeys();
		SocketServiceKDS.logoffBrowser();
		SummaryItemsService.setFixedItems(null);
		environment.unsetLocalEnvironment();
		Utilities.clearAllTimeOutsAndIntervals();
		ScreenService.openWindow('initialParameters');
	}

	function clearKDSWidgetDataSource() {
		var widget =  environment.getKDSWidget();
		widget.dataSource.data = [];
	}

	this.executeOneEvent = function() {
		lastKeyOne = true;
		ApplicationContext.ShortcutKeyService.bindOneMappedKeys();
	};

	this.executeOneOneEvent = function() {
		if (lastKeyOne)
			ApplicationContext.ShortcutKeyService.bindOneOneMappedKeys();
		else 
			ApplicationContext.ShortcutKeyService.bindMappedKeys();
	};

	this.executeOneOneOneEvent = function() {
		if (lastKeyOne) {
			ApplicationContext.ShortcutKeyService.bindPopUpKeys();
			ScreenService.confirmMessage(
				'Deseja sair do setor?',
				'question', 
				function(){
					ApplicationContext.ShortcutKeyService.unbindPopUpKeys();
					logoffConfirmation();
				},
				function (){ 
					setTimeout(() => {
						ApplicationContext.ShortcutKeyService.bindShortcutsOrderPanel();
						ApplicationContext.ShortcutKeyService.unbindPopUpKeys();
					}, 300);
				}
			);
		} else {
			ApplicationContext.ShortcutKeyService.bindMappedKeys();
		}
	};

	this.executeDotEvent = function () {
		lastKeyOne = false;
		var currentOrder = NavigatorService.getSelectedOrder();
		var widget = environment.getKDSWidget();
		if (currentOrder != undefined) {
            
            // IDHABLIBDIRETOEXP = Permite liberacao direta do pedido. S = Sim - N = Nao
			if (currentOrder.isCallClientAvailable() && currentOrder.status == 'expedite' || widget.IDHABLIBDIRETOEXP == 'S') {
				currentOrder.dot = true;
				ClientNotificationService.notifyClient(currentOrder);
			}
		}
	};

	this.executeReloadEvent = function() {
		window.customKDSReload();
	};

	//--------------------Zoom Events
	this.executeZoomBackspaceEvent = function() {
		self.executeBackspaceEvent();
	};

	//This will call expedite function for choosen items on zoom
	function applyExpeditionToTempList(listToConclude){
		listToConclude.forEach( item => {
			ConcludeQueue.insert(item, true);
		});
	}

	this.executeZoomEnterEvent = function (){
		var widget = environment.getKDSWidget();
		var orderItem = NavigatorService.getSelectedOrderItem(widget.zoomItem);
		//anda com o selected do zoom
		if(orderItem.IDSITITPEDFOS == 'F'){
			//Muda o foco
			NavigatorService.setSelectedByOrderKeyZoom(widget.zoomItem, widget.zoomSelectedOrderKey, true);
			//Se tiver subitens ? subitens  [do contrário] item
			orderItem = orderItem.items ? orderItem.items : [orderItem];
			//Coloca na fila de conclusão
			orderItem.isInConclusionQueue = true;
			//Trata cada item de ordemItem, seja um ou vários
			orderItem.forEach(function(item){
				
				NavigatorService.setInConclusionOnDataSource(item);
				if(item.IDSITITPEDFOS == 'F'){
					if(!hasItemOnOrder(widget.zoomItem, 1)){
						if(!hasItemOnOrder(widget.zoomItem, 0)){
							var toDeleteOrder = NavigatorService.getSelectedOrder();
							var index = widget.dataSource.data.indexOf(toDeleteOrder);
							widget.dataSource.data[index].isInConclusionQueue = true;
							NavigatorService.setOrderAsSelected(index, 'move', 'down');
						}
						widget.zoomItem = null;
						widget.bindPanelShortcutKeys();
						widget.updateKDSTemplate();
						//não sei se pode fazer isso mas ta ai
					}else{
						NavigatorService.handleScrollbarOnZoomDown();
					}
					if(item) {
						if(widget) {
							if(widget.zoomItem){
								if (!widget.zoomItem.listToConclude){
									widget.zoomItem.listToConclude = [];
								}
								widget.zoomItem.listToConclude.push(item);
							} else{
								ConcludeQueue.insert(item, true);
							}							
						} else {
							ConcludeQueue.insert(item, true);
						}
					}
				}
			});
		}
	};

	function hasItemOnOrder(order, qtty) {
		var itemQtty = 0, compItemQtty = 0, groupQtty = 0;
		return order && order.some(function(position){
			return position && position.some(function(item){
				if (!item.isInConclusionQueue) {
					if(item.items){
						groupQtty++;
					}else{
						itemQtty++;
					}
				}
				return (itemQtty > qtty && !item.items) || (item.items && item.items.some(function(compItem){
					if (!compItem.isInConclusionQueue) {
						compItemQtty++;
					}
					return compItemQtty > qtty || ((groupQtty + itemQtty) > qtty) ;
				}));
			});
		});
	}

	this.executeZoomZoomEvent = function(event, key) {
		var widget = environment.getKDSWidget();
		if(widget.zoomItem) {
			if(widget.zoomItem.listToConclude){
				applyExpeditionToTempList(widget.zoomItem.listToConclude);
			}
			widget.zoomItem = null;
			widget.bindPanelShortcutKeys();			
			widget.updateKDSTemplate();
		}
	};

	this.executeZoomUpEvent =  function(){
		var widget = environment.getKDSWidget();
		NavigatorService.setSelectedByOrderKeyZoom(widget.zoomItem, widget.zoomSelectedOrderKey, false);
		widget.updateKDSTemplate();
		NavigatorService.handleScrollbarOnZoomUp();
	};

	this.executeZoomDownEvent = function() {
		var widget = environment.getKDSWidget();
		NavigatorService.setSelectedByOrderKeyZoom(widget.zoomItem, widget.zoomSelectedOrderKey, true);
		widget.updateKDSTemplate();
		NavigatorService.handleScrollbarOnZoomDown();
	};

	/* jshint ignore:start */
	this.executeSlashEvent = function () {
		lastKeyOne = false;
		var widget = environment.getKDSWidget();
		var localEnvironment = environment.getLocalEnvironment();
		if(localEnvironment.IDLINHAPRINCIPAL != 'S') {
			ApplicationContext.ShortcutKeyService.unbindMappedKeys();
			ApplicationContext.ShortcutKeyService.bindSlashMappedKeys();
			var popup = localEnvironment.IDTIPOSETOR == 'P' ? templateManager.containers.production.getWidget("lineActivationPassword") : templateManager.containers.expedition.getWidget("lineActivationPassword") ; 
			ScreenService.openPopup(popup);
			setTimeout(() => {
				document.getElementById('linePassword').value = "";
				document.getElementById('linePassword').focus();
			}, 300);
		} else {
			ScreenService.notificationMessage('Não é possível desativar a linha principal.', 'error', 3000);
		}

		NavigatorService.unSelectAllOrders();
		setTimeout(() => {
			NavigatorService.setFirstOrderAsSelected();
		}, 300);
	};

	this.executeSlashEnterEvent = async function () {
		var widget = templateManager.container.getActiveWidget();
		localEnvironment = environment.getLocalEnvironment();
		let CDSETOR = localEnvironment.CDSETOR;
		let checkStatusRoute = '/StatusLineProduction/get';
        var password = widget.getField('linePassword').element.val();
		
        if (password == 5050){
			ScreenService.closePopup(true); 
            var params = {
                requestType: "Row",
                row: {
                        "CDSETOR": CDSETOR
                }
            };
    
			var lineStatusAction = await RestEngine.post(checkStatusRoute, params).then(function (response) {
				if (response.dataset.checkLineProductionStatus.lineStatus[0]) {
					return response.dataset.checkLineProductionStatus.lineStatus[0].IDATIVO == 'S' ? 'desativar' : 'ativar';
				} else {
					return null; 
				}
            }).catch(err => {
                return err;
            });
    
            if(lineStatusAction){
				ApplicationContext.ShortcutKeyService.bindSlashEnterMappedKeys();
                ApplicationContext.ShortcutKeyService.bindPopUpKeys();
                ScreenService.confirmMessage('Deseja ' + lineStatusAction + ' esta linha de produção?',
                    'question',
                    function () {
                        let lineProductionRoute = '/StatusLineProduction/update';
        
                        var params = {
                            requestType: "Row",
                            row: {
								"CDFILIAL": localEnvironment.CDFILIAL,
                                "CDSETOR": CDSETOR
                            }
                        };
        
                        RestEngine.post(lineProductionRoute, params).then(function (response) {
                            console.log(response.dataset);
                        }).catch(err => {
                            return err;
                        })
                        .finally(() => {
                            window.customKDSReload();
                        });
                    },
                    function () { 
						setTimeout(() => {
							ApplicationContext.ShortcutKeyService.bindShortcutsOrderPanel();
							ApplicationContext.ShortcutKeyService.unbindPopUpKeys();
						}, 300);
					}
                );
            } else {
                ScreenService.notificationMessage('Não está parametrizada uma linha de produção neste setor', 'alert', 3000);
            }
        } else {
            ScreenService.notificationMessage('Senha incorreta', 'alert', 3000);
        }
	};

	this.executeSlashEnterEnterEvent = function() {
		setTimeout(() => {
			ApplicationContext.ShortcutKeyService.bindShortcutsOrderPanel();
		}, 300);
	};

	/* jshint ignore:end */
	this.executeSlashSlashEvent = function(){
		ScreenService.closePopup(true); 
        ApplicationContext.ShortcutKeyService.bindShortcutsOrderPanel();
	};

	this.executePicklistEvent = function() {
		lastKeyOne = false;
		var widget = environment.getKDSWidget();
		var currentOrder = NavigatorService.getSelectedOrder();
		if (currentOrder != undefined && currentOrder.items.length > 0) {
            let picklistRoute = '/printPicklist';
			var params = {
				requestType: "Row",
				row: currentOrder.items
			};

			RestEngine.post(picklistRoute, params).then(function(response) {
				if(response.dataset.printPicklist.printed === false) {
					ScreenService.notificationMessage('Impressão por atalho não parametrizada neste setor', 'alert', 3000);
				}
			}) 
			.catch(err => {
				console.log(err);
			});
		}
	};
	
	this.executePriorityPanelEvent = function () {
		lastKeyOne = false;
		try {
			let priorityPanelActive = environment.getLocalEnvironment().IDHABPRIORIDADE == 'S';
			let hasPriorityOrder = environment.getKDSWidget().dataSource.hasPriorityOrders;

			let orderContainerPriority = document.getElementById('priority-orders') || false;
			let orderContainerdefault = document.getElementById('default-orders') || false;
	
			let setDefaultBehaviour = () => {
				NavigatorService.unSelectAllOrders();
				orderContainerdefault && orderContainerdefault.classList.add("selected-container"); // jshint ignore:line
				NavigatorService.setFirstOrderAsSelected();
				orderContainerdefault && orderContainerdefault.scroll({left: 0, behavior: 'smooth'}); // jshint ignore:line
			};
	
			if (priorityPanelActive) {
				if (hasPriorityOrder) {
					if(orderContainerdefault && orderContainerdefault.classList.contains("selected-container") && orderContainerPriority) {
						NavigatorService.unSelectAllOrders();
						orderContainerdefault.classList.remove("selected-container");
						orderContainerPriority.classList.add("selected-container");
						NavigatorService.setFirstOrderAsSelected();
						orderContainerPriority.scroll({top: 0, behavior: 'smooth'});
					} else if (orderContainerPriority && orderContainerPriority.classList.contains("selected-container")) {
						NavigatorService.unSelectAllOrders();
						orderContainerPriority.classList.remove("selected-container");
						orderContainerdefault && orderContainerdefault.classList.add("selected-container"); // jshint ignore:line
						NavigatorService.setFirstOrderAsSelected();
						orderContainerdefault && orderContainerdefault.scroll({left: 0, behavior: 'smooth'}); // jshint ignore:line
					} else {
						setDefaultBehaviour();
					}
				} else {
					ScreenService.notificationMessage('Não há pedidos no painel de pedidos prioritarios', 'alert', 3000);
					setDefaultBehaviour();
				}
			} else {
				ScreenService.notificationMessage('O painel de pedidos prioritarios não esta habilitado para este setor', 'alert', 3000);
				setDefaultBehaviour();
			}    
		} catch (error) {
			console.log('PriorityPanelError');
			console.log(error);
		}
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('KeyboardEventsHandler', KeyboardEventsHandler);
});

// FILE: js/services/ProductionLineService.js
function ProductionLineService(ScreenService, RestEngine, environment, ClientNotificationService, OrderActionService, ApplicationContext, templateManager) {

    var self = this;

    this.closePopup = function(){
        ScreenService.closePopup(); 
        ApplicationContext.ShortcutKeyService.bindShortcutsOrderPanel();
    };


    this.updatePercentage = function(){
        var widget = templateManager.container.getActiveWidget();
        var localEnvironment = environment.getLocalEnvironment();
        var field = widget.getField('linePercentage').value();
        let lineRoute = '/LinePercentage/update';

        if(field){
            var params = {
                requestType: "Row",
                row: {
                    "CDFILIAL": localEnvironment.CDFILIAL,
                    "CDSETOR": localEnvironment.CDSETOR,
                    "PERCENT": field                
                }
            };
    
            RestEngine.post(lineRoute, params).then(function(response) {	
                console.log(response);
            })
            .catch(err =>{
                console.log(err);
            });
    
            self.closePopup(); 
        } else {
			ScreenService.notificationMessage('Digite um percentual válido', 'alert', 5200);
		}

    };

    this.startScanning = function(){
        document.onkeypress = self.keypress.bind(self);
    };

    /* jshint ignore:start */
    this.activateLine = async function(){
        var widget = templateManager.container.getActiveWidget();
        localEnvironment = environment.getLocalEnvironment();
        let CDSETOR = localEnvironment.CDSETOR;
		let checkStatusRoute = '/StatusLineProduction/get';
        var password = widget.getField('linePassword').element.val();
		
        if (password == 5050){
            ScreenService.closePopup(true); 
            var params = {
                requestType: "Row",
                row: {
                        "CDSETOR": CDSETOR
                }
            };
            var lineStatusAction = await RestEngine.post(checkStatusRoute, params).then(function (response) {
				if (response.dataset.checkLineProductionStatus.lineStatus[0]) {
					return response.dataset.checkLineProductionStatus.lineStatus[0].IDATIVO == 'S' ? 'desativar' : 'ativar';
				} else {
					return null; 
				}
            }).catch(err => {
                return err;
            });
    
            if(lineStatusAction){
                ApplicationContext.ShortcutKeyService.bindPopUpKeys();
                ScreenService.confirmMessage('Deseja ' + lineStatusAction + ' esta linha de produção?',
                    'question',
                    function () {
                        let lineProductionRoute = '/StatusLineProduction/update';
        
                        var params = {
                            requestType: "Row",
                            row: {
                                "CDFILIAL": localEnvironment.CDFILIAL,
                                "CDSETOR": localEnvironment.CDSETOR
                            }
                        };
        
                        RestEngine.post(lineProductionRoute, params).then(function (response) {
                            console.log(response.dataset);
                        }).catch(err => {
                            return err;
                        })
                        .finally(() => {
                            window.customKDSReload();
                        });
                    },
                    function () { 
                        setTimeout(() => {
							ApplicationContext.ShortcutKeyService.bindShortcutsOrderPanel();
						}, 300);
                    }
                );
            } else {
                ScreenService.notificationMessage('Não está parametrizada uma linha de produção neste setor', 'alert', 3000);
            }
        } else {
            ScreenService.notificationMessage('Senha incorreta', 'alert', 3000);
        }
    };
    /* jshint ignore:end */

}

Configuration(function (ContextRegister) {
    ContextRegister.register('ProductionLineService', ProductionLineService);
});

// FILE: js/services/ScannerDetection.js
function ScannerDetection(ScreenService, RestEngine, environment, ClientNotificationService, OrderActionService, ApplicationContext) {

    var self = this;

    this.options = {
      onComplete: false, // Callback after detection of a successful scanning
      onError: false, // Callback after detection of a unsuccessful scanning
      onReceive: false, // Callback after receive a char
      timeBeforeScanTest: 1500, // Wait duration (ms) after keypress event to check if scanning is finished
      avgTimeByChar: 30, // Average time (ms) between 2 chars. Used to do difference between keyboard typing and scanning
      minLength: 6, // Minimum length for a scanning
      endChar: [9, 13], // Chars to remove and means end of scanning
      stopPropagation: false, // Stop immediate propagation on keypress event
      preventDefault: false // Prevent default action on keypress event
    };
    this.firstCharTime = 0;
    this.lastCharTime = 0;
    this.stringWriting = '';
    this.callIsScanner = false;
    this.testTimer = false;

    this.keypress = function (e) {
        self.options.onComplete = function(receive) {         
            if(receive){
                var widget = environment.getKDSWidget();
                
                var CDFILIAL;
                if(receive.length > 18 && receive.indexOf("http") == -1){
                    CDFILIAL = receive.substr(1, 4);
                    var NRVENDAREST = receive.substr(5, 10);
                    var NRCOMANDA = receive.substr(15);
                    widget.dataSource.data.forEach(function(order){
                        if(order.CDFILIAL == CDFILIAL &&
                           order.NRVENDAREST == NRVENDAREST &&
                           order.NRCOMANDA == NRCOMANDA)
                        {
                            // IDHABLIBDIRETOEXP = Permite liberacao direta do pedido. S = Sim - N = Nao
                            if (widget.IDHABLIBDIRETOEXP == 'S' || order.status == 'expedite') {
                                order.dot = true;
                                ClientNotificationService.notifyClient(order);
                            }
                        }
                    });
                    ApplicationContext.ShortcutKeyService.bindMappedKeys();
                }else if(receive.indexOf("http") != -1 ){  /* identificar quando é URL */

                    let QrCodeRoute = '/order/get';

                    var params = {
                        requestType: "Row",
                        row: {
                            "DSQRCODENFCE": receive,
                        }
                    };
                    
                    var orderToExped = {};

                    RestEngine.post(QrCodeRoute, params).then(function(response) {	
                        orderToExped = response.dataset.qrCode.qrCode[0];
                        widget.dataSource.data.forEach(function(order){
                            if(order.CDFILIAL == orderToExped.CDFILIAL && (order.NRCOMANDA == orderToExped.NRCOMANDAVND || order.NRSEQVENDA == orderToExped.NRSEQVENDA)){
                                        
                                if (order.isCallClientAvailable() && order.status == 'expedite') {
                                    order.dot = true;
                                    ClientNotificationService.notifyClient(order);
                                }
                                if(widget.IDHABSCANCODE == 'S'){
                                    if(order.availableAction == null){
                                        order.isProducted = true;
                                        order.status = 'expedite';
                                        order.updateAvailableAction();
                                    }
                                    OrderActionService.orderOnClick(order);
                                }
                            }
                        });
                    })
                    .catch(err =>{
                        console.log(err);
                        ApplicationContext.ShortcutKeyService.bindMappedKeys();
                    });
                    ApplicationContext.ShortcutKeyService.bindMappedKeys();
                } else{
                    var CDCAIXA = receive.substr(0, 3);
                    CDFILIAL = receive.substr(3, 4);
                    var NRSEQVENDA = receive.substr(7);
                    widget.dataSource.data.forEach(function(order){
                        if(order.CDFILIAL == CDFILIAL &&
                           order.CDCAIXA == CDCAIXA &&
                           order.NRSEQVENDA == NRSEQVENDA)
                        {
                            if (order.isCallClientAvailable() && order.status == 'expedite') {
                                order.dot = true;
                                ClientNotificationService.notifyClient(order);
                            }
                            if(widget.IDHABSCANCODE == 'S'){
                                OrderActionService.orderOnClick(order);
                            }
                        }
                    });
                }
            }
        };
        self.options.onError = function(){};
        self.options.onReceive = function(){};
        
        if (self.options.stopPropagation) e.stopImmediatePropagation();
        if (self.options.preventDefault) e.preventDefault();

        if (self.firstCharTime && self.options.endChar.indexOf(e.which) !== -1) {
          e.preventDefault();
          e.stopImmediatePropagation();
          self.callIsScanner = true;
        } else {
          self.stringWriting += String.fromCharCode(e.which);
          self.callIsScanner = false;
        }

        if (!self.firstCharTime) {
          self.firstCharTime = e.timeStamp;
        }
        self.lastCharTime = e.timeStamp;

        if (self.testTimer) clearTimeout(self.testTimer);
        if (self.callIsScanner) {
          self.scannerDetectionTest();
          self.testTimer = false;
        } else {
          self.testTimer = setTimeout(() => {
            self.scannerDetectionTest();
          }, self.options.timeBeforeScanTest);
        }

        if (self.options.onReceive) self.options.onReceive.call(self, e);
        self.trigger('scannerDetectionReceive', { evt: e });
    };

    this.trigger = function (options) {};

    this.scannerDetectionTest = function (s) {
        // If string is given, test it
        if (s) {
          self.firstCharTime = self.lastCharTime = 0;
          self.stringWriting = s;
        }
        // If all condition are good (length, time...), call the callback and re-initialize the plugin for next scanning
        // Else, just re-initialize
        if (
          self.stringWriting.length >= self.options.minLength &&
          self.lastCharTime - self.firstCharTime <
            self.stringWriting.length * self.options.avgTimeByChar
        ) {
            if (self.options.onComplete) {
                self.options.onComplete.call(self, self.stringWriting);
            }
            self.initScannerDetection();
            return true;
        } else {
            if (self.options.onError) {
                self.options.onError.call(self, self.stringWriting);
            }
            self.initScannerDetection();
            return false;
        }
    };

    this.initScannerDetection = function () {
        self.firstCharTime = 0;
        self.stringWriting = '';
    };

    this.stopScanning = function() {
        document.onkeypress = null;
    };

    this.startScanning = function(){
        document.onkeypress = self.keypress.bind(self);
    };


}

Configuration(function (ContextRegister) {
    ContextRegister.register('ScannerDetection', ScannerDetection);
});

// FILE: js/services/ShortcutKeyService.js
function ShortcutKeyService(environment, KeyboardEventsHandler, Utilities) {

	var self = this;
	var keysMap = {};

	var last;

	var defaultKeysMap = {
		'down'      : ['2'        ],
		'left'      : ['4'        ],
		'zoom'      : ['YmFuYW5pbmhh','space'],
		'right'     : ['6'        ],
		'up'        : ['8'        ],
		'dot'       : ['apelidoDot'],
		'reload'    : ['**'       ],
		'backspace' : ['backspace'],
		'enter'     : ['enter'    ],
		'numlock'   : ['numlock'  ],
		'slash'		: ['slash'],
		'picklist'  : ['9'  ],
		'one'   	: ['1'  ],
		'priorityPanel': ['7']
	};

	this.bindShortcutsOrderPanel = function() {
		var localEnvironment = environment.getLocalEnvironment();
		var localKeyMap = localEnvironment.keyMapping;
		var mappedKeys = {};
		bindNumlockTurnedOffConvertion();

		// merge the backend keys whit the defalt keys and save in the front for future uses
		keysMap = Util.clone(defaultKeysMap);
		localKeyMap.forEach(function(shortcutKey) {
			if(keysMap[shortcutKey.name]){
				if(keysMap[shortcutKey.name].indexOf(shortcutKey.keyCode) == -1){
					keysMap[shortcutKey.name].push(shortcutKey.keyCode);
				}
			}else{
				keysMap[shortcutKey.name] = [shortcutKey.keyCode];
			}
		});

		bindMappedKeys(keysMap);
	};

	function bindMappedKeys() {
		unbindMappedKeys();
		Object.keys(keysMap).forEach(function(eventName) {
			var callbackName = 'execute' + Utilities.capitalizeFirstLetter(eventName) + 'Event';
			if(KeyboardEventsHandler[callbackName]) {
				Mousetrap.bind(keysMap[eventName], Util.buildDebounceMethod(KeyboardEventsHandler[callbackName]), 'keyup');
			}
		});
	}

	this.bindZoomMappedKeys = function() {
		unbindMappedKeys();
		Object.keys(keysMap).forEach(function(eventName) {
			var callbackName = 'executeZoom' + Utilities.capitalizeFirstLetter(eventName) + 'Event';
			if(KeyboardEventsHandler[callbackName]) {
				Mousetrap.bind(keysMap[eventName], Util.buildDebounceMethod(KeyboardEventsHandler[callbackName]), 'keyup');
			}
		});
	};

	this.bindSlashMappedKeys = function() {
		unbindMappedKeys();
		Object.keys(keysMap).forEach(function(eventName) {
			var callbackName = 'executeSlash' + Utilities.capitalizeFirstLetter(eventName) + 'Event';
			if(KeyboardEventsHandler[callbackName]) {
				Mousetrap.bind(keysMap[eventName], Util.buildDebounceMethod(KeyboardEventsHandler[callbackName]), 'keyup');
			}
		});
	};

	this.bindOneMappedKeys = function() {
		// unbindMappedKeys();
		Object.keys(keysMap).forEach(function(eventName) {
			var callbackName = 'executeOne' + Utilities.capitalizeFirstLetter(eventName) + 'Event';
			if(KeyboardEventsHandler[callbackName]) {
				Mousetrap.bind(keysMap[eventName], Util.buildDebounceMethod(KeyboardEventsHandler[callbackName]), 'keyup');
			}
		});
	};

	this.bindOneOneMappedKeys = function() {
		// unbindMappedKeys();
		Object.keys(keysMap).forEach(function(eventName) {
			var callbackName = 'executeOneOne' + Utilities.capitalizeFirstLetter(eventName) + 'Event';
			if(KeyboardEventsHandler[callbackName]) {
				Mousetrap.bind(keysMap[eventName], Util.buildDebounceMethod(KeyboardEventsHandler[callbackName]), 'keyup');
			}
		});
	};

	this.bindSlashEnterMappedKeys = function() {
		unbindMappedKeys();
		Object.keys(keysMap).forEach(function(eventName) {
			var callbackName = 'executeSlashEnter' + Utilities.capitalizeFirstLetter(eventName) + 'Event';
			if(KeyboardEventsHandler[callbackName]) {
				Mousetrap.bind(keysMap[eventName], Util.buildDebounceMethod(KeyboardEventsHandler[callbackName]), 'keyup');
			}
		});
	};

	this.bindPopUpKeys = function() {
		var bind = localStorage.getItem('bindPopupKeys');
		if(bind && bind == 'true') {
			unbindMappedKeys();
			document.addEventListener('keydown', handlePopupNavigation);
			last = 1;
			setTimeout(() => {
				document.querySelectorAll('.zh-alert-button')[0].focus();
			}, 500);
		}
		else {
			bindPopUpKeys();
		}
	};

	function handlePopupNavigation(e) {
		var buttons = document.querySelectorAll('.zh-alert-button');
 		if (e.key == '2' || e.key == 'ArrowDown') {
			buttons[last].focus();
			last = last == 0 ? 1 : 0;
		} else if (e.key == '8' || e.key == 'ArrowUp') {
			buttons[last].focus();
			last = last == 0 ? 1 : 0;
		} else if (e.key == 'Enter') {
			last = last == 0 ? 1 : 0;
			buttons[last].click();
			last = 0;
		} else {
			last = 0;
			buttons[last].focus();
			last = last == 0 ? 1 : 0;
		}
	}

	this.unbindPopUpKeys = function() {
		bindMappedKeys();
		document.removeEventListener('keydown', handlePopupNavigation, false);
	};

	this.unbindMappedKeys = function() {
		unbindMappedKeys();
	};

	this.bindMappedKeys = function() {
		bindMappedKeys();
	};

	function unbindMappedKeys() {
		Object.keys(keysMap).forEach(function(shortcutKey) {
			Mousetrap.unbind(keysMap[shortcutKey], 'keyup');
		});
	}

	function bindPopUpKeys() {
		Mousetrap.addKeycodes({
			37: 'left',
			38: 'up',
			39: 'right',
			40: 'down',
			104: 'up',
			98: 'down',
			100: 'left',
			102: 'right'
		});
	}

	function bindNumlockTurnedOffConvertion() {
		Mousetrap.addKeycodes({
			'144' : 'numlock',
			'109' : 'apelidoDot'    ,
			'36'  : '7'    ,
			'38'  : '8'    ,
			'33'  : '9'    ,
			'37'  : '4'    ,
			'12'  : '5'    ,
			'39'  : '6'    ,
			'194' : '.'    ,
			'35'  : '1'    ,
			'40'  : '2'    ,
			'34'  : '3'    ,
			'13'  : 'enter',
			'45'  : '0'    ,
			'46'  : ','    ,
			'110' : ',',
			'111' : 'slash',
			//o + e o * não binda no keyup sabe se lá o porque, mas com apelido funciona
			'107' : 'YmFuYW5pbmhh',
			'106' : '**',
			'104': '8',
			'97': '1',
			'98': '2',
			'100': '4',
			'102': '6',
			'101': '5',
		});
	}

}

Configuration(function(ContextRegister) {
	ContextRegister.register('ShortcutKeyService', ShortcutKeyService);
});

// FILE: js/services/SocketResponseHandler.js
function SocketResponseHandler(eventAggregator){

	function publishEvent(eventName, params) {
		eventAggregator.publish(eventName, params);
	}

	this.sendChanges = function(data) {
		publishEvent("onProductionResponse", data.orders);
	};

	this.sendConclusions = function(data) {
		publishEvent("onProductionConclude", data);
	};

	this.sendStatistics= function(data){
		publishEvent("onStatisticsUpdate", data);
	};

	this.deleteOrders = function(data) {
		publishEvent("onDeleteOrders", data);
	};

	this.freezeUpdates = function(data) {
		publishEvent("onFreezeUpdates", data);
	};

	this.unfreezeUpdates = function(data) {
		publishEvent("onUnfreezeUpdates", data);
	};

	this.newMessage = function(data) {
		publishEvent("onNewMessage", data);
	};

	this.deleteMessage = function(data) {
		publishEvent("onDeleteMessage", data);
	};

    this.pong = function(data) {
		publishEvent("onPong", data);
    };
}

// FILE: js/directives/kdsGearIcon.js
var KDSModule = angular.module("ZeedhiDirectives");

KDSModule.directive('kdsGearIcon', function() {

	return {
		restrict: 'E',
		templateUrl: 'templates/widget/kds_gear_icon.html', 
		link: function($scope, element, attrs) {}
	};
});

// FILE: js/directives/kdsMessage.js
var KDSModule = angular.module("ZeedhiDirectives");

KDSModule.directive('kdsMessage', function(ApplicationContext) {
	return {
        restrict: 'E',
		templateUrl: 'templates/widget/kds_message.html',
		compile: function(element, attributes){
         return {
                pre: function($scope, element, attributes, controller, transcludeFn){
                    $scope.configureAndPlayMessage = function(){
                        if($scope.message){
                            var animationTime = ($scope.message.DSMENSAGEM.length/3);
                            animationTime = animationTime > 30 ? animationTime : 30;
                            element.css({"-moz-animation": "scroll-left "+ animationTime +"s linear infinite",
                                         "-webkit-animation": "scroll-left "+ animationTime +"s linear infinite",
                                         "animation": "scroll-left "+ animationTime +"s linear infinite"});
                            $('.kds-message').css({"-moz-animation": "shrink "+ animationTime +"s linear infinite",
                                         "-webkit-animation": "shrink "+ animationTime +"s linear infinite",
                                         "animation": "shrink "+ animationTime +"s linear infinite"});
                        }
                    };

                    $scope.configureAndPlayMessage();
                },
                post: function($scope, element, attributes, controller, transcludeFn){
                }
            };
        }
	};
});

// FILE: js/directives/kdsMessagePanel.js
var KDSModule = angular.module("ZeedhiDirectives");

KDSModule.directive('kdsMessagePanel', function(ApplicationContext) {
	return {
		restrict: 'E',
		templateUrl: 'templates/widget/kds_message_panel.html',
		link: function($scope, element, attrs) {
            $scope.getMessages = function() {
                return ApplicationContext.ScreenMessagesService.getMessage();
            };
        }
	};
});

// FILE: js/directives/kdsOrder.js
var app = angular.module("ZeedhiDirectives");

app.filter('startFrom', function() {
    return function(input, start) {
        start = +start; 
        return input.slice(start);
    };
});

app.directive('kdsOrder', function (ApplicationContext){
	return {
		restrict: 'E',
		templateUrl: 'templates/widget/kds_order.html',
		link: function($scope, element, attrs){
			
			$scope.currentPage = 0;
			$scope.pageSize =  20;

			$scope.isNotConcludedItem = function(orderKey) {
				return !ApplicationContext.ConcludeQueue.checkItemExistence(orderKey);
			};

			$scope.putColorOnObservations = function(observation) {
				if (observation && observation.NRCORSINAL) {
					let cor = ApplicationContext.Utilities.idealTextColor(observation.NRCORSINAL);
					return `background: #${observation.NRCORSINAL}; color: ${cor};`;
				}
				let cor = ApplicationContext.Utilities.idealTextColor('FFFFFF');
				return `background: #FFFFFF; color: ${cor};`;
			};

			$scope.isGroupTotallyProducted = function(items) {
				items.every(function(item) {
					return item[widget.orderItemStatusField] == widget.itemDoneStatus || item[widget.orderItemStatusField] == widget.itemCancelledStatus;
				});
			};

			$scope.checkPositionIsOnHeader = function(positionIndex, position){
				return (positionIndex > 0 || position.length > 1);
			};

			$scope.checkItemsIsOnHeader = function(itemIndex, positionIndex){
				return (itemIndex > 0 && positionIndex === 0)|| (positionIndex > 0);
			};

			$scope.positionHasItemsOutOfConclusionQueue = function(position) {
				return position.some(function(item) {
					if(!item.isInConclusionQueue) {
						return true;
					} else if(item.items){
						item.items.some(function(compItem) {
							return !compItem.isInConclusionQueue;
						});
					} else {
						return false;
					}
				});
			};

			$scope.groupHasItemsOutOfConclusionQueue = function(group) {
				if(group.items){
					return group.items.some(function(item) {
						return !item.isInConclusionQueue;
					});
				} else {
					return !group.isInConclusionQueue;
				}
			};

			$scope.isOrderVisisble = function(order) {
                if (!order.canExibOrder) {
					return false;
				}
				if(order.IDCONDPEDPAG == 'N' || (order.IDCONDPEDPAG == 'S' && !!order.NRSEQMOVDLV && order.IDSITUVENDA == 'O') ) {
					if(!order.isInConclusionQueue){
						if(order && order.items) {
							return order.items.some(function(item) {
								return !item.isInConclusionQueue;
							});
						}
					} else {
						return false;
					}
				} else {
					return false;
				}
			};

			$scope.isOrderPriority = function(order) {
                if (order && order.IDPRODPRIORIDADE == false) {
					return false;
				}
				return true;
			};

			$scope.showOnlySummary = function(order) {
				if(order.items[0].IDSOMRESUMO == 'S'){
					order.hide = true;
					return true;
				} else {
					order.hide = false;
					return false;
				}
			};

			$scope.isOrderItemCancelled = function(order) {
				if(order && order.items && order.items.length > 0) {
					return order.items.every(checkIfItemIsCancelled);
				} else {
					return checkIfItemIsCancelled(order);
				}
			};

			$scope.isOrderBlinking = function(order){
				return order && order.colorParams && order.colorParams.blink;
			};

			function checkIfItemIsCancelled(item) {
				return item.IDSITITPEDFOS == 'C';
			}
		}
	};
});

// FILE: js/directives/kdsOrderHeader.js
var app = angular.module("ZeedhiDirectives");

app.directive('kdsOrderHeader', function ($interval, $timeout,  ApplicationContext){
	return {
		restrict: 'E',
		templateUrl: 'templates/widget/kds_order_header.html',
		link: function($scope, element, attrs){

			var widget = $scope.widget;
			var order = $scope.order;
			var orderActionService = ApplicationContext.OrderActionService;
			var colorService = ApplicationContext.ColorService;
			var environment = ApplicationContext.environment;
			var SoundService = ApplicationContext.SoundService;
			var Utilities = ApplicationContext.Utilities;
			var SummaryItemsService = ApplicationContext.SummaryItemsService;
			var ClientNotificationService = ApplicationContext.ClientNotificationService;
			var localEnvironment = environment.getLocalEnvironment();
			$scope.localEnvironment = environment.getLocalEnvironment();

            // Atribui um campo "quantidade" para o item pai, que será exibido no front evitando
            // a situação do pai repetir a quantidade do primeiro filho
            currentItens = order.items;

            currentItens.forEach(element => {
                
                if (element.IDPAIIMPRIME == 1 && element.IDPAICOMBO == 3){
                    element.fatherQtty = parseInt(element.QTPRODPAI);
                } else {
                    element.fatherQtty = element.QTPRODPEFOS;
                }
            });

            // Se for um pedido delivery (iFood, etc.), agrupa os adicionais no produto principal, se necessário
            if (currentItens[0].IDORIGEMVENDA && currentItens[0].IDORIGEMVENDA.substring(0, 3) === 'DLV') {

                let ultimoProdutoSemAgrupador = {
                    CDPRODUTO: null,
                    index: null
                };

                currentItens.forEach((item, index) => {
                    
                    if (item.NRSEQPRODCOMKDS === null) {
                        ultimoProdutoSemAgrupador.CDPRODUTO = item.CDPRODUTO;
                        ultimoProdutoSemAgrupador.index = index;
                    } else if (item.CDPRODPROMOKDS === ultimoProdutoSemAgrupador.CDPRODUTO) {
						if (currentItens[ultimoProdutoSemAgrupador.index]) {
                        	currentItens[ultimoProdutoSemAgrupador.index].NRSEQPRODCOMKDS = item.NRSEQPRODCOMKDS;
						}
                    }
                });
            }

			var orderStatus = {
				'start' : {
					'actionLabel': 'INICIAR',
					// 'actionIconPath': widget.imgSrc + 'start.svg',
					'actionIconPath': './images/start.svg',
					'actionCallback': orderActionService.startOrderProduction,
					'actionCondition': orderCanStart
				},
				'cancel' : {
					'actionLabel': 'CANCELAR',
					// 'actionIconPath': widget.imgSrc + 'cancel.svg',
					'actionIconPath': './images/cancel.svg',
					'actionCallback': orderActionService.expediteOrder,
					'actionCondition': orderCanCancel
				},
				'conclude': {
					'actionLabel': 'CONCLUIR',
					// 'actionIconPath': widget.imgSrc + 'conclude.svg',
					'actionIconPath': './images/conclude.svg',
					'actionCallback': orderActionService.concludeOrderProduction,
					'actionCondition': orderCanConclude
				},
				'expedite' :  {
					'actionLabel': 'LIBERAR',
					// 'actionIconPath': widget.imgSrc + 'expedite.svg',
					'actionIconPath': './images/expedite.svg',
					'actionCallback': orderActionService.expediteOrder,
					'actionCondition': orderCanExpedite
				},
				'continue': {
					'actionLabel': 'CONTINUAR',
					// 'actionIconPath': widget.imgSrc + 'start.svg',
					'actionIconPath': './images/start.svg',
					'actionCallback': orderActionService.continueOrder,
					'actionCondition': orderCanContinue
				},
				'waiting_payment' : {
					'actionLabel': 'AGUARD. PGTO.',
					'actionCondition': orderWaitingPg,
					'actionCallback': _ => null,
				}
			};

			$scope.onOrderClick = function(order, isAutoClick) {

				orderActionService.orderOnClick(order, isAutoClick);
				if (order.isCallClientAvailable() && order.status == 'expedite') {
					ClientNotificationService.notifyClient(order);
				}

			};

			order.updateAvailableAction = function () {
				order.availableAction = null;
				Object.keys(orderStatus).forEach(function (action) {
					if (orderStatus[action].actionCondition()) {
						order.availableAction = orderStatus[action];
						order.status = action;
						if (localEnvironment.PLAYANDCONCLUDE && localEnvironment.IDTIPOSETOR == 'P' && order.availableAction.actionLabel == 'INICIAR') {
							let isAutoClick = true;
                            $scope.onOrderClick(order, isAutoClick);
						}
					}
				});
				if (!$scope.$$phase) {
					$scope.$apply();
				}
			};

			function orderCanStart() {
				var orderCurrentTime = moment(order[widget.orderCurrentTime]);
				var orderShowTime = moment(order[widget.orderShowTime]);
				return  order[widget.orderTimeField] !== 'wait' &&
					widget.kdsType === 'production' &&
					order.items.filter(function(item) {
						return item[widget.orderItemStatusField] !== widget.itemCancelledStatus;
					}).length &&
					orderCurrentTime.isAfter(orderShowTime) &&
					order.items.every(function(item) {
						return item[widget.orderItemStatusField] !== widget.itemProductionStatus;
					});
			}

			function orderCanExpedite() {
				var	canExpedite = widget.kdsType === 'expedition' &&
					!orderCanCancel() &&
					order.items.every(function(item) {
						return item[widget.orderItemSelectField];
					});

				if(localEnvironment.IDEXIBCONDLIB !== 'S' || canExpedite || widget.kdsType === 'production' ){
					order.canExibOrder = true;
				}
				return canExpedite;
			}

			function orderCanContinue() {
				return order.isPausedByInterval;
			}

			function orderCanConclude() {
				var concludeDisabled = localEnvironment.concludeDisabled;
				var canConclude =  order.isProducted || (!concludeDisabled && widget.kdsType ===  'production' &&
					!order.isPausedByInterval &&
					order.items.filter(function(item) {
						return item[widget.orderItemStatusField] !== widget.itemCancelledStatus;
					}).length &&
					order.items.some(function(item) {
						return item[widget.orderItemStatusField] === widget.itemProductionStatus;
					}));

					if(canConclude){
						
						return true;
					}
			}

			function orderCanCancel() {
				return order.items.every(function(item) {
					return item[widget.orderItemStatusField] === widget.orderCancelledStatus || (order.items.every(function(item) {
						return item[widget.orderItemStatusField] === widget.orderCancelledStatus;
					}));
				});
			}

			function orderWaitingPg(){
				return ((orderCanExpedite()) && (localEnvironment.IDTIPOSETOR == 'E') && (order.DTVENDA == null && order.NRSEQMOVDLV == null && order.DSTIPOPEDFOS == 'COMANDA') && (localEnvironment.IDLIBCONDPG == 'S'));
			}

			function createDurationString(duration) {
				var durationString = '00:00:00';
				if(duration) {
					durationString = Utilities.zerofillNumberFormat(duration.hours(), 2) + ':' + Utilities.zerofillNumberFormat(duration.minutes(), 2) + ':' + Utilities.zerofillNumberFormat(duration.seconds(), 2);
				}
				return durationString;
			}

			$scope.showOrderOrigin = function () {
				return localEnvironment.IDMOSTRAORIGEM && order[widget.orderOrigin];
			};

			$scope.isDelivery = function (orderOrigin) {
				return orderOrigin && orderOrigin.includes("DELIVERY");
			};

			$scope.isToGo = function (orderOrigin) {
				return orderOrigin && orderOrigin.toUpperCase().includes("VIAGEM");
			};

			$scope.getOrderTimerString = function() {
				var productionTime;
				if(order && order.timerValue) {
					if(moment.isDuration(order.timerValue)) {
						productionTime = createDurationString(order.timerValue);

						if(productionTime == "00:00:00") {
							//this may fail, if for some reason the front time is walking diferent than the back
							order[widget.orderTimeField] = "0001/01/01 00:00:00";
							order.timerValue = null;
							order[widget.orderCurrentTime] = order.DTHREXIBICAOPRODISO;
							order[widget.orderShowTime] = order[widget.orderCurrentTime];
							order.prepareOrder();
							order.updateAvailableAction();
							order.startOrderTimer();
							order.getOrderColor();
						}
					} else {
						productionTime = order.timerValue.format('HH:mm:ss');
					}
				} else {
					productionTime = "00:00:00";
				}
				return productionTime;
			};

			$scope.getFreezedTime = function() {
				var currentDTHPRIEXPED;
				if(order.DTHRPRIEXPED){
					currentDTHPRIEXPED = moment(order.DTHRPRIEXPED).diff(moment(order.DTHREXIBICAOPROD));
					return moment.utc(currentDTHPRIEXPED).format('HH:mm:ss');
				} else {
					return null;
				}
			};

			order.updateOrderTimer = function() {
				
				let productionTime = moment(moment.utc().toDate() - moment.utc(order.DTHREXIBICAOPROD).toDate());
				if(moment.isDuration(order.timerValue)) {
					if(order.timerValue.asMilliseconds() !== 0) {
						order.timerValue.subtract(1, 'seconds');
					}
				} else {
					order.timerValue = productionTime;
					//order.timerValue.add(1, 'seconds');
				}
				updateProductionTimer();
			};

			function updateProductionTimer() {
				if(!order.isInConclusionQueue){
					if(order.IDSITITPEDFOS == 'P' && widget.productionProgressBar){
						if(order.productionTimer.format('HH:mm:ss')!=='00:00:00' && !order.isPausedByInterval){
							order.productionTimer.subtract(1, 'seconds');
						} else {
							if(localEnvironment.IDHABSOMPROD!='N'){
								$timeout(function(){
									if(localEnvironment.IDHABCONTSOMPROD!='N'){
										SoundService.beep(true);
										order.isOrderBeeping = true;
									} else {
										SoundService.beep(false);
									}
								}, localEnvironment.NRATRASOSOMPROD*1000 || 0, false);
							}
							if(orderCanConclude()){
								if(localEnvironment.IDCONCAUT === 'S' && order.getProductionTimer() === "00:00:00" ){
									orderActionService.concludeOrderProduction(order);
								}
							}
						}
					}
				}
			}

			function startUpdateOrderInterval() {
				//order.timerInterval = $interval(updateOrderTimer, 1000, 0, false);
			}


			function calcuteWaitingTimeToStartProduction(order) {
				var exibitionTime = moment(order.DTHREXIBICAOPRODISO, 'YYYY/MM/DD HH:mm:ss');
				var now = moment(order.NOW, 'YYYY/MM/DD HH:mm:ss');
				var diffTime = exibitionTime.diff(now);
				return moment.duration(diffTime, 'milliseconds');
			}

			order.startOrderTimer = function() {
				//calcula o tempo do cabeçalho
				if(!order.timerValue) {
					var elapsedTime = order[widget.orderTimeField];//timer
					if (elapsedTime == 'wait') {
						order.timerValue = calcuteWaitingTimeToStartProduction(order);
						order.productionTimer = moment((Math.floor(order.NRTEMPOEXIB/60)).toString()+':'+(order.NRTEMPOEXIB%60).toString(), 'mm:ss');
					} else {
						elapsedTime = moment(elapsedTime, 'YYYY/MM/DD HH:mm:ss');
						elapsedTimeText = moment(elapsedTime);
						order[widget.orderTimeField] = elapsedTime.format('YYYY/MM/DD HH:mm:ss');
						order.timerValue = elapsedTime;
						//calcula o tempo de produção
						if(widget.productionProgressBar){
							var elapsedProductionTime = {};
							if(order.IDSITITPEDFOS == 'P'){
								elapsedProductionTime = moment(order.NOW).diff(order.DTHRINIPRODUCAO, 's');
								setOrderProductionItervals(order, elapsedProductionTime);
								elapsedProductionTime = order.NRTEMPOEXIB - elapsedProductionTime;
								elapsedProductionTime = elapsedProductionTime < 0 ? 0 : elapsedProductionTime;
							} else {
								elapsedProductionTime = order.NRTEMPOEXIB;
							}
							order.productionTimer = moment((Math.floor(elapsedProductionTime/60)).toString()+':'+(elapsedProductionTime%60).toString(), 'mm:ss');
						}
					}
					startUpdateOrderInterval();
				}
			};

			function setOrderProductionItervals(order, elapsedProductionTime) {
				if(Util.isArray(order.INTERVALS) && !Util.isEmptyOrBlank(order.INTERVALS)) {
					var productionInterval = parseInt(order.INTERVALS.shift().NRTEMPOINTER.toString());
					if(productionInterval < elapsedProductionTime) {
						setOrderProductionItervals(order, elapsedProductionTime);
					} else {
						var milisecondsToPause = (productionInterval - elapsedProductionTime) * 1000;
						order.productionIntervalTimeout = $timeout(function(){
							order.isPausedByInterval = true;
							order.updateAvailableAction();
						}, milisecondsToPause, false);
					}
				}
			}

			order.getProductionTimer = function(){
				if(!moment.isMoment(order.productionTimer)){
					order.productionTimer = moment(order.productionTimer);
				}
				return order.productionTimer.format('HH:mm:ss');
			};

			order.setOrderColor = function setOrderColor(orderColor, isOrderBlinking){
				order.colorParams = {
					color :  orderColor,
					blink : isOrderBlinking
				};
			};

			order.updateOrderColor = function(color) {
				if (!orderCanCancel()) {
					order.setOrderColor(color.NRCORPEDIDO, color.IDPISCAPED === 'S');
					order.setNextChangeColorTimer();
				}
			};
			//getorder color AND set delayed beep on expedition
			order.getOrderColor = function getOrderColor() {
				if(orderCanCancel()){
					order.setOrderColor('CCCCCC', false);
				}else{
					if(!order.keepOrderColor){
						order.setOrderColor('000000', false);
						if(order.colorTimeout && !order.keepOrderColor){
							$timeout.cancel(order.colorTimeout);
						}
						var elapsedTime = order[widget.orderTimeField];//timer
						var orderColor, isBlinking, delay = 0;
						if (elapsedTime == 'wait'){
							order.setOrderColor('ADADAD', false);
						} else {
							//se tiver cancelado não precisa preocupar com delaytime
							var exibitionTime = moment(order.DTHREXIBICAOPRODISO, 'YYYY/MM/DD HH:mm:ss');
							var now = moment(order.NOW, 'YYYY/MM/DD HH:mm:ss');
							var diffTime = now.diff(exibitionTime, 'seconds');
							delay = diffTime - order.NRTEMPOPROD;
							order.setDelayedBeepTimeout(delay);
							var colors = colorService.getControlColors();
							if (!Util.isEmptyOrBlank(colors)){
								order.setChangeColorTimers(delay, colors);
								order.setNextChangeColorTimer();
								if(delay < 0 && order.IDSITITPEDFOS === 'E' && !localEnvironment.PLAYANDCONCLUDE && localEnvironment.IDTIPOSETOR == 'P') {
									order.setOrderColor('0066ff', false);// azul baldi
								}
							} else { //caso não tenha cor parametrizada para este intervalo
								order.setOrderColor('000000', false);
							}
						}
					}
				}
			};

			order.setChangeColorTimers = function setChangeColorTimers (delay, colors){
				order.orderColorQueue = [];
				var lastDelay = 0;
				for(index = 0; index < colors.length; index++){
					color = colors[index];
					if(delay <= color.NRINICIOCOR){
						color.delayTime = color.NRINICIOCOR - delay - lastDelay;
						if(delay < 0 && color.NRINICIOCOR === 0){
							order.setOrderColor(color.NRCORPEDIDO, color.IDPISCAPED === 'S');
						}else{
							lastDelay = color.delayTime + lastDelay;
							order.orderColorQueue.push(color);
						}
					}else if(delay >= color.NRINICIOCOR && delay <= color.NRFINALCOR){
						order.setOrderColor(color.NRCORPEDIDO, color.IDPISCAPED === 'S');
					}
				}
			};

			order.setNextChangeColorTimer = function setNextChangeColorTimer() {
				if(order){
					var nextColor = order.orderColorQueue.shift();
					if(nextColor){
						order.colorTimeout = $timeout(function(){order.updateOrderColor(nextColor);}, nextColor.delayTime * 1000, 0, true);
					}
				}
			};

			order.setDelayedBeepTimeout = function setDelayedBeepTimeout(delay) {
				if(localEnvironment.IDTIPOSETOR == 'E' && localEnvironment.NRTEMPOATRASOEXP != null){
					order.expBeepTimeout = $timeout(function(){
						order.expeditionDelayBeep = true;
						SoundService.playSound(localEnvironment.SOMBEEPEXP, 'expeditionDelayBeep');
					}, (localEnvironment.NRTEMPOATRASOEXP - delay) * 1000, 0, true);
				}
			};

			$scope.getKDSTitleStyle = function() {
				let IDHABPRIORIDADE = localEnvironment.IDHABPRIORIDADE;
				// caso o pedido seja prioritario e o setor esteja com o parametro IDHABPRIORIDADE habilitado a cor do pedido deve ser roxa
				if (IDHABPRIORIDADE && IDHABPRIORIDADE === 'S') {
					if(order && order.IDPRODPRIORIDADE === true) {
						let bgColor = '#8A05BE';
						let toApplyClass = {
							'color' : order.colorParams.blink ? '#FFF': ApplicationContext.Utilities.idealTextColor(bgColor),
							'background-color' : bgColor,
							'background-size' : order.IDSITITPEDFOS === 'E' ? '10px 10px' : 'initial'
						};
						return toApplyClass;
					} else {
						if(order && order.colorParams) {
							let bgColor = order.colorParams.color;
							let toApplyClass = {
								'color' : order.colorParams.blink ? '#FFF': ApplicationContext.Utilities.idealTextColor(bgColor),
								'background-color' : '#' + bgColor,
								'background-size' : order.IDSITITPEDFOS === 'E' ? '10px 10px' : 'initial'
							};
							return toApplyClass;
						}
					}
				} else if(order && order.colorParams) {
					let bgColor = order.colorParams.color;
					let toApplyClass = {
						'color' : order.colorParams.blink ? '#FFF': ApplicationContext.Utilities.idealTextColor(bgColor),
						'background-color' : '#' + bgColor,
						'background-size' : order.IDSITITPEDFOS === 'E' ? '10px 10px' : 'initial'
					};
					return toApplyClass;
				}
			};

			$scope.isCommandNumberVisible = function() {
				return order[widget.orderCommandNumberField] && order[widget.orderCommandNumberField].length && parseInt(order[widget.orderCommandNumberField]).toString() !== 'NaN';
			};

			$scope.getCommandNumber = function() {
				return parseInt(order[widget.orderCommandNumberField]).toString();
			};

			$scope.getKDSItemQtdStyle = function() {
                if(order && order.colorParams) {
                    const color = ['000000', 'ffffff'].includes(order.colorParams.color) ? order.colorParams.color : '000000';
                    var toApplyClass = {
                        'color' : '#' + color
                    };
                    return toApplyClass;
                }
            };

			$scope.getKDSItemIconStyle = function() {
				if(order && order.colorParams) {
					var toApplyClass = {
						'stroke' : '#' + order.colorParams.color
					};
					return toApplyClass;
				}
			};

			$scope.getBorderStyle = function() {
				var toApplyClass = {
					'border-color':  order && order.colorParams && order.isSelected ? '#' + order.colorParams.color : '#dddddd',
					'-webkit-animation': order && order.colorParams && order.isSelected ? 'order-border-blink 0.5s linear infinite alternate' : 'initial'
				};
				return toApplyClass;
			};

			$scope.isOrderBlinking = function(){
				return order && order.colorParams && order.colorParams.blink;
			};

			$scope.getCallClientStyle = function() {
				var bgColor = order.colorParams.color;
				return {
					'fill':  ApplicationContext.Utilities.idealTextColor(bgColor)
				};
			};

			$scope.getProgressBarPercentage = function() {
				// porcentagem do tempo de produção (fiz regra de 3 mesmo)
				var productionTotalTime = order.NRTEMPOEXIB;
				var elapsedProductionTime = order.productionTimer;

				var productionPorc = ((((elapsedProductionTime.minutes() * 60) + elapsedProductionTime.seconds()) * 100) / (productionTotalTime));
				var width = 100 - productionPorc;
				if ((width < 0) || (width > 100)) {
					width = 100;
				}
				if(width > 80 && order.IDSITITPEDFOS == 'P' && !order.isProducted){
					order.isProducted = true;
					order.updateAvailableAction();
				}
				return {
					'width' : width + '%',
					'background-color' : tinycolor(order.colorParams.color).setAlpha(0.5).toRgbString()
				};
			};

			order.isCallClientAvailable = function () {
				var canExpedite = orderCanExpedite();
				var isExpeditionSector = localEnvironment.IDTIPOSETOR == 'E';
				var projectConfig = environment.getProjectConfig();
				return canExpedite && projectConfig.callClientIntegration && isExpeditionSector;
			};

			order.isCallPanelAvailable = function () {
				var canExpedite = orderCanExpedite();
				var isExpeditionSector = localEnvironment.IDTIPOSETOR == 'E';
				var projectConfig = environment.getProjectConfig();
				return canExpedite && projectConfig.callPanelIntegration && isExpeditionSector;
			};

			order.beepOnNewOrder = function() {
				var isTimeToBeep = localEnvironment.IDHABSOMKDS == 'S' && order.timer !== 'wait' && ((order.IDSITITPEDFOS == 'E' && widget.IDTIPOSETOR == 'P') || (order.isNewOrder && widget.IDTIPOSETOR != 'P') );
				if (isTimeToBeep) {
					if(localEnvironment.NRPRODATRAINISOM) {
						$timeout(function() {
							if(localEnvironment.IDCONTSOMKDS!='N'){
								SoundService.beep(true);
								order.isOrderBeeping = true;
								// order.beepUntilStart = $interval(SoundService.beep, 1000, 0, false);
							}
						}, localEnvironment.NRPRODATRAINISOM * 1000, true);
					}
					SoundService.beep(false);
				}
			};

			order.prepareOrder = function() {
				if(!order.isNewOrder) {
					order.timerValue = null;
				}
				order.items.forEach(function(item) {
					if (item.QTPRODPEFOS && (item.QTPRODPEFOS instanceof Number || item.QTPRODPEFOS.toString().indexOf('.') != -1)) {
						item.QTPRODPEFOS = item.QTPRODPEFOS.toFixed(3);
					}
					if (item[widget.orderItemObsField]) {
						try {
							if (typeof item[widget.orderItemObsField] === 'string') {
								item[widget.orderItemObsField] = JSON.parse(item[widget.orderItemObsField]);

                                item[widget.orderItemObsField].forEach(((obs, index) => {

                                    if (obs.OBSERVACAO.includes(';')) {
                                        item[widget.orderItemObsField].splice(index, 1);
                                    }
                                    obs.OBSERVACAO = obs.OBSERVACAO.replace('[', '').replace(']', '');
                                }));
							}
						} catch (e) {
							var obsArray = (item[widget.orderItemObsField].split && item[widget.orderItemObsField].split(';') || []),
								retArray = [];
							obsArray.forEach(function(observation, idx, obsArray) {
								if (observation) {
									var returnObs = {};
									returnObs.OBSERVACAO = observation;
									returnObs.NRCORSINAL = 'FFFFFF';
									retArray.push(returnObs);
								} else if (obsArray.length-1 == idx){
									item[widget.orderItemObsField] = retArray;
								}
							});
						}
					}
					if (item[widget.itemCanceledArray]) {
						if (typeof item[widget.itemCanceledArray] === 'string') {
							item[widget.itemCanceledArray] = JSON.parse(item[widget.itemCanceledArray]);
						}
					}
				});
			};

			order.clearBeeps = function() {
				order.isOrderBeeping = false;
			};

			$scope.$on('$destroy', function() {
				order.clearBeeps();
			});

			order.clearBeeps();
			order.prepareOrder();
			order.updateAvailableAction();
			order.startOrderTimer();
			order.getOrderColor();
			order.beepOnNewOrder();

		}
	};
});

// FILE: js/directives/kdsPriorityPanel.js
var KDSModule = angular.module("ZeedhiDirectives");

KDSModule.directive('kdsPriorityPanel', function(ApplicationContext) {
	return {
		restrict: 'E',
		templateUrl: 'templates/widget/kds_priority_panel.html'
	};
});

// FILE: js/directives/kdsResumePanel.js
var KDSModule = angular.module("ZeedhiDirectives");

KDSModule.directive('kdsResumePanel', function(ApplicationContext) {
	return {
		restrict: 'E',
		templateUrl: 'templates/widget/kds_resume_panel.html',
		link: function($scope, element, attrs) {
			var environment = ApplicationContext.environment;
			var localEnvironment = environment.getLocalEnvironment();
			$scope.aumentaResumo = function () {
				if (localEnvironment.IDAUMENTARESUMO == 'S'){
					return true;
				} else {
					return false;
				}
				
			};
			
        }
	};
});

// FILE: js/directives/kdsStatistics.js
var app = angular.module("ZeedhiDirectives");

app.directive('kdsStatistics', function (ApplicationContext){
	return {
		restrict: 'E',
		templateUrl: 'templates/widget/kds_statistics.html',
		link: function($scope, element, attrs){
			var environment = ApplicationContext.environment;
            let widget =  environment.getKDSWidget();
			$scope.localEnvironment = environment.getLocalEnvironment();

			$scope.kdsHabPedLib = $scope.localEnvironment.IDHABPEDLIB == 'S';
			$scope.habNewStatistics = function () {
				if ($scope.localEnvironment.IDHABESTATISTICAS == 'S'){
					return true;
				} else {
					return false;
				}				
			};

            widget.showDateTime = localStorage.getItem('showDateTime');

            if (widget.showDateTime) {

                widget.dateTime = moment().format('DD/MM/YYYY - HH:mm');
                let waitMilliSeconds = (60 - moment().seconds()) * 1000;
                // O setTimeout sincroniza o horário da primeira atualização com o relógio. Em seguida, o setInterval
                // passa a repetir a atualização a cada 60 segundos.
                setTimeout(() => {

                    widget.dateTime = moment().format('DD/MM/YYYY - HH:mm');
                    setInterval(() => {
                        widget.dateTime = moment().format('DD/MM/YYYY - HH:mm'); 
                    }, 60000);

                }, waitMilliSeconds);
            }
		}
	};
});

// FILE: js/directives/kdsSummary.js
var KDSModule = angular.module("ZeedhiDirectives");

KDSModule.directive('kdsSummary', function() {
	return {
		restrict: 'E',
		templateUrl: 'templates/widget/kds-summary.html',
		link: function($scope, element, attrs) {

        }
	};
});

// FILE: js/directives/kdsZoom.js
var app = angular.module("ZeedhiDirectives");

app.directive('kdsZoom', function (ApplicationContext){
	return {
		restrict: 'E',
		templateUrl: 'templates/widget/kds_zoom.html',
		link: function($scope, element, attrs){
			ApplicationContext.ShortcutKeyService.bindZoomMappedKeys();
		}
	};
});

// FILE: js/directives/zhKdsContainer.js
var KDSModule = angular.module("ZeedhiDirectives");

KDSModule.directive('zhKdsContainer', function($interval, ApplicationContext) {
	return {
		restrict: 'A',
		link: function($scope, element, attrs) {
			"use strict";

			var widget = $scope.widget;
			let watchers;

			$scope.$on('suspend', function () {
				watchers = $scope.$$watchers;
				$scope.$$watchers = [];
			});
		
			$scope.$on('resume', function () {
				if (watchers)
					$scope.$$watchers = watchers;
		
				watchers = void 0;
			});

			$interval(updateOrdersTimers, 1000, 0, false);

			function updateOrdersTimers () {
				widget.dataSource.data.forEach(function(order) {
					if(order.updateOrderTimer) {
						order.updateOrderTimer();
					}
				});
			}

			widget.bindPanelShortcutKeys = function() {
				ApplicationContext.ShortcutKeyService.bindShortcutsOrderPanel();
			};

			widget.orderDataSouce = function () {
				removeOrdersCompleted();
				widget.dataSource.data = removeOrders();
				// widget.dataSource.data = removeEmptyOrders();
				widget.dataSource.data = groupItemsExpedition(widget.dataSource.data); // Agrupa os Itens Iguais na expedição e montagem
				widget.dataSource.data.sort(sortDataSource);
				widget.dataSource.hasPriorityOrders = hasPriorityOrders();
				// Dar um update template se não rolar
			};

			widget.orderDataSourceByPositionTable = function () {
				widget.dataSource.data.sort(sortDataSourceByPositionTable);
			};

			widget.orderDataSourceByActionLabel = function () {
				widget.dataSource.data.sort(sortDataSourceByActionLabel);
			};

			function sortDataSource(a, b) {
				var orderDateA,orderDateB;
				orderDateA = chooseOrdenationTime(a, orderDateA);
				orderDateB = chooseOrdenationTime(b, orderDateB);

				if(orderDateA.isSame(orderDateB, 'second')){
					if(a.NRPEDIDOFOS == b.NRPEDIDOFOS){
						if(moment(a.DTACRPRIORTIME).isSame(moment(b.DTACRPRIORTIME), 'second')) {
							return a.orderKey > b.orderKey? 1:-1;
						} else {
							if(moment(a.DTACRPRIORTIME).isBefore(moment(b.DTACRPRIORTIME))) {
								return -1;
							} else {
								return 1;
							}
						}
					} else {
						return a.NRPEDIDOFOS > b.NRPEDIDOFOS? 1:-1;
					}
				} else {
					return orderDateA.isBefore(orderDateB, 'second') ? -1 : 1;
				}

			}

			function sortDataSourceByPositionTable (a, b) {
				if (a.NRPEDIDOFOS == b.NRPEDIDOFOS) {
					if(a.NRLUGARMESAIT == b.NRLUGARMESAIT){
						if(parseInt(a.NRTEMPOPROD) == parseInt(b.NRTEMPOPROD)){
							if(a.NRPRODCOMVEN){
								if(parseInt(a.NRPRODCOMVEN) < parseInt(b.NRPRODCOMVEN)){
									return -1;
								}else{
									return 1;
								}
							} else if (parseInt(a.NRSEQUITVEND < b.NRSEQUITVEND)){
								return -1;
							} else {
								return 0;
							}
						} else {
							return parseInt(a.NRTEMPOPROD) < parseInt(b.NRTEMPOPROD) ? 1 : -1;
						}						
					}else{
						return a.NRLUGARMESAIT > b.NRLUGARMESAIT? 1 : -1;
					}
				}else {
					return a.NRPEDIDOFOS > b.NRPEDIDOFOS? 1 : -1;
				}
			}

			function sortDataSourceByActionLabel (a, b) {
				/* ordena o array colocando os itens produzidos nas primeiras posições */
				let actionA = orderCanExpedite(a);
				let actionB = orderCanExpedite(b);
				let nrPedidoA = parseInt(a.NRPEDIDOFOS);
				let nrPedidoB = parseInt(b.NRPEDIDOFOS);

				if (actionA === actionB) {
					if (nrPedidoA === nrPedidoB) {
						return parseInt(a.NRTEMPOPROD) < parseInt(b.NRTEMPOPROD) ? 1 : -1;
					}
					return nrPedidoA > nrPedidoB ? 1 : -1;
				}

				return (actionA === true) ? -1 : 1;
			}

			function orderCanExpedite(order) {
				// Verifica se todos os itens estão produzidos
				let	canExpedite = order.items.every(function(item) {
					return item[widget.orderItemSelectField];
				});
				return canExpedite;
			}

			function chooseOrdenationTime(order, orderDate) {
				if (order.NRSEQLIBERADO || order.items[0] && (order.items[0].NRATRAPRODITPE && order.items[0].NRATRAPRODITPE !== 0)) {
					var NRMAIORTEMPO = order.NRMAIORTEMPO && widget.IDTIPOSETOR == 'P' ? order.NRMAIORTEMPO: 0;
					return moment(order.DTACRPRIORTIME).subtract(NRMAIORTEMPO, 'seconds');
				} else {
					return moment(order.DTACRPRIORTIME);
				}
			}

			function groupItemsExpedition (orders) {
				let allowGroup = localStorage.getItem('groupItemsExpedition') === 'true';
				let environment = ApplicationContext.environment.getLocalEnvironment();
				if (!allowGroup) {
					return orders;
				}

				for (let i = 0; i < orders.length; i++) {
					let order = orders[i];
					if (order && order.IDTIPOSETOR != 'P'){
						order.items = checkAndGroupItems(order.items);
					}
				}
				return orders;
			}
			// armazena a checkAndGroupItems no widget para ser chamada quando tranferir os itens de mesa
			widget.checkAndGroupItems = checkAndGroupItems;

			function checkAndGroupItems(items) {
				if (!items || items.length <= 1) {
					return items;
				}
			
				let groupedItens = [];
				let itemsToRemove = new Set();
			
				items.forEach((e) => {
					if (itemsToRemove.has(e)) {
						return;
					}
			
					let itensIguais = items.filter((element) => {
						return (
							element.CDSETOR === e.CDSETOR &&
							element.NRPEDIDOFOS === e.NRPEDIDOFOS &&
							element.CDSENHAPED === e.CDSENHAPED &&
							element.NRSEQPRODCOMKDS === e.NRSEQPRODCOMKDS &&
							element.CDPRODPROMOKDS === e.CDPRODPROMOKDS &&
							element.CDPRODUTO === e.CDPRODUTO &&
							element.DSLUGARMESAIT === e.DSLUGARMESAIT &&
							element.TXPRODPED === e.TXPRODPED &&
							((e.IDSITITPEDFOS !== 'C' && ['E', 'P', 'F'].includes(element.IDSITITPEDFOS)) ||
							 (e.IDSITITPEDFOS === 'C' && element.IDSITITPEDFOS === 'C'))
						);
					});
			
					if (itensIguais.length > 1) {
						/* jshint ignore:start */
						let groupedItem = { ...e }; // jshint ignore:line
						/* jshint ignore:end */
						let quantidade = itensIguais.reduce((total, it) => total + parseFloat(it.QTPRODPEFOS), 0);
			
						itensIguais.forEach(it => itemsToRemove.add(it));
						
						groupedItem.QTPRODPEFOS = quantidade;
						groupedItem.groupedItens = itensIguais;
						groupedItem[widget.orderItemSelectField] = itensIguais.every(item => item[widget.orderItemSelectField]);
						groupedItem[widget.orderItemStatusField] = itensIguais.every(it => it[widget.orderItemStatusField] === widget.itemDoneStatus) ? 'F' : 'E';
			
						groupedItens.push(groupedItem);
					}
				});
			
				items = items.filter(item => !itemsToRemove.has(item));
				items = items.concat(groupedItens);
			
				return items;
			}

			widget.updateKDSTemplate = function() {
				if(!$scope.$$phase) {
					$scope.$broadcast('suspend');
					$scope.$digest();
					$scope.$broadcast('resume');
				}
			};

			widget.getItemsArray = function(order){
				const sortItemsByComboOrder =  localStorage.getItem('sortItemsByComboOrder') || false;

				//TA VENDO ISSO AQUI? É UM CACHE NA MÃO
				if (order.LASTUPDATE != order.cacheLastUpdate) {
					order.cacheLastUpdate = order.LASTUPDATE;
					var processedItens = buildOrderItemsArray(order, sortItemsByComboOrder);
					order.processedItens = processedItens;
				}
				order.processedItens.forEach(function(items){
					items.sort(function (a, b) {
						var comparadorA = a[widget.orderItemGroupDescField] || a[widget.orderItemDescField];
						var comparadorB = b[widget.orderItemGroupDescField] || b[widget.orderItemDescField];
						if (sortItemsByComboOrder === 'true' || sortItemsByComboOrder === true) {
							comparadorA = a[widget.orderItemComboPositionField] || a[widget.orderItemGroupDescField] || a[widget.orderItemGroupDescField];
							comparadorB = b[widget.orderItemComboPositionField] || b[widget.orderItemGroupDescField];
						}
						if (comparadorA > comparadorB) {
							return 1;
						}
						if (comparadorA < comparadorB) {
							return -1;
						}
						if(a.NRITPEDIDOFOS > b.NRITPEDIDOFOS){
							return 1;
						}else{
							return -1;
						}
						// a must be equal to b
						return 0;
					});
				});
				return order.processedItens;
			};

			function buildOrderItemsArray(order, sortItemsByComboOrder = false) {
				var orderGroupedByTablePosition = groupOrderByTablePosition(order);

				let retorno = Object.keys(orderGroupedByTablePosition).map(function(tablePosition) {
					var groups = buildItemsGroups(orderGroupedByTablePosition[tablePosition]);
					var items = buildLonelyItems(orderGroupedByTablePosition[tablePosition]);
					return items.concat(groups);
				});
				// esconde o item igual ao pai
				retorno = hiddenSameItems(retorno);
				// ordena os itens pelo NRSEQITVENDAEST - controle de combo do combo
				if (sortItemsByComboOrder === 'true') {
					return sortOrderByComboPosition(retorno);
				}

				return retorno;
			}

			function hiddenSameItems(orders) {
				let localEnvironment = ApplicationContext.environment.getLocalEnvironment();
				if (widget.IDTIPOSETOR === 'P' && localEnvironment.IDHABAGRUPPROD !== 'S') {
					return orders;
				}
				if (widget.IDTIPOSETOR === 'P' && localEnvironment.SHOWCOMBOPRODUCTION !== 'S') {
					return orders;
				}
				if (widget.IDTIPOSETOR !== 'P' && widget.IDHABEXPFLUID === 'S') {
					return orders;
				}
				// verificar IDHABEXPFUID e SHOWCOMBOPRODUCTION
				if (orders && orders.length) {
					for (let i = 0; i < orders.length; i++) {
						if (orders[i] && orders[i].length) {
							for (let ii = 0; ii < orders[i].length; ii++) {
								if (orders[i][ii] && orders[i][ii].items && orders[i][ii].items.length > 0) {
									for (let iii = 0; iii < orders[i][ii].items.length; iii++) {
										let hasItems = orders[i][ii].items && orders[i][ii].items[iii];
										let condition1 = orders[i][ii].items[iii].CDPRODUTO === orders[i][ii].items[iii].CDPRODPROMOKDS;
										let condition2 = orders[i][ii].items[iii].NMPRODUTO === orders[i][ii].items[iii].NMPROMOCAO;
										let condition3 = false; // (!orders[i][ii].items[iii].NMPROMOCAO && !orders[i][ii].NMPROMOCAO && orders[i][ii].items[iii].NMPRODUTO === orders[i][ii].NMPRODUTO);
										let condition4 = parseFloat(orders[i][ii].items[iii].QTPRODPEFOS) === parseFloat(orders[i][ii].items[iii].fatherQtty);
										if (!hasItems) {
											continue;
										}
										if ((condition1 || condition2 || condition3) && condition4) {
											orders[i][ii].items[iii].isVisibleItem = false;
											if (!orders[i][ii].OBSPAI) {
												orders[i][ii].OBSPAI = orders[i][ii].items[iii].TXPRODPED;
											} else if (orders[i][ii].items[iii].TXPRODPED && Array.isArray(orders[i][ii].items[iii].TXPRODPED)) {
												for (let co = 0; co < orders[i][ii].items[iii].TXPRODPED.length; co++) {
													let obsF = orders[i][ii].items[iii].TXPRODPED[co];
													if (checkObservations(obsF, orders[i][ii].OBSPAI)) {
														orders[i][ii].OBSPAI.push(obsF);
													}
												}
											} else {
												orders[i][ii].OBSPAI = '';
											}
										}
									}
								}
							}
						}
					}
				}
				return orders;
			}

			function checkObservations (obs, observations) {
				return !observations.find(e => obs.OBSERVACAO === e.OBSERVACAO);
			}
			function groupOrderByTablePosition(order) {
				var orderItems = order[widget.orderItemsField];
				return ApplicationContext.Utilities.groupArrayByProperty(orderItems, widget.orderItemPositionField);
			}
		
			function sortOrderByComboPosition(groupedItems) {
				groupedItems.forEach(group => {
					// Verificando se é necessário ordenar um grupo de itens aninhados.
					group.forEach(itemOrGroup => {
						// Se tem a propriedade 'items', trata-se de um grupo com itens aninhados.
						if (itemOrGroup.items) {
							itemOrGroup.items.sort((a, b) => parseInt(a.NRSEQITVENDAEST, 10) - parseInt(b.NRSEQITVENDAEST, 10));
						}
					});
			
					// Ordenando diretamente se a estrutura for plana (sem itens aninhados).
					// Isso se aplica se 'group' é na verdade apenas um array de itens sem 'items' aninhados.
					if (!group.some(item => item.items)) {
						group.sort((a, b) => parseInt(a.NRSEQITVENDAEST, 10) - parseInt(b.NRSEQITVENDAEST, 10));
					}

				});

				return groupedItems;
			}

			function buildLonelyItems(orders) {
				var items = [];
				if(orders) {
					items = orders.filter(function(item) {
						return !item[widget.orderItemGroupField];
					});
				}
				return items;
			}

			function buildItemsGroups(orders) {
				var groups = [], IDSITITPEDFOS = 'E', qtty= 0;
				if(orders) {
					var groupFathers = ApplicationContext.Utilities.groupArrayByProperty(orders, 'NRSEQPRODCOMKDS');
					if(groupFathers){
						Object.keys(groupFathers).forEach(function(father) {
							var fatherObj = null;
							if(father !== 'null'){
								fatherObj = {};
								var groupItems = groupFathers[father];
								groupItems.forEach(function(groupItem) {
									groupItem.items = null;
									if(groupItem.IDSITITPEDFOS == 'P'){
										fatherObj.IDSITITPEDFOS = 'P';
									}else if(IDSITITPEDFOS == 'E' && groupItem.IDSITITPEDFOS == 'F'){
										qtty++;
									}
									if(qtty == groupItems.length){
										IDSITITPEDFOS = 'F';
									}
								});
								fatherObj = Util.extend(fatherObj, groupItems[0], true);
								fatherObj.items = groupItems;
								fatherObj.isGroup = true;
								groups.push(fatherObj);
							}
						});
					}
				}
				return groups;
			}

			function hasPriorityOrders () {
				if (!widget.IDHABPRIORIDADE || widget.IDHABPRIORIDADE !== 'S') {
					return false;
				}
				if (widget.IDTIPOSETOR !== 'P') {
					return false;
				}
				if (widget.dataSource.data.length === 0) {
					return false;
				}
				return !!widget.dataSource.data.find((p) => {
					return p.IDPRODPRIORIDADE === true && ((p.IDCONDPEDPAG === 'S' && p.IDSITUVENDA === 'O') || p.IDCONDPEDPAG === 'N');

				});
			}

			function removeOrders () {
				let ordersToRemove = [];
				let localEnvironment = ApplicationContext.environment.getLocalEnvironment();
				let orders = widget.dataSource.data;
				for (let i = 0; i < orders.length; i++) {
					if (!orders[i].items || orders[i].items.length === 0) {
						// armazena para ser removido posteriormente
						ordersToRemove.push(orders[i]);
					}
					// Esconde os pedidos que ainda não podem ser liberados
					let	canExpedite = widget.IDTIPOSETOR === 'E' && !orderCanCancel(orders[i]) && teste(orders[i]);

					// esconde os itens que nn podem ser expedidos ainda caso o parametro IDEXIBCONDLIB esteja habilitado
					if(localEnvironment.IDEXIBCONDLIB !== 'S' || canExpedite || widget.kdsType === 'production' ){
						orders[i].canExibOrder = true;
					} else {
						orders[i].canExibOrder = false;
						orders[i].hide = true;
					}
				}
				// remove do array os pedidos que não tem itens
				ordersToRemove.forEach((e) => {
					let idx = orders.indexOf(e);
					orders.splice(idx, 1);
				});
				return orders;
			}

			function orderCanCancel(order) {
				return order.items.every(function(item) {
					return item[widget.orderItemStatusField] === widget.orderCancelledStatus || (order.items.every(function(item) {
						return item[widget.orderItemStatusField] === widget.orderCancelledStatus;
					}));
				});
			}

			function teste (order) {
				return order.items.every(function(item) {
					return item[widget.orderItemSelectField];
				});
			}

			function removeOrdersCompleted() {
				// impede que os pedidos que ja foram concluidos voltem para a tela
				widget.completedOrders = widget.completedOrders || {};
				Object.keys(widget.completedOrders).forEach((c) => {
					let time = (moment().unix()) - (moment(widget.completedOrders[c], ('HH:mm')).unix());
					if (time > 60) {
						Reflect.deleteProperty(widget.completedOrders, c);
					}
				});
				const cpOrders = Object.values(widget.completedOrders).length;
				for (let i = 1; i < cpOrders.length + 1; i++) {
					/* jshint -W083 */
						setTimeout(() => {
							if (widget && widget.dataSource && widget.dataSource.data && widget.dataSource.data.length > 0) {
								let filteredOrders = widget.dataSource.data.filter((order) => {
									let key = order.CDSETOR + order.NRPEDIDOFOS + order.NRITPEDIDOFOS;
									return !widget.completedOrders[key];
								});
								widget.dataSource.data = filteredOrders;
							}
					}, (i * 100)/2);
				}
			}

			widget.bindPanelShortcutKeys();

		}
	};
});

// FILE: js/controllers/AuthController.js
function AuthController(ScreenService, environment, SocketServiceKDS, ConcludeQueue, templateManager, SoundService, Utilities, SummaryItemsService) {

	var self = this;

	self.PRODUCTION_SECTOR = 'P';
	self.EXPEDITION_SECTOR = 'E';
	self.ASSEMBLY_SECTOR = 'M';

	// variaveis de controle do seletor de setores -> global para poder ser acessada pelo evento
	var fieldOpen = false;
	var focusedElement = -1;
	collection = document.getElementsByClassName("zh-list-item-label"); // pega todos os elementos da lista de setores

	this.loginOnEnter = function(widget){
		var localEnvironment = environment.getLocalEnvironment();
		if (localEnvironment) {
			self.login(localEnvironment, widget);
		} else {
			setBranchOnLoginForm(widget);
			loadSectorsField(widget);
			
			//evento de keydown para escolher o setor
			document.widget = widget; // salva o widget no document para ser acessado no evento
			document.addEventListener("keydown", handleSectorSelection, false);
		}
	};

	function handleSectorSelection(e) {
		var widget = document.widget;

		if (e.key == 'Enter' && fieldOpen == false) { // abre a lista de setores
			fieldOpen = true;
			focusedElement = -1;
			widget.getField('SELECTCDSETOR').openField();
		}
		else if (e.key == 'Enter' && fieldOpen == true) { // seleciona setor
			fieldOpen = false;
			if (collection[focusedElement]) {
				collection[focusedElement].style.fontWeight = "normal";
				// a colletion é uma lista de setores mas o elemento clicável é uma div dentro de cada item da lista, ou seja, o neto do elemento que tem a classe zh-list-item-label
				childrenDiv = collection[focusedElement].children[0]; 
				clickableDiv = childrenDiv.children[0];
				clickableDiv.click();
				focusedElement = -1; // reseta o foco
				setTimeout(() => {
					self.loginButtonOnClick(widget);
				}, 200); // entra no setor
			}
		}
 
		if (e.key == '2' || e.key == 'ArrowDown') { // navega para baixo na lista de setores
			if (focusedElement != -1) collection[focusedElement].style.fontWeight = "normal";
			focusedElement++;
			if (focusedElement > collection.length - 1) { // se for o ultimo elemento, volta para o primeiro
				focusedElement = 0;
			}
			if (collection[focusedElement]) {
				collection[focusedElement].style.fontWeight = "bold";
				collection[focusedElement].focus();
				collection[focusedElement].scrollIntoViewIfNeeded();
			}
		}

		if (e.key == '8' || e.key == 'ArrowUp') { // navega para cima na lista de setores
			if (focusedElement != -1)  collection[focusedElement].style.fontWeight = "normal";
			focusedElement--;
			if (focusedElement < 0) { // se for o primeiro elemento, volta para o ultimo
				focusedElement = collection.length - 1;
			}
			if (collection[focusedElement]) {
				collection[focusedElement].style.fontWeight = "bold";
				collection[focusedElement].focus();
				collection[focusedElement].scrollIntoViewIfNeeded();
			}
		}
	}

	function setBranchOnLoginForm(widget) {
		environment.getBranch().then(function(response) {
			if(response.error) {
				ScreenService.showMessage(response.message, 'ALERT');
			} else {
				var branch = response.branch;
				var branchLabel = branch.CDFILIAL + ' - ' + branch.NMFILIAL;
				widget.getField('LABELFILIAL').value(branchLabel);
				widget.useDarkMode = response.useDarkMode;
				handleDarkMode(response.useDarkMode, widget);
				localStorage.setItem('bindPopupKeys', response.bindPopupKeys);		
				localStorage.setItem('sortItemsByComboOrder', response.sortItemsByComboOrder);	
			}
		});
	}

	function handleDarkMode (useDarkMode, widget) {
		if (useDarkMode) {
			widget.getField('odhen_kds_horizontal').source = 'images/odhen_kds_horizontal_branco.png';

			// salva o parametro de dark mode no local storage para ser utilizado posteriormente
			localStorage.setItem('useDarkMode', true);
			
			document.querySelectorAll(".zh-application-content")[0].classList.add("dark-mode");
		}
	}

	function loadSectorsField(widget) {
		environment.getBranchSectors().then(function(response) {
			if(response.error) {
				ScreenService.showMessage(response.message, 'ALERT');
			} else {
				widget.getField('SELECTCDSETOR').dataSource.data = response.sectors;
			}
		});
	}

	this.loginButtonOnClick = function(widget) {
		document.removeEventListener("keydown", handleSectorSelection, false); // remove o evento de keydown para que não seja chamado quando entrar no setor
		if(widget.isValid()) {
			this.login(widget.currentRow);
		}
	};

	function cleanRequestsOnLogin(){
		return ScreenService.confirmMessage('Você deseja limpar todos os pedidos antes de ir para tela de pedidos?',
		'question',
			function() {
				let localEnvironment = environment.getLocalEnvironment();
				SocketServiceKDS.clearScreen(localEnvironment.CDFILIAL, localEnvironment.CDSETOR, 3);
			},
			function() {}
		);
	}

	this.login = function(row, widget){
		return environment.getRegisterControlParams(row.CDSETOR).then(function(response) {
			if (response.error) {
				ScreenService.showMessage(response.message, 'ALERT');
				environment.unsetLocalEnvironment();
				setBranchOnLoginForm(widget);
				loadSectorsField(widget);
			} else {
				var controlParams = response.productionControlParams;
                // Esse parâmetro recebido do backend precisa ser passado para a widget, que ainda não está acessível
                // neste ponto, por isso o localStorage é usado para recuperá-la no momento de incluir na widget
                localStorage.setItem('showDateTime', controlParams.showDateTime);
				localStorage.setItem('groupItemsExpedition', controlParams.groupItemsExpedition);

				controlParams = prepareControlParams(row, controlParams);
				environment.setLocalEnvironment(controlParams);
				SoundService.configureBeep();
				environment.setProjectConfig().then((data)=>{
					SocketServiceKDS.startWebSocket();
					if(data.cleanRequestsOnLogin){
						cleanRequestsOnLogin().then(()=> {
							redirectToCorrectSector(controlParams.IDTIPOSETOR);
						}, ()=>{
							redirectToCorrectSector(controlParams.IDTIPOSETOR);
						});
					} else {
						redirectAndStartSocket(controlParams.IDTIPOSETOR);
					}
				});

			}
		});
	};

	function prepareControlParams (row, controlParams) {
		return {
			'CDSETOR'          : row.CDSETOR,
			'IDTIPOSETOR'      : row.IDTIPOSETOR,
			'CDFILIAL'	       : controlParams.CDFILIAL,
			'labelContainer'   : 'KDS - Setor: '+ row.NMSETOR,
			'NMSETOR'          : row.NMSETOR,
			'hasSummary'       : row.hasSummary == undefined && row.IDTIPOSETOR == 'P' ? true : row.hasSummary,
			'IDHABROLLBKDS'    : controlParams.IDHABROLLBKDS,
			'IDHABSOMKDS'      : controlParams.IDHABSOMKDS,
			'IDHABTIMERPROD'   : controlParams.IDHABTIMERPROD,
			'IDHABSOMPROD'     : controlParams.IDHABSOMPROD,
			'IDHABCONTSOMPROD' : controlParams.IDHABCONTSOMPROD,
			'IDBLOQPRODKDS'    : controlParams.IDBLOQPRODKDS,
			'NRROLLBTIME'      : controlParams.NRROLLBTIME === null ? 0 : controlParams.NRROLLBTIME * 1000,
			'IDCONTSOMKDS'     : controlParams.IDCONTSOMKDS,
			'IDHABPREENVIO'    : controlParams.IDHABPREENVIO === 'S',
			'NRATRASOSOMPROD'  : controlParams.NRATRASOSOMPROD,
			'NRPRODATRAINISOM' : controlParams.NRPRODATRAINISOM,
			'PLAYANDCONCLUDE'  : controlParams.IDCONCDIRETO === 'S',
			'SOMBEEP'          : controlParams.SOMBEEP,
			'keyMapping'	   : controlParams.keyMapping,
			'concludeDisabled' : controlParams.IDBLOQPRODKDS === 'S',
			'IDMOSTRAORIGEM'   : controlParams.IDMOSTRAORIGEM === 'S',
			'NRTEMPOATRASOEXP' : controlParams.NRTEMPOATRASOEXP,
			'SOMBEEPEXP'       : controlParams.SOMBEEPEXP,
			'NRMULTIPULO'      : controlParams.NRMULTIPULO,
            'IDHABAGRUPPROD'   : controlParams.IDHABAGRUPPROD,
			'IDLIBCONDPG'      : controlParams.IDLIBCONDPG,
			'IDFRNONNTF'	   : controlParams.IDFRNONNTF,
			'IDCONCAUT'		   : controlParams.IDCONCAUT,
			'IDHABCANCDLV'     : controlParams.IDHABCANCDLV,
			'IDHABCANCBAL'     : controlParams.IDHABCANCBAL,
			'IDHABCANCMES'     : controlParams.IDHABCANCMES,
			'IDHABCANCCMD'     : controlParams.IDHABCANCCMD,
			'IDHABCANCTHR'     : controlParams.IDHABCANCTHR,
			'IDHABCANCTGO'     : controlParams.IDHABCANCTGO,
			'IDAUMENTARESUMO'  : controlParams.IDAUMENTARESUMO,
			'IDHABESTATISTICAS': controlParams.IDHABESTATISTICAS,
			'IDHABSCANCODE'	   : controlParams.IDHABSCANCODE,
			'IDHABLIBDIRETOEXP': controlParams.IDHABLIBDIRETOEXP,
			'IDHABESTGERAL'	   : controlParams.IDHABESTGERAL,
            'IDHABEXPFLUID'	   : controlParams.IDHABEXPFLUID,
			'IDEXIBCONDLIB'    : controlParams.IDEXIBCONDLIB,
			'CDLINHAPROD'      : controlParams.CDLINHAPROD,
			'IDLINHAPRINCIPAL' : controlParams.IDLINHAPRINCIPAL,
			'IDHABAGRUPPRODPOSMESA' : controlParams.IDHABAGRUPPRODPOSMESA,
			'IDHABPRIORIDADE'  : controlParams.IDHABPRIORIDADE || 'N',
			'IDORDPEDCON': controlParams.IDORDPEDCON,
			'SHOWCOMBOPRODUCTION': controlParams.SHOWCOMBOPRODUCTION
		};
	}

	function redirectToCorrectSector(IDTIPOSETOR) {
		switch(IDTIPOSETOR){
			case self.PRODUCTION_SECTOR:
				// ScreenService.openWindow('ordenation');
				return ScreenService.openWindow('production');
				
			case self.EXPEDITION_SECTOR:
			case self.ASSEMBLY_SECTOR:
				// ScreenService.openWindow('ordenation');
				return ScreenService.openWindow('expedition');
		}
	}

	function redirectAndStartSocket(IDTIPOSETOR) {
		redirectToCorrectSector(IDTIPOSETOR);
	}

	this.logoff = function() {
		ScreenService.toggleSideMenu();
		if(ConcludeQueue.isQueueEmpty()) {
			logoffConfirmation();
		} else {
			openLastOpenedWindow();
			ScreenService.showMessage('Ainda existem pedidos aguardando para serem liberados, favor aguardar');
		}
	};

	function logoffConfirmation() {
		clearKDSWidgetDataSource();
		SocketServiceKDS.logoffBrowser();
		SummaryItemsService.setFixedItems(null);
		environment.unsetLocalEnvironment();
		Utilities.clearAllTimeOutsAndIntervals();
		ScreenService.openWindow('initialParameters');
	}

	function openLastOpenedWindow() {
		var lastOpenedContainerName = templateManager.historyItems[templateManager.historyItems.length - 2];
		ScreenService.openWindow(lastOpenedContainerName);
	}

	function clearKDSWidgetDataSource() {
		var widget =  environment.getKDSWidget();
		widget.dataSource.data = [];
	}

}

Configuration(function(ContextRegister) {
	ContextRegister.register('AuthController', AuthController);
});

// FILE: js/controllers/DisplayController.js
function DisplayController(ScreenService, environment, AuthController, NavigatorService,
	Utilities, templateManager, SocketServiceKDS, ConcludeQueue, $timeout, $interval, SoundService,
	eventAggregator, ShortcutKeyService, SummaryItemsService, ScreenMessagesService, OrderActionService, RestEngine, ScannerDetection) {

	"use strict";

	var self = this;
	var KDS_SCREEN_PRODUCTION = 'production';
	var KDS_SCREEN_EXPEDITION = 'expedition';
	var intervalUpdateOrderTimers;
	var localEnvironment = {};
	var KDSWidget;

	this.kdsOnEnter = function (widget, type) {
		environment.setKDSWidgetOnEnvironment(widget);
		localEnvironment = environment.getLocalEnvironment();

        // Passa o parâmetro IDHABEXPFLUID para dentro da Widget, para acessá-lo facilmente de dentro do HTML
        widget.IDHABEXPFLUID = localEnvironment.IDHABEXPFLUID;
		// Passa o parâmetro IDHABPRIORIDADE para dentro da Widget, para acessá-lo facilmente de dentro do HTML
        widget.IDHABPRIORIDADE = localEnvironment.IDHABPRIORIDADE;

		self.beginToWork(widget, type);
		let label = "Liberação por Código de Barras";

		//@TODO directive
		widget.getSummaryItems = function () {
			return SummaryItemsService.getSummaryItems();
		};
		widget.orderSummaryItems = function (order) {
			return SummaryItemsService.sortOrders(order);
		};
		if (localEnvironment.IDTIPOSETOR == "E") {
			this.getButtonPosition(label).isVisible = true;
		} else {
			this.getButtonPosition(label).isVisible = false;
		}

	};

	this.getButtonPosition = function (label) {
		return templateManager.project.getMenu("kdsOptions").menus.find(button => button.label == label);
	};

	this.changeStateScanCode = function (widget) {
		localEnvironment = environment.getLocalEnvironment();
		let IDHABSCANCODE = localEnvironment.IDHABSCANCODE == 'S' ? 'N' : 'S';
		widget.IDHABSCANCODE = IDHABSCANCODE;
		localEnvironment.IDHABSCANCODE = IDHABSCANCODE;
		let CDSETOR = localEnvironment.CDSETOR;
		let scanCodeRoute = '/StateScanCode/update';

		var params = {
			requestType: "Row",
			row: {
				"IDHABSCANCODE": IDHABSCANCODE,
				"CDSETOR": CDSETOR
			}
		};

		RestEngine.post(scanCodeRoute, params).then(function (response) {
		}).catch(err => {
			return err;
		})
			.finally(() => {
				window.customKDSReload();
			});
	};

	this.showOverallStatistics = function (widget) {
		localEnvironment = environment.getLocalEnvironment();
		let CDSETOR = localEnvironment.CDSETOR;
		let CDFILIAL = localEnvironment.CDFILIAL;
		let statisticsRoute = "/ShowOverallStatistics/update";

		var params = {
			requestType: "Row",
			row: {
				"CDSETOR": CDSETOR,
				"CDFILIAL": CDFILIAL
			}
		};

		RestEngine.post(statisticsRoute, params).then(function (response) {
			console.log(response.dataset);
		}).catch(err => {
			return err;
		})
			.finally(() => {
				window.customKDSReload();
			});
	};

	this.beginToWork = function (widget, type) {
		widget.IDTIPOSETOR = localEnvironment.IDTIPOSETOR;
		widget.IDHABSCANCODE = localEnvironment.IDHABSCANCODE;
		widget.IDHABLIBDIRETOEXP = localEnvironment.IDHABLIBDIRETOEXP;
		widget.IDHABESTGERAL = localEnvironment.IDHABESTGERAL;
		widget.productionProgressBar = localEnvironment.IDHABTIMERPROD === 'S' && widget.IDTIPOSETOR === 'P';
		widget.container.label = localEnvironment.labelContainer;
		widget.hasSummary = localEnvironment.hasSummary;
		KDSWidget = widget;
		SummaryItemsService.updateSummaryItems();
		ScannerDetection.startScanning();

		// recupera o parametro useDarkMode do localStorage e salva no widget
		widget.useDarkMode = localStorage.getItem('useDarkMode') === 'true';
		handleDarkMode(widget);

		// setTimeout(function(){SocketServiceKDS.getData(localEnvironment.CDFILIAL, localEnvironment.CDSETOR);}, 3333/4);
		if (type == KDS_SCREEN_PRODUCTION) {
			this.handleNotInitializedProducts(widget);
		}

	};

	function handleDarkMode(widget) {
		if (widget.useDarkMode) {
			document.querySelectorAll(".zh-application-content")[0].classList.add("dark-mode");
		}
	}

	function findOrderByOrderKey(orderKey) {
		if (KDSWidget && KDSWidget.dataSource && KDSWidget.dataSource.data) {
			return KDSWidget.dataSource.data.filter(function (currentOrder) {
				return currentOrder.orderKey == orderKey;
			}).shift();
		}
	}

	this.onItemDelete = function (itemsToDelete) {
		if (KDSWidget) {
			Object.keys(itemsToDelete).forEach(function (orderKey) {
				if (localEnvironment.IDTIPOSETOR == 'P') {
                    if (localEnvironment.IDHABAGRUPPROD == "S"){
                        if(KDSWidget && KDSWidget.dataSource && KDSWidget.dataSource.data){
							KDSWidget.dataSource.data.forEach(function(order){
								order.items.forEach(function(orderItem, index, items){
									if(orderItem.NRPEDIDOFOS == itemsToDelete[orderKey].NRPEDIDOFOS &&
										orderItem.NRITPEDIDOFOS == itemsToDelete[orderKey].NRITPEDIDOFOS)
									{
										if(orderItem.NRITPEDIDOFOS == itemsToDelete[orderKey].NRITPEDIDOFOS){
											var now = new moment();
											if(order.LASTUPDATE == now.format()){
												now.add(1, 'second');
											}
											order.LASTUPDATE = now.format();
											items.splice(index, 1);
											// KDSWidget.updateKDSTemplate();
											order.updateAvailableAction();
											order.getOrderColor();
										}
									}
								});
							});
						}
                    } else {
                        var toRemoveOrder = findOrderByOrderKey(orderKey);
                        var toRemoveOrderIndex = KDSWidget.dataSource.data.indexOf(toRemoveOrder);
                        if (~toRemoveOrderIndex) {
                            toRemoveOrder.clearBeeps();
                            //SoundService.clearOrderBeeps(toRemoveOrder);
                            if (toRemoveOrder.timerInterval) {
                                $interval.cancel(toRemoveOrder.timerInterval);
                                delete toRemoveOrder.timerInterval;
                            }
                            NavigatorService.setOrderAsSelected(toRemoveOrderIndex, 'move');
                            KDSWidget.dataSource.data.splice(toRemoveOrderIndex, 1);
                            SummaryItemsService.updateSummaryItems();
                        }
                    }
				} else {
					if (KDSWidget && KDSWidget.dataSource && KDSWidget.dataSource.data) {
						KDSWidget.dataSource.data.forEach(function (order, orderIndex, orders) {
							if (order.NRPEDIDOFOS == itemsToDelete[orderKey].NRPEDIDOFOS) {
								order.items.forEach(function (item, index, items) {
									if (item.groupedItens && item.groupedItens.length > 0) {
										let gItemsToRemove = [];
										item.groupedItens.forEach((gItem) => {
											if (gItem.NRITPEDIDOFOS == itemsToDelete[orderKey].NRITPEDIDOFOS) {
												gItemsToRemove.push(gItem);
											}
										});
										if (gItemsToRemove) {
											gItemsToRemove.forEach((e) => {
												let idx = item.groupedItens.indexOf(e);
												item.groupedItens.splice(idx, 1);
											});
											let items = item.groupedItens;
											order.items.splice(index, 1);
											order.items = items;
										}
									} 
									if (item.NRITPEDIDOFOS == itemsToDelete[orderKey].NRITPEDIDOFOS && !item.groupedItens) {
										items.splice(index, 1);
									}
								});
								let now = new moment();
								if (order.LASTUPDATE == now.format()) {
									now.add(1, 'second');
								}
								// KDSWidget.updateKDSTemplate();
								order.updateAvailableAction();
								order.getOrderColor();
								order.LASTUPDATE = now.format();
								order.items = KDSWidget.checkAndGroupItems(order.items);
							}
						});
					}

				}
			});
			KDSWidget.updateKDSTemplate();
		}
	};

	function findOrderInScreen(orderToCompare) {
		var order = KDSWidget.dataSource.data.filter(function (currentOrder) {
			return checkOrderExistence(currentOrder, orderToCompare.orderKey);
		}).shift();
		return KDSWidget.dataSource.data.indexOf(order);
	}

	//isso serve para no caso do primeiro item de um pedido for trocado, nao duplicar.
	function checkOrderExistence(order, orderKey) {
		if (order.items) {
			return order.items.some(function (item) {
				return item.orderKey == orderKey;
			});
		}
	}

	function checkOrderToAdd(order, senhaped, nrmesa) {
		if(order.items){
			return order.items.some(function(item) {
				return (item.CDSENHAPED == senhaped && item.NRMESA == nrmesa);
			});
		}
	}

	function replaceZoomItem(order) {
		if (KDSWidget.zoomItem) {
			if (checkOrderExistence(KDSWidget.zoomItem, order.orderKey)) {
				KDSWidget.zoomItem = Util.extend(KDSWidget.zoomItem, order, true);
			}
		}
	}
	
	function findOrderGroup(orderToCompare){
		var order = KDSWidget.dataSource.data.filter(function(currentOrder){
			return checkOrderToAdd(currentOrder, orderToCompare.CDSENHAPED, orderToCompare.NRMESA);
		}).shift();
		return KDSWidget.dataSource.data.indexOf(order);
	}

	this.onItemProducaoChange = function (requests) {
		if (KDSWidget) {
			var oldLength = KDSWidget.dataSource.data.length;
			Object.keys(requests).forEach(function (key) {
				if (!ConcludeQueue.checkItemExistence(key)) {
					var indexToReplace = findOrderInScreen(requests[key]);
                    var indexToAgroup = null;

                    if (localEnvironment.IDHABAGRUPPROD == "S") {
                        indexToAgroup = findOrderGroup(requests[key]);
                    }

					if (indexToReplace) {
						replaceZoomItem(requests[key]);
					}
					//isso olha se já esta na hora de mostrar o item que mais demora desse pedido.
					//se não estiver e o pedido não estiver liberado ele vai pro waiting items.

					if ((requests[key].IDLIBERADO == 'N' && localEnvironment.IDHABPREENVIO && requests[key].waiting)) {
						requests[key].timer = 'wait';
						if (moment(requests[key].DTHREXIBICAOPROD).subtract(requests[key].NRMAIORTEMPO, 'seconds').isBefore(requests[key].NOW)) {
							requests[key].IDLIBERADO = 'S';
						}
					}
					if (requests[key].IDLIBERADO == 'S' || requests[key].waiting) {
						if (~indexToReplace && indexToReplace != null) {
							requests[key].isNewOrder = false;

                            if (requests[key].IDSITITPEDFOS == "C" && localEnvironment.IDHABAGRUPPROD == "S") {
                                //Para agrupar um item cancelado de acordo com a resposta do websocket
                                KDSWidget.dataSource.data[indexToReplace].items.forEach(function(item, index, array) {
									// Se o pedido tiver itens cancelados procura o item cancelado e marca como cancelado no front
									var itemToReplace = requests[key].items.find(function(newItem) {
										return newItem.NRPEDIDOFOS == item.NRPEDIDOFOS && newItem.NRITPEDIDOFOS == item.NRITPEDIDOFOS;
									});
									if (itemToReplace.IDSITITPEDFOS == "C") {
										item.IDSITITPEDFOS = "C";
									}
                                    if (requests[key].NRPEDIDOFOS == item.NRPEDIDOFOS && requests[key].NRITPEDIDOFOS == item.NRITPEDIDOFOS) {
                                        item.IDSITITPEDFOS = "C";
                                    }
                                });
								KDSWidget.dataSource.data[indexToReplace].updateAvailableAction();
								KDSWidget.dataSource.data[indexToReplace].getOrderColor();
                            } else if (localEnvironment.IDHABAGRUPPROD == "S" && ~indexToAgroup) {
                                //Para agrupar um item que estava em espera para iniciar de acordo com a resposta do websocket
                                KDSWidget.dataSource.data[indexToReplace].items.forEach(function(item, index, array) {
                                    if (requests[key].NRPEDIDOFOS == item.NRPEDIDOFOS) {
                                        requests[key].items.forEach(function(itemR, indexR, arrayR) {
                                            if (item.NRITPEDIDOFOS == itemR.NRITPEDIDOFOS) {
                                                if(!Array.isArray(itemR.TXPRODPED)){
                                                    if (itemR.TXPRODPED != ''){
                                                        itemR.TXPRODPED = JSON.parse(itemR.TXPRODPED);
                                                    }
                                                }
                                                KDSWidget.dataSource.data[indexToReplace].items[index] = Util.extend(KDSWidget.dataSource.data[indexToReplace].items[index], itemR, true);
                                            }
                                        });
                                    }
                                });
                                KDSWidget.dataSource.data[indexToReplace].updateAvailableAction();
                                KDSWidget.dataSource.data[indexToReplace].getOrderColor();
                                KDSWidget.dataSource.data[indexToAgroup].LASTUPDATE = new moment().format();
                                // KDSWidget.updateKDSTemplate();
                            } else {
                                KDSWidget.dataSource.data[indexToReplace] = Util.extend(KDSWidget.dataSource.data[indexToReplace], requests[key], true);
                            }	
						} else if (~indexToAgroup && indexToAgroup != null) {
                            requests[key].isNewOrder = false;
                            if (KDSWidget.dataSource.data[indexToAgroup].NRATRAPRODITPE == requests[key].NRATRAPRODITPE) {
                                KDSWidget.dataSource.data[indexToAgroup].items = KDSWidget.dataSource.data[indexToAgroup].items.concat(requests[key].items);
                                var now = new moment();
                                KDSWidget.dataSource.data[indexToAgroup].LASTUPDATE = now.format();
                                // KDSWidget.updateKDSTemplate();
                            } else {
                                requests[key].isNewOrder = true;
                                KDSWidget.dataSource.data.push(requests[key]);
                            }
                        } else if (localEnvironment.IDHABPREENVIO || requests[key].IDLIBERADO == 'S') {
							indexToReplace = KDSWidget.dataSource.data.length;
							requests[key].isNewOrder = true;
							KDSWidget.dataSource.data.push(requests[key]);
						} else {

							indexToReplace = KDSWidget.dataSource.data.length;
							requests[key].isNewOrder = true;

						}
						// KDSWidget.updateKDSTemplate();
						if (requests[key].IDSITITPEDFOS == "C" && localEnvironment.IDHABAGRUPPROD == "N") {
							KDSWidget.dataSource.data.forEach((pedido, indexPedido) => {
								if(pedido.NRPEDIDOFOS == requests[key].NRPEDIDOFOS && pedido.NRITPEDIDOFOS == requests[key].NRITPEDIDOFOS) {
									KDSWidget.dataSource.data[indexPedido] = requests[key];
								}
							});
							let confirmCancel = self.verifyCancelOrigem(localEnvironment, requests[key]);
							if (confirmCancel == "N") {
								OrderActionService.orderOnClick(KDSWidget.dataSource.data[indexToReplace]);
							}
						} else if (requests[key].IDSITITPEDFOS == "C") {
							let confirmCancel = self.verifyCancelOrigem(localEnvironment, requests[key]);
							if (confirmCancel == "N") {
								OrderActionService.orderOnClick(KDSWidget.dataSource.data[indexToReplace]);
							}
						}
						// KDSWidget.updateKDSTemplate();
					}
				}
			});

            if(localEnvironment.IDHABEXPFLUID == 'S'){	
                let tempData = KDSWidget.dataSource.data;
    
                tempData.forEach( order  => {
                    //Verificar através do NRSEQPRODCOMKDS se estou no mesmo combo
                    let currentProm = '';
                    order.items.forEach( (item, idx) => {
    
    
                        if(!item.processed){
    
                            item.firstPromo = true;
                            item.lastPromo = false;
                            item.isPromo = false;
    
                            //Algo mudou
                            if( currentProm != '' && currentProm != item.NRSEQPRODCOMKDS){
                                if(currentProm != ''){
                                    //Insere DivisoriaInicial
                                    item.lastPromo = true;
                                }
                            }
    
                            if (item.NRSEQPRODCOMKDS){
                                item.isPromo = true;
                            }
    
                            //Atualiza currentProm
                            currentProm = item.NRSEQPRODCOMKDS;
    
                            item.DSCHAVEPEDPAI = '';
                            item.NRPRATO = item.NRSEQPRODCOMKDS;
                            item.NRSEQPRODCOMKDS = '';
                            item.NMPROMOCAO = '';
                            item.processed = true;
                        }
                    });
                });
            }
			if (localEnvironment.IDHABAGRUPPRODPOSMESA === 'S') {
				KDSWidget.orderDataSourceByPositionTable();
				changeColor();
			}else if (localEnvironment.IDORDPEDCON === 'S') {
				KDSWidget.orderDataSourceByActionLabel();
			} else {
				KDSWidget.orderDataSouce();
			}
			SummaryItemsService.updateSummaryItems();
			if (oldLength === 0 && KDSWidget.dataSource.data[0]) {
				NavigatorService.setFirstOrderAsSelected();
			}
			KDSWidget.updateKDSTemplate();
		}
	};

	this.verifyCancelOrigem = function (env, order) {
		var origem = order.IDORIGEMVENDA.slice(0, 3);
		var confirCancel = '';
		switch (origem) {
			case 'DLV':
				confirCancel = env.IDHABCANCDLV;
				break;
			case 'THR':
				confirCancel = env.IDHABCANCTHR;
				break;
			case 'TGO':
				confirCancel = env.IDHABCANCTGO;
				break;
			case 'BAL':
				confirCancel = env.IDHABCANCBAL;
				break;
			case 'CMD':
				confirCancel = env.IDHABCANCCMD;
				break;
			case 'MES':
				confirCancel = env.IDHABCANCMES;
				break;
			default:
				confirCancel = env.IDHABCANCBAL;
				break;
		}

		return confirCancel;
	};


	this.onItemProducaoConclude = function (requestkey) {
		let length = requestkey.length;
		let parseKey = requestkey.substring(length - 2, length);
		let toDeleteOrder = KDSWidget.dataSource.data.filter(function (order) {
			if (order.orderKey.substring(0, 15) == requestkey.substring(0, 15) && parseKey == '01') {
				return order.orderKey;
			}
			return order.orderKey == requestkey;
		});
		/* let toDeleteOrder = KDSWidget.dataSource.data.filter(function (order) {
			if (order.orderKey.substring(0, 15) == requestkey.substring(0, 15)) {
				return order;
			}
			return order.orderKey == requestkey;
		}); */
		/* if (toDeleteOrder) {
			let index = KDSWidget.dataSource.data.indexOf(toDeleteOrder);
			if (~index) {
				KDSWidget.dataSource.data.splice(index, 1);
				SummaryItemsService.updateSummaryItems();
			}
		} */
		if (toDeleteOrder.length && toDeleteOrder.length > 0) {
			for (let i = 0; i < toDeleteOrder.length; i++) {
				let index = KDSWidget.dataSource.data.indexOf(toDeleteOrder[i]);
				if (~index) {
					KDSWidget.dataSource.data.splice(index, 1);
				}
			}
		}
		SummaryItemsService.updateSummaryItems();
	};

	this.reloadWidget = function (widget) {
		widget.dataSource.data = [];
		widget.getWaitingItems = self.getWaitingItems;
	};

	this.getWaitingItems = function (items, widget, setWaitingItems) {
		var keys = Object.keys(items);
		if (keys.length > 0) {
			for (var key in items) {
				var waitingItems = [];
				let waitingItem = {};
				var productName = waitingItems[items[key][widget.orderItemsField][0][widget.orderItemDescField]];
				if (productName) {
					waitingItems[productName[widget.orderItemDescField]][widget.orderItemQttyField] += items[key][widget.orderItemsField][0][widget.orderItemQttyField];
				} else {
					waitingItem[widget.orderItemQttyField] = items[key][widget.orderItemsField][0][widget.orderItemQttyField];
					waitingItem[widget.orderItemDescField] = items[key][widget.orderItemsField][0][widget.orderItemDescField];
					waitingItems[waitingItem[widget.orderItemDescField]] = waitingItem;
				}
				if (key == keys[keys.length - 1]) {
					setWaitingItems(waitingItems);
				}
			}
		} else {
			KDSWidget.tratedWaitingItems = [];
		}
	};

	this.setWaitingItems = function (waitingItems) {
		var returnArray = [];
		for (var index in waitingItems) {
			returnArray.push(waitingItems[index]);
			KDSWidget.tratedWaitingItems = returnArray;
		}
	};

	this.clearRequestsScreen = function () {
		ScreenService.confirmMessage('Você tem certeza que deseja limpar todos os pedidos?',
			'question',
			function () {
				var CDFILIAL = localEnvironment.CDFILIAL;
				var CDSETOR = localEnvironment.CDSETOR;

				SocketServiceKDS.clearScreen(CDFILIAL, CDSETOR);
				ScreenService.successNotification('Operação concluida com sucesso!');
				ScreenService.toggleSideMenu();
				window.customKDSReload();
			},
			function () { }
		);
	};

	this.productionLineDeactivated = function () {
		ScreenService.confirmMessage('Deseja desativar esta linha de produção?',
			'question',
			function () {
				localEnvironment = environment.getLocalEnvironment();
				let CDSETOR = localEnvironment.CDSETOR;
				let lineProductionRoute = '/StatusLineProduction/update';

				var params = {
					requestType: "Row",
					row: {
						"CDSETOR": CDSETOR
					}
				};

				RestEngine.post(lineProductionRoute, params).then(function (response) {
				}).catch(err => {
					return err;
				})
					.finally(() => {
						window.customKDSReload();
					});
			},
			function () { }
		);
	};

	this.toggleNotInitializedProducts = function () {
		if (localEnvironment.IDTIPOSETOR === 'P') {
			localEnvironment.hasSummary = !localEnvironment.hasSummary;
			environment.setLocalEnvironment(localEnvironment);
			var widget = templateManager.container.getActiveWidget();
			this.handleNotInitializedProducts(widget);
			ScreenService.toggleSideMenu();
		} else {
			ScreenService.toggleSideMenu();
			ScreenService.alertNotification('Opção não válida para setor de expedição');
		}
	};

	this.toggleWaitingProducts = function () {
		if (localEnvironment.IDTIPOSETOR === 'P') {
			localEnvironment.hasWaiting = !localEnvironment.hasWaiting;
			environment.setLocalEnvironment(localEnvironment);
			var widget = templateManager.container.getActiveWidget();
			this.handleWaitingProducts(widget);
			ScreenService.toggleSideMenu();
		} else {
			ScreenService.toggleSideMenu();
			ScreenService.alertNotification('Opção não válida para setor de expedição');
		}
	};

	this.handleWaitingProducts = function (widget) {
		widget.hasWaiting = localEnvironment.hasWaiting;
		templateManager.updateTemplate();
	};

	this.handleNotInitializedProducts = function (widget) {
		widget.hasSummary = localEnvironment.hasSummary;
		templateManager.updateTemplate();
	};

	this.freezeUpdates = function (widget) {
		if (!KDSWidget.frozen) {
			ScreenService.confirmMessage('Você tem certeza que deseja pausar o funcionamento do KDS? (não será possivel modificar os pedidos presentes e pedidos feitos enquanto pausado serão ignorados)',
				'question',
				function () {
					SocketServiceKDS.freezeUpdates();
					ScreenService.successNotification('Operação concluida com sucesso!');
					ScreenService.toggleSideMenu();
				},
				function () { }
			);
		} else {
			SocketServiceKDS.unfreezeUpdates();
			ScreenService.successNotification('Operação concluida com sucesso!');
			ScreenService.toggleSideMenu();
		}
	};

	this.onFreezeUpdates = function (widget) {
		KDSWidget.frozen = true;
		self.reloadWidget(KDSWidget);
		ScreenMessagesService.clearMessages();
	};
	this.onUnfreezeUpdates = function (widget) {
		KDSWidget.frozen = false;
		setTimeout(function () { SocketServiceKDS.getData(environmentParams.CDFILIAL, environmentParams.CDSETOR, environmentParams.IDTIPOSETOR); }, 1000);
	};

	// statistics
	this.onStatisticsUpdate = function (data) {
		if (KDSWidget) {
			var medio = moment({
				'seconds': data.TEMPOMEDIO
			});
			var inicio = moment({
				'seconds': data.TEMPOINICIO
			});
			var atraso = moment({
				'seconds': data.TEMPOATRASO
			});
			var mediasetordia = moment({
				'seconds': parseFloat(data.MEDIAKDSDIA)
			});
			var mediasetorhora = moment({
				'seconds': parseFloat(data.MEDIAKDSHR)
			});
			var mediasetorhoraant = moment({
				'seconds': parseFloat(data.MEDIAKDSULTHR)
			});
			var mediaexpedia = moment({
				'seconds': parseFloat(data.MEDIADIA)
			});
			var mediaexpehora = moment({
				'seconds': parseFloat(data.MEDIAHORA)
			});

			var mediaexpehoraant = moment({
				'seconds': parseFloat(data.MEDIAHORAANT)
			});

			medio._isValid = true;
			KDSWidget.TEMPOMEDIO = medio.format('HH:mm:ss');
			inicio._isValid = true;
			KDSWidget.TEMPOINICIO = inicio.format('HH:mm:ss');
			atraso._isValid = true;
			KDSWidget.TEMPOATRASO = atraso.format('HH:mm:ss');
			mediasetordia._isValid = true;
			KDSWidget.MEDIAKDSDIA = mediasetordia.format('HH:mm:ss');
			mediasetorhora._isValid = true;
			KDSWidget.MEDIAKDSHR = mediasetorhora.format('HH:mm:ss');
			mediasetorhoraant._isValid = true;
			KDSWidget.MEDIAKDSULTHR = mediasetorhoraant.format('HH:mm:ss');
			mediaexpedia._isValid = true;
			KDSWidget.MEDIADIA = mediaexpedia.format('HH:mm:ss');
			mediaexpehora._isValid = true;
			KDSWidget.MEDIAHORA = mediaexpehora.format('HH:mm:ss');
			mediaexpehoraant._isValid = true;
			KDSWidget.MEDIAHORAANT = mediaexpehoraant.format('HH:mm:ss');

			KDSWidget.QTDIA = Math.trunc(data.QTDIA);
			KDSWidget.QTHORA = Math.trunc(data.QTHORA);
			KDSWidget.QTHORAANT = Math.trunc(data.QTHORAANT);
			KDSWidget.SUMQTCONCLUIDO = Math.trunc(data.SUMQTCONCLUIDO);
			KDSWidget.SUMQTCONCLUIDOHR = Math.trunc(data.SUMQTCONCLUIDOHR);
			KDSWidget.SUMQTCONCLUIDOULTHR = Math.trunc(data.SUMQTCONCLUIDOULTHR);

			var porc = parseFloat(data.PORCENTAGEM).toFixed(2).toString().replace('.', ',');
			KDSWidget.PORCENTAGEM = porc + "%";
		}
	};

	function changeColor(){
		let controle = false;
		var nrpedidofosPrimeiro = KDSWidget.dataSource.data[0].NRPEDIDOFOS;

		KDSWidget.dataSource.data.forEach((individual, index) => {
			var nrpedidofos  = individual.NRPEDIDOFOS;
			if(nrpedidofos == nrpedidofosPrimeiro){
				KDSWidget.dataSource.data[index].MUDACOR = controle;
			}else{
				nrpedidofosPrimeiro = individual.NRPEDIDOFOS;
				controle = !controle;
				KDSWidget.dataSource.data[index].MUDACOR = controle;
			}
		});
	}

	function initialize() {
		eventAggregator.subscribe('onProductionResponse', self.onItemProducaoChange);
		eventAggregator.subscribe('onProductionConclude', self.onItemProducaoConclude);
		eventAggregator.subscribe('onStatisticsUpdate', self.onStatisticsUpdate);
		eventAggregator.subscribe('onDeleteOrders', self.onItemDelete);
		eventAggregator.subscribe('onPong', SocketServiceKDS.onPong);
		eventAggregator.subscribe('onNewMessage', ScreenMessagesService.onNewMessage);
		eventAggregator.subscribe('onDeleteMessage', ScreenMessagesService.onDeleteMessage);
		eventAggregator.subscribe('onFreezeUpdates', self.onFreezeUpdates);
		eventAggregator.subscribe('onUnfreezeUpdates', self.onUnfreezeUpdates);
	}

	initialize();
}

Configuration(function (ContextRegister, eventEngine) {
	//register the event to be used with KDS
	eventEngine.registerEvent('KDSOrderClick', 'click');
	eventEngine.registerEvent('KDSOrderSelect', 'onSelect');
	eventEngine.registerEvent('KDSOrderUnselect', 'onUnselect');
	eventEngine.registerEvent('KDSOrderRollback', 'rollback');
	ContextRegister.register('DisplayController', DisplayController);
});

// FILE: js/controllers/ResumePanelController.js
function ResumePanelController(ScreenService, templateManager, environment, SummaryItemsService) {

	var self = this;

	this.toggleResumePanel = function() {
		var localEnvironment = environment.getLocalEnvironment();
		var widget = templateManager.container.getWidget('KDS');
		if(widget && localEnvironment.IDTIPOSETOR == 'P') {
			let isResumeOpened = !widget.hasSummary;
			widget.hasSummary = isResumeOpened;
			localEnvironment.hasSummary = isResumeOpened;
			environment.setLocalEnvironment(localEnvironment);
			SummaryItemsService.updateSummaryItems();
		} else {
			ScreenService.alertNotification('Opção válida somente para setores de produção!');
		}
		ScreenService.toggleSideMenu();
	};

	// $scope.isResumePanelOpen = false;

	// $scope.toggleResumePanel = function() {
	// 	var localEnvironment = environment.getLocalEnvironment();
	// 	if(localEnvironment.IDTIPOSETOR == 'P') {
	// 		$scope.isResumePanelOpen = !$scope.isResumePanelOpen;			
	// 	} else {
	// 		$rootScope.$broadcast('showNotification', {
	// 			message: "Opção válida somente para setores de produção!", 
	// 			time: 3000, 
	// 			type: "success"
	// 		});
	// 	}
	// };

	// $scope.$on('toggleResumePanel', $scope.toggleResumePanel);

	
}

Configuration(function(ContextRegister) {
	ContextRegister.register('ResumePanelController', ResumePanelController);
});