function ScannerDetection(ScreenService, RestEngine, environment, ClientNotificationService, OrderActionService, ApplicationContext) {

    var self = this;

    this.options = {
      onComplete: false, // Callback after detection of a successful scanning
      onError: false, // Callback after detection of a unsuccessful scanning
      onReceive: false, // Callback after receive a char
      timeBeforeScanTest: 1500, // Wait duration (ms) after keypress event to check if scanning is finished
      avgTimeByChar: 30, // Average time (ms) between 2 chars. Used to do difference between keyboard typing and scanning
      minLength: 6, // Minimum length for a scanning
      endChar: [9, 13], // Chars to remove and means end of scanning
      stopPropagation: false, // Stop immediate propagation on keypress event
      preventDefault: false // Prevent default action on keypress event
    };
    this.firstCharTime = 0;
    this.lastCharTime = 0;
    this.stringWriting = '';
    this.callIsScanner = false;
    this.testTimer = false;

    this.keypress = function (e) {
        self.options.onComplete = function(receive) {         
            if(receive){
                var widget = environment.getKDSWidget();
                
                var CDFILIAL;
                if(receive.length > 18 && receive.indexOf("http") == -1){
                    CDFILIAL = receive.substr(1, 4);
                    var NRVENDAREST = receive.substr(5, 10);
                    var NRCOMANDA = receive.substr(15);
                    widget.dataSource.data.forEach(function(order){
                        if(order.CDFILIAL == CDFILIAL &&
                           order.NRVENDAREST == NRVENDAREST &&
                           order.NRCOMANDA == NRCOMANDA)
                        {
                            // IDHABLIBDIRETOEXP = Permite liberacao direta do pedido. S = Sim - N = Nao
                            if (widget.IDHABLIBDIRETOEXP == 'S' || order.status == 'expedite') {
                                order.dot = true;
                                ClientNotificationService.notifyClient(order);
                            }
                        }
                    });
                    ApplicationContext.ShortcutKeyService.bindMappedKeys();
                }else if(receive.indexOf("http") != -1 ){  /* identificar quando é URL */

                    let QrCodeRoute = '/order/get';

                    var params = {
                        requestType: "Row",
                        row: {
                            "DSQRCODENFCE": receive,
                        }
                    };
                    
                    var orderToExped = {};

                    RestEngine.post(QrCodeRoute, params).then(function(response) {	
                        orderToExped = response.dataset.qrCode.qrCode[0];
                        widget.dataSource.data.forEach(function(order){
                            if(order.CDFILIAL == orderToExped.CDFILIAL && (order.NRCOMANDA == orderToExped.NRCOMANDAVND || order.NRSEQVENDA == orderToExped.NRSEQVENDA)){
                                        
                                if (order.isCallClientAvailable() && order.status == 'expedite') {
                                    order.dot = true;
                                    ClientNotificationService.notifyClient(order);
                                }
                                if(widget.IDHABSCANCODE == 'S'){
                                    if(order.availableAction == null){
                                        order.isProducted = true;
                                        order.status = 'expedite';
                                        order.updateAvailableAction();
                                    }
                                    OrderActionService.orderOnClick(order);
                                }
                            }
                        });
                    })
                    .catch(err =>{
                        console.log(err);
                        ApplicationContext.ShortcutKeyService.bindMappedKeys();
                    });
                    ApplicationContext.ShortcutKeyService.bindMappedKeys();
                } else{
                    var CDCAIXA = receive.substr(0, 3);
                    CDFILIAL = receive.substr(3, 4);
                    var NRSEQVENDA = receive.substr(7);
                    widget.dataSource.data.forEach(function(order){
                        if(order.CDFILIAL == CDFILIAL &&
                           order.CDCAIXA == CDCAIXA &&
                           order.NRSEQVENDA == NRSEQVENDA)
                        {
                            if (order.isCallClientAvailable() && order.status == 'expedite') {
                                order.dot = true;
                                ClientNotificationService.notifyClient(order);
                            }
                            if(widget.IDHABSCANCODE == 'S'){
                                OrderActionService.orderOnClick(order);
                            }
                        }
                    });
                }
            }
        };
        self.options.onError = function(){};
        self.options.onReceive = function(){};
        
        if (self.options.stopPropagation) e.stopImmediatePropagation();
        if (self.options.preventDefault) e.preventDefault();

        if (self.firstCharTime && self.options.endChar.indexOf(e.which) !== -1) {
          e.preventDefault();
          e.stopImmediatePropagation();
          self.callIsScanner = true;
        } else {
          self.stringWriting += String.fromCharCode(e.which);
          self.callIsScanner = false;
        }

        if (!self.firstCharTime) {
          self.firstCharTime = e.timeStamp;
        }
        self.lastCharTime = e.timeStamp;

        if (self.testTimer) clearTimeout(self.testTimer);
        if (self.callIsScanner) {
          self.scannerDetectionTest();
          self.testTimer = false;
        } else {
          self.testTimer = setTimeout(() => {
            self.scannerDetectionTest();
          }, self.options.timeBeforeScanTest);
        }

        if (self.options.onReceive) self.options.onReceive.call(self, e);
        self.trigger('scannerDetectionReceive', { evt: e });
    };

    this.trigger = function (options) {};

    this.scannerDetectionTest = function (s) {
        // If string is given, test it
        if (s) {
          self.firstCharTime = self.lastCharTime = 0;
          self.stringWriting = s;
        }
        // If all condition are good (length, time...), call the callback and re-initialize the plugin for next scanning
        // Else, just re-initialize
        if (
          self.stringWriting.length >= self.options.minLength &&
          self.lastCharTime - self.firstCharTime <
            self.stringWriting.length * self.options.avgTimeByChar
        ) {
            if (self.options.onComplete) {
                self.options.onComplete.call(self, self.stringWriting);
            }
            self.initScannerDetection();
            return true;
        } else {
            if (self.options.onError) {
                self.options.onError.call(self, self.stringWriting);
            }
            self.initScannerDetection();
            return false;
        }
    };

    this.initScannerDetection = function () {
        self.firstCharTime = 0;
        self.stringWriting = '';
    };

    this.stopScanning = function() {
        document.onkeypress = null;
    };

    this.startScanning = function(){
        document.onkeypress = self.keypress.bind(self);
    };


}

Configuration(function (ContextRegister) {
    ContextRegister.register('ScannerDetection', ScannerDetection);
});